@@ -1,5 +1,5 @@
 # ------------------
-# enblend 3.2   
+# enblend 4.0   
 # ------------------
 # $Id: enblend3.sh 1908 2007-02-05 14:59:45Z ippei $
 # Copyright (c) 2007, Ippei Ukai
@@ -19,10 +19,40 @@
 
 # -------------------------------
 # 20091206.0 sg Script tested and used to build 2009.4.0-RC3
+# 20091209.0 sg Script enhanced to build Enblemd-Enfuse 4.0
+# 20091210.0 hvdw Removed code that downgraded optimization from -O3 to -O2
+# 20091223.0 sg Added argument to configure to locate missing TTF
+#               Building enblend documentation requires tex. Check if possible.
+# 20100624.0 hvdw More robust error checking on compilation
 # -------------------------------
 
 # init
 
+fail()
+{
+        echo "** Failed at $1 **"
+        exit 1
+}
+
+
+# Fancy doc builds on Enblend 3.2 are doomed to failure, so don't even try...
+AC_INIT=$(grep AC_INIT Configure.in)
+TEX=$(which tex)
+
+# If NOT 3.2 and if tex is installed, and if FreeSans.ttf is in the right place...
+if [ -z "$(echo $AC_INIT|grep 3.2,)" ] && \
+	 [ -n "$TEX" ]  && [ -f "/Users/$LOGNAME/Library/Fonts/FreeSans.ttf" ]; then 
+	buildDOC="yes"
+	extraConfig="--with-ttf-path=/Users/$LOGNAME/Library/Fonts --enable-split-doc=no"
+	extraBuild="ps pdf xhtml"
+	extraInstall="install-ps install-pdf install-xhtml"
+else
+  buildDOC="no"
+  extraConfig=""
+  extraBuild=""
+  extraInstall=""
+fi 
+
 let NUMARCH="0"
 
 for i in $ARCHS
@@ -76,17 +106,35 @@ do
    CXX=$x64CXX
  fi
 
+# To build documentation, you will need to install the following (port) packages:
+#   freefont-ttf
+#   gnuplot
+#   ghostscript
+#   texi2html
+#   transfig
+#   tidy
+#  *teTeX
+# This script presumes you have installed the fonts in ~/Library/Fonts. 
+# See <http://trac.macports.org/ticket/16938> for how to do this.
+# (Port installs the fonts here: /opt/local/share/fonts/freefont-ttf/)
+# The port version of teTeX did not install cleanly for me. Instead, I downloaded a pre-built distro
+# called MacTeX <http://www.tug.org/mactex/2009/>. After installing, you will need to add this
+# directory to your PATH, as shown on the next line: 
+# export PATH=/usr/local/texlive/2009/bin/universal-darwin:$PATH
+# To make the change permanent, edit ~/.profile.
+
  env \
    CC=$CC CXX=$CXX \
-   CFLAGS="-isysroot $MACSDKDIR -arch $ARCH $ARCHARGs $OTHERARGs -dead_strip" \
-   CXXFLAGS="-isysroot $MACSDKDIR -arch $ARCH $ARCHARGs $OTHERARGs -dead_strip" \
+   CFLAGS="-isysroot $MACSDKDIR -I$REPOSITORYDIR/include -arch $ARCH $ARCHARGs $OTHERARGs -dead_strip" \
+   CXXFLAGS="-isysroot $MACSDKDIR -I$REPOSITORYDIR/include -arch $ARCH $ARCHARGs $OTHERARGs -dead_strip" \
    CPPFLAGS="-I$REPOSITORYDIR/include -I$REPOSITORYDIR/include/OpenEXR -I/usr/include" \
+   LIBS="-lGLEW -framework GLUT -lobjc -framework OpenGL -framework AGL" \
    LDFLAGS="-L$REPOSITORYDIR/lib -L/usr/lib -mmacosx-version-min=$OSVERSION -dead_strip" \
    NEXT_ROOT="$MACSDKDIR" \
    PKG_CONFIG_PATH="$REPOSITORYDIR/lib/pkgconfig" \
    ./configure --prefix="$REPOSITORYDIR" --disable-dependency-tracking --enable-image-cache=yes \
      --host="$TARGET" --exec-prefix=$REPOSITORYDIR/arch/$ARCH --with-apple-opengl-framework \
-   ;
+     --with-glew $extraConfig  || fail "configure step for $ARCH";
 
  # hack; AC_FUNC_MALLOC sucks!!
 
@@ -95,13 +143,17 @@ do
      -e 's/rpl_malloc/malloc/' \
      "./config.h-copy" > "./config.h";
 
- # well, I prefer -O2
- mv src/Makefile src/Makefile.copy
- sed 's/-O3/-O2/g' src/Makefile.copy > src/Makefile
+
+ # Default to standard -O3 optimization as this improves performance
+ # and shrinks the binary
+ # If you prefer -O2, change -O3 to -O2 in the 3rd line (containing the sed command).
+ [ -f src/Makefile.bak ] && rm src/Makefile.bak
+ mv src/Makefile src/Makefile.bak
+ sed -e "s/-O[0-9]/-O3/g" "src/Makefile.bak" > src/Makefile
 
  make clean;
- make all;
- make install;
+ make all $extraBuild || fail "failed at make step of $ARCH";
+ make install $extraInstall || fail "make install step of $ARCH";
  
 done
 
@@ -115,7 +167,7 @@ do
    if [ -f $REPOSITORYDIR/arch/$ARCHS/$program ] ; then
 		 echo "Moving arch/$ARCHS/$program to $program"
   	 mv "$REPOSITORYDIR/arch/$ARCHS/$program" "$REPOSITORYDIR/$program";
-  	 strip "$REPOSITORYDIR/$program";
+  	 strip -x "$REPOSITORYDIR/$program";
   	 continue
 	 else
 		 echo "Program arch/$ARCHS/$program not found. Aborting build";
@@ -137,11 +189,6 @@ do
  done
 
  lipo $LIPOARGs -create -output "$REPOSITORYDIR/$program";
- #strip "$REPOSITORYDIR/$program";
+ strip -x "$REPOSITORYDIR/$program";
 
 done
-
-for program in bin/enblend bin/enfuse
-do
- strip -x "$REPOSITORYDIR/$program"
-done
