@@ -19,10 +19,18 @@
 
 # -------------------------------
 # 20091206.0 sg Script tested and used to build 2009.4.0-RC3
+# 20100624.0 hvdw More robust error checking on compilation
 # -------------------------------
 
 # init
 
+fail()
+{
+        echo "** Failed at $1 **"
+        exit 1
+}
+
+
 let NUMARCH="0"
 
 mkdir -p "$REPOSITORYDIR/bin";
@@ -80,18 +88,19 @@ do
 
  env \
   CC=$CC CXX=$CXX \
-  CFLAGS="-isysroot $MACSDKDIR -arch $ARCH $ARCHARGs $OTHERARGs -O2 -dead_strip" \
-  CXXFLAGS="-isysroot $MACSDKDIR -arch $ARCH $ARCHARGs $OTHERARGs -O2 -dead_strip" \
+  CFLAGS="-isysroot $MACSDKDIR -arch $ARCH $ARCHARGs $OTHERARGs -O3 -dead_strip" \
+  CXXFLAGS="-isysroot $MACSDKDIR -arch $ARCH $ARCHARGs $OTHERARGs -O3 -dead_strip" \
   CPPFLAGS="-I$REPOSITORYDIR/include" \
   LDFLAGS="-L$REPOSITORYDIR/lib -mmacosx-version-min=$OSVERSION -dead_strip -prebind" \
   NEXT_ROOT="$MACSDKDIR" \
   ./configure --prefix="$REPOSITORYDIR" --disable-dependency-tracking \
     --host="$TARGET" --exec-prefix=$REPOSITORYDIR/arch/$ARCH \
-    --program-transform-name='s/^make$/gnumake/';
+   --with-libiconv-prefix=$REPOSITORYDIR --with-libintl-prefix=$REPOSITORYDIR \
+   --program-transform-name='s/^make$/gnumake/' || fail "configure step for $ARCH";
 
  make clean;
- make;
- make install;
+ make || fail "failed at make step of $ARCH";
+ make install || fail "make install step of $ARCH";
 
 done
 
