@@ -19,10 +19,23 @@
 
 # -------------------------------
 # 20091206.0 sg Script NOT tested but uses std boilerplate
+# 20100111.0 sg Script tested for building dylib
+# 20100121.0 sg Script updated for 0.19
+# 20100624.0 hvdw More robust error checking on compilation
 # -------------------------------
 
 # init
 
+fail()
+{
+        echo "** Failed at $1 **"
+        exit 1
+}
+
+
+EXIV2VER_M="6"
+EXIV2VER_FULL="$EXIV2VER_M.3.1"
+
 let NUMARCH="0"
 
 for i in $ARCHS
@@ -34,10 +47,6 @@ mkdir -p "$REPOSITORYDIR/bin";
 mkdir -p "$REPOSITORYDIR/lib";
 mkdir -p "$REPOSITORYDIR/include";
 
-EXIV2VER_M="5"
-EXIV2VER_FULL="$EXIV2VER_M.3.1"
-
-
 # compile
 
 for ARCH in $ARCHS
@@ -82,17 +91,18 @@ do
 
  env \
   CC=$CC CXX=$CXX \
-  CFLAGS="-isysroot $MACSDKDIR -arch $ARCH $ARCHARGs $OTHERARGs -O2 -dead_strip" \
-  CXXFLAGS="-isysroot $MACSDKDIR -arch $ARCH $ARCHARGs $OTHERARGs -O2 -dead_strip" \
+  CFLAGS="-isysroot $MACSDKDIR -arch $ARCH $ARCHARGs $OTHERARGs -O3 -dead_strip" \
+  CXXFLAGS="-isysroot $MACSDKDIR -arch $ARCH $ARCHARGs $OTHERARGs -O3 -dead_strip" \
   CPPFLAGS="-I$REPOSITORYDIR/include" \
-  LDFLAGS="-L$REPOSITORYDIR/lib -mmacosx-version-min=$OSVERSION -dead_strip -prebind" \
+  LDFLAGS="-L$REPOSITORYDIR/lib -arch $ARCH -mmacosx-version-min=$OSVERSION -dead_strip -prebind" \
   NEXT_ROOT="$MACSDKDIR" \
   ./configure --prefix="$REPOSITORYDIR" --disable-dependency-tracking \
   --host="$TARGET" --exec-prefix=$REPOSITORYDIR/arch/$ARCH \
   --enable-shared --with-libiconv-prefix=$REPOSITORYDIR --with-libintl-prefix=$REPOSITORYDIR \
-  ;
+  --enable-static  || fail "configure step for $ARCH";
 
- [ -f "libtool-bk" ] || mv "libtool" "libtool-bk"; # just move it once, fix it many times
+ [ -f "libtool-bk" ] && rm libtool-bk; 
+ mv "libtool" "libtool-bk"; 
  sed -e "s#-dynamiclib#-shared-libgcc -dynamiclib -arch $ARCH -isysroot $MACSDKDIR#g" \
      -e 's/-all_load//g' "libtool-bk" > "libtool";
  chmod +x libtool
@@ -104,16 +114,15 @@ do
  cd ../../;
 
  cd src;
- make $OTHERMAKEARGs lib;
- make install-lib;
+ make $OTHERMAKEARGs || fail "failed at make step of $ARCH";
+ make install || fail "make install step of $ARCH";
  cd ../;
-
 done
 
 
 # merge libexiv2
 
-for liba in lib/libexiv2.a lib/libexiv2.$EXIV2VER_FULL.dylib
+for liba in lib/libexiv2.a lib/libexiv2.$EXIV2VER_M.dylib
 do
 
  if [ $NUMARCH -eq 1 ] ; then
@@ -149,13 +158,58 @@ do
 done
 
 
-if [ -f "$REPOSITORYDIR/lib/libexiv2.$EXIV2VER_FULL.dylib" ]
+if [ -f "$REPOSITORYDIR/lib/libexiv2.$EXIV2VER_M.dylib" ]
 then
- install_name_tool -id "$REPOSITORYDIR/lib/libexiv2.$EXIV2VER_FULL.dylib" "$REPOSITORYDIR/lib/libexiv2.$EXIV2VER_FULL.dylib"
- ln -sfn libexiv2.$EXIV2VER_FULL.dylib $REPOSITORYDIR/lib/libexiv2.$EXIV2VER_M.dylib;
- ln -sfn libexiv2.$EXIV2VER_FULL.dylib $REPOSITORYDIR/lib/libexiv2.dylib;
+ install_name_tool -id "$REPOSITORYDIR/lib/libexiv2.$EXIV2VER_M.dylib" "$REPOSITORYDIR/lib/libexiv2.$EXIV2VER_M.dylib"
+# ln -sfn libexiv2.$EXIV2VER_FULL.dylib $REPOSITORYDIR/lib/libexiv2.$EXIV2VER_M.dylib;
+ ln -sfn libexiv2.$EXIV2VER_M.dylib $REPOSITORYDIR/lib/libexiv2.dylib;
 fi
 
+# merge execs
+
+for program in bin/exiv2
+do
+
+ if [ $NUMARCH -eq 1 ] ; then
+   if [ -f $REPOSITORYDIR/arch/$ARCHS/$program ] ; then
+		 echo "Moving arch/$ARCHS/$program to $program"
+  	 mv "$REPOSITORYDIR/arch/$ARCHS/$program" "$REPOSITORYDIR/$program";
+  	 strip -x "$REPOSITORYDIR/$program";
+  	 continue
+	 else
+		 echo "Program arch/$ARCHS/$program not found. Aborting build";
+		 exit 1;
+	 fi
+ fi
+
+ LIPOARGs=""
+
+ for ARCH in $ARCHS
+ do
+ 	if [ -f $REPOSITORYDIR/arch/$ARCH/$program ] ; then
+		echo "Adding arch/$ARCH/$program to bundle"
+ 		LIPOARGs="$LIPOARGs $REPOSITORYDIR/arch/$ARCH/$program"
+	else
+		echo "File arch/$ARCH/$program was not found. Aborting build";
+		exit 1;
+	fi
+ done
+
+ lipo $LIPOARGs -create -output "$REPOSITORYDIR/$program";
+ strip -x "$REPOSITORYDIR/$program";
+
+done
+
+# Last step for exiv2. exiv2 is linked during build against it's own libexiv2.dylib and therefore has an install_name
+# based on the arch/$ARCH directory. We need to change that. Unfortunately we need to do it for every arch even 
+# though it is only mentioned once for one of the arc/$ARCHs.
+
+for ARCH in $ARCHS
+do
+  install_name_tool -change $REPOSITORYDIR/arch/$ARCH/lib/libexiv2.$EXIV2VER_M.dylib $REPOSITORYDIR/lib/libexiv2.$EXIV2VER_M.dylib $REPOSITORYDIR/bin/exiv2
+done
+
+
 
 #pkgconfig
 for ARCH in $ARCHS
