@@ -20,8 +20,16 @@
 
 # -------------------------------
 # 20091206.0 sg Script tested and used to build 2009.4.0-RC3
+# 20100121.0 sg Script updated for 3.9.2
+# 20100624.0 hvdw More robust error checking on compilation
 # -------------------------------
 
+fail()
+{
+        echo "** Failed at $1 **"
+        exit 1
+}
+
 uname_release=$(uname -r)
 uname_arch=$(uname -p)
 [ $uname_arch = powerpc ] && uname_arch="ppc"
@@ -114,20 +122,20 @@ do
 
  env \
    CC=$CC CXX=$CXX \
-   CFLAGS="-isysroot $MACSDKDIR -arch $ARCH $ARCHARGs $OTHERARGs -O2 -dead_strip" \
-   CXXFLAGS="-isysroot $MACSDKDIR -arch $ARCH $ARCHARGs $OTHERARGs -O2 -dead_strip" \
+   CFLAGS="-isysroot $MACSDKDIR -arch $ARCH $ARCHARGs $OTHERARGs -O3 -dead_strip" \
+   CXXFLAGS="-isysroot $MACSDKDIR -arch $ARCH $ARCHARGs $OTHERARGs -O3 -dead_strip" \
    CPPFLAGS="-I$REPOSITORYDIR/include" \
    LDFLAGS="-L$REPOSITORYDIR/lib -mmacosx-version-min=$OSVERSION -dead_strip" \
    NEXT_ROOT="$MACSDKDIR" \
    ./configure --prefix="$REPOSITORYDIR" --disable-dependency-tracking \
      --host="$TARGET" --exec-prefix=$REPOSITORYDIR/arch/$ARCH \
      --enable-static --enable-shared --with-apple-opengl-framework --without-x \
-     ;
+     || fail "configure step for $ARCH" ;
 
  [ -f $REPOSITORYDIR/$crt1obj ] && rm  $REPOSITORYDIR/$crt1obj;
  make clean;
- cd ./port; make $OTHERMAKEARGs;
- cd ../libtiff; make $OTHERMAKEARGs install;
+ cd ./port; make $OTHERMAKEARGs || fail "failed at make step of $ARCH";
+ cd ../libtiff; make $OTHERMAKEARGs install || fail "make install step of $ARCH";
  cd ../;
 
  rm $REPOSITORYDIR/include/tiffconf.h;
