@@ -22,13 +22,14 @@
 #ifndef __CELESTE__
 #define __CELESTE__
 
+#include <hugin_shared.h>
 #include "svm.h"
 #include <string>
 #include <vector>
 
 using namespace std;
 
-void get_gabor_response(string&,unsigned int&,string&,double&,string&,vector<double>&);
+IMPEX void get_gabor_response(string&,unsigned int&,string&,double&,string&,vector<double>&);
 
 #endif
 
