@@ -3,7 +3,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: PanoCommand.h 3208 2008-07-18 13:58:08Z gpatters1 $
+ *  $Id$
  *
  *  This is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -30,6 +30,7 @@
 //#include "PanoImage.h"
 #include "Panorama.h"
 //#include "PanoToolsInterface.h"
+#include <panodata/StandardImageVariableGroups.h>
 
 
 
@@ -92,19 +93,15 @@ namespace PT {
     class AddImagesCmd : public PanoCommand
     {
     public:
-        AddImagesCmd(Panorama & pano, const std::vector<PanoImage> & images)
+        AddImagesCmd(Panorama & pano, const std::vector<SrcPanoImage> & images)
             : PanoCommand(pano), imgs(images)
             { };
 
         virtual bool processPanorama(Panorama& pano)
             {
-
-                VariableMap var;
-                fillVariableMap(var);
-
-                std::vector<PanoImage>::const_iterator it;
+                std::vector<SrcPanoImage>::const_iterator it;
                 for (it = imgs.begin(); it != imgs.end(); ++it) {
-                    pano.addImage(*it,var);
+                    pano.addImage(*it);
                 }
                 pano.changeFinished();
 
@@ -117,7 +114,7 @@ namespace PT {
             }
 
     private:
-        std::vector<PanoImage> imgs;
+        std::vector<SrcPanoImage> imgs;
     };
 
 
@@ -477,151 +474,6 @@ namespace PT {
     //=========================================================================
 
 
-    /** update LensVariables for one lens */
-    class SetLensVariableCmd : public PanoCommand
-    {
-    public:
-        SetLensVariableCmd(Panorama & p, int lens, const LensVarMap & var)
-            : PanoCommand(p), lensNr(lens), vars(var)
-            { };
-
-        virtual bool processPanorama(Panorama& pano)
-            {
-                LensVarMap::const_iterator it;
-                for (it = vars.begin(); it != vars.end(); ++it) {
-                    pano.updateLensVariable(lensNr, it->second);
-                }
-                pano.changeFinished();
-
-                return true;
-            }
-        
-        virtual std::string getName() const
-            {
-                return "set lens variable";
-            }
-
-    private:
-        unsigned lensNr;
-        LensVarMap vars;
-    };
-
-
-    //=========================================================================
-    //=========================================================================
-
-
-    /** update LensVariables for multiple lenses */
-    class SetLensesVariableCmd : public PanoCommand
-    {
-    public:
-		SetLensesVariableCmd(Panorama & p, UIntSet lenses, const std::vector<LensVarMap> & var)
-            : PanoCommand(p), lensNrs(lenses), vars(var)
-            { };
-
-        virtual bool processPanorama(Panorama& pano)
-            {
-				assert(lensNrs.size() == vars.size());
-				std::vector<LensVarMap>::iterator vit = vars.begin();
-				for (UIntSet::iterator lit = lensNrs.begin(); lit != lensNrs.end(); ++lit, ++vit) {
-					LensVarMap::const_iterator it;
-					for (it = (*vit).begin(); it != (*vit).end(); ++it) {
-						pano.updateLensVariable(*lit, it->second);
-					}
-					pano.changeFinished();
-				}
-                return true;
-            }
-        
-        virtual std::string getName() const
-            {
-                return "set lens variable";
-            }
-
-    private:
-        UIntSet lensNrs;
-        std::vector<LensVarMap> vars;
-    };
-
-
-    //=========================================================================
-    //=========================================================================
-
-
-    /** update LensVariables for one lens */
-    class SetLensVariablesCmd : public PanoCommand
-    {
-        public:
-            SetLensVariablesCmd(Panorama & p, UIntSet lenses, const std::vector<LensVarMap> & var)
-            : PanoCommand(p), lensNrs(lenses), vars(var)
-            { };
-
-            virtual bool processPanorama(Panorama& pano)
-            {
-                for (UIntSet::iterator itl = lensNrs.begin();
-                     itl != lensNrs.end(); ++itl)
-                {
-                    // lens var map it.
-                    LensVarMap::const_iterator it;
-                    for (it = vars[*itl].begin(); it != vars[*itl].end(); ++it) {
-                        pano.updateLensVariable(*itl, it->second);
-                    }
-                }
-                pano.changeFinished();
-
-                return true;
-            }
-            
-        virtual std::string getName() const
-            {
-                return "set lens variables";
-            }
-    
-    private:
-            UIntSet lensNrs;
-            std::vector<LensVarMap> vars;
-    };
-
-
-    //=========================================================================
-    //=========================================================================
-
-    /** change the lens for an image */
-    class SetImageLensCmd : public PanoCommand
-    {
-    public:
-        SetImageLensCmd(Panorama & p, const UIntSet & imgNrs, int lensNr)
-            : PanoCommand(p),
-              imgNrs(imgNrs), lensNr(lensNr)
-            { };
-
-        virtual bool processPanorama(Panorama& pano)
-            {
-                for (UIntSet::iterator it = imgNrs.begin();
-                     it != imgNrs.end(); ++it)
-                {
-                    pano.setLens(*it, lensNr);
-                }
-                pano.changeFinished();
-
-                return true;
-            }
-        
-        virtual std::string getName() const
-            {
-                return "set lens";
-            }
-
-    private:
-        UIntSet imgNrs;
-        unsigned int lensNr;
-    };
-
-
-    //=========================================================================
-    //=========================================================================
-
-
     /** center panorama horizontically */
     class CenterPanoCmd : public PanoCommand
     {
@@ -836,137 +688,6 @@ namespace PT {
     };
 
 
-    //=========================================================================
-    //=========================================================================
-
-
-    /** add a new lens */
-    class AddLensCmd : public PanoCommand
-    {
-    public:
-        AddLensCmd(Panorama & p, const Lens & lens)
-            : PanoCommand(p), lens(lens)
-            { };
-
-        virtual bool processPanorama(Panorama& pano)
-            {
-                pano.addLens(lens);
-                pano.changeFinished();
-
-                return true;
-            }
-        
-        virtual std::string getName() const
-            {
-                return "addLens";
-            }
-
-    private:
-        Lens lens;
-    };
-
-
-    //=========================================================================
-    //=========================================================================
-
-
-    /** add a new lens to some images */
-    class AddNewLensToImagesCmd : public PanoCommand
-    {
-    public:
-        AddNewLensToImagesCmd(Panorama & p, const Lens & lens, const UIntSet & imgs)
-            : PanoCommand(p), lens(lens), imgNrs(imgs)
-            { };
-
-        virtual bool processPanorama(Panorama& pano)
-            {
-                unsigned int lnr = pano.addLens(lens);
-                for (UIntSet::iterator it = imgNrs.begin();
-                     it != imgNrs.end(); ++it)
-                {
-                    pano.setLens(*it, lnr);
-                }
-                pano.changeFinished();
-
-                return true;
-            }
-        
-        virtual std::string getName() const
-            {
-                return "addLens and images";
-            }
-
-    private:
-        Lens lens;
-        UIntSet imgNrs;
-    };
-
-
-    //=========================================================================
-    //=========================================================================
-
-
-    /** change lens */
-    class ChangeLensCmd : public PanoCommand
-    {
-    public:
-        ChangeLensCmd(Panorama & p, unsigned int lensNr, const Lens & lens)
-            : PanoCommand(p), lensNr(lensNr), newLens(lens)
-            { };
-
-        virtual bool processPanorama(Panorama& pano)
-            {
-                pano.updateLens(lensNr, newLens);
-                pano.changeFinished();
-
-                return true;
-            }
-        
-        virtual std::string getName() const
-            {
-                return "change lens";
-            }
-
-    private:
-        unsigned int lensNr;
-        Lens newLens;
-    };
-
-
-    //=========================================================================
-    //=========================================================================
-
-    /** change lenses */
-    class ChangeLensesCmd : public PanoCommand
-    {
-    public:
-        ChangeLensesCmd(Panorama & p, UIntSet & lNr, const LensVector & lenses)
-            : PanoCommand(p), change(lNr), vect(lenses)
-            { };
-
-        virtual bool processPanorama(Panorama& pano)
-            {
-                UIntSet::iterator it;
-                LensVector::const_iterator v_it = vect.begin();
-                for (it = change.begin(); it != change.end(); ++it) {
-                    pano.updateLens(*it, *v_it);
-                    v_it++;
-                }
-                pano.changeFinished();
-
-                return true;
-            }
-        
-        virtual std::string getName() const
-            {
-                return "change lens";
-            }
-
-    private:
-        UIntSet change;
-        LensVector vect;
-    };
-
 
     //=========================================================================
     //=========================================================================
@@ -1027,7 +748,7 @@ namespace PT {
         
         virtual std::string getName() const
             {
-                return "change lens";
+                return "swap images";
             }
 
     private:
@@ -1036,6 +757,31 @@ namespace PT {
     };
 
 
+    /** merge two project files */
+    class MergePanoCmd : public PanoCommand
+    {
+    public:
+        MergePanoCmd(Panorama & p, Panorama & p2)
+            : PanoCommand(p), newPano(p2)
+            { };
+
+        virtual bool processPanorama(Panorama& pano)
+            {
+                pano.mergePanorama(newPano);
+                pano.changeFinished();
+
+                return true;
+            }
+        
+        virtual std::string getName() const
+            {
+                return "merge panorama";
+            }
+
+    private:
+        Panorama newPano;
+    };
+
     //=========================================================================
     //=========================================================================
 
@@ -1266,33 +1012,21 @@ namespace PT {
     class SetVigCorrCmd : public PanoCommand
     {
         public:
-            SetVigCorrCmd(Panorama & p, unsigned int lensNr,
+            SetVigCorrCmd(Panorama & p, unsigned int imageNr,
                           unsigned int vigCorrMode, std::vector<double> & coeff,
                           const std::string & flatfield)
-            : PanoCommand(p), m_lensNr(lensNr),
+            : PanoCommand(p), m_imageNr(imageNr),
               m_mode(vigCorrMode), m_coeff(coeff), m_flat(flatfield)
             { };
 
             virtual bool processPanorama(Panorama& pano)
             {
                 // set data inside panorama options
-                for (unsigned int i = 0; i < pano.getNrOfImages(); i++) {
-                    if (pano.getImage(i).getLensNr() == m_lensNr) {
-                        // modify panorama options.
-                        PT::ImageOptions opts = pano.getImage(i).getOptions();
-                        opts.m_vigCorrMode = m_mode;
-                        opts.m_flatfield = m_flat;
-                        pano.setImageOptions(i, opts);
-                    }
-                }
-                // set new correction variables
-                const char *vars[] = {"Va", "Vb", "Vc", "Vd", "Vx", "Vy"};
-                for (unsigned int i=0; i < 6 ; i++) {
-                    LensVariable lv = const_map_get(pano.getLens(m_lensNr).variables, vars[i]);
-                    lv.setValue(m_coeff[i]);
-                    lv.setLinked(true);
-                    pano.updateLensVariable(m_lensNr, lv);
-                }
+                // modify panorama options.
+                PT::ImageOptions opts = pano.getImage(m_imageNr).getOptions();
+                opts.m_vigCorrMode = m_mode;
+                opts.m_flatfield = m_flat;
+                pano.setImageOptions(m_imageNr, opts);
                 pano.changeFinished();
 
                 return true;
@@ -1305,7 +1039,7 @@ namespace PT {
     
     private:
             ImageOptions options;
-            unsigned int m_lensNr;
+            unsigned int m_imageNr;
             unsigned int m_mode;
             std::vector<double> m_coeff;
             std::string m_flat;
@@ -1342,6 +1076,386 @@ namespace PT {
         double y,p,r;
     };
 
+    //=========================================================================
+    //=========================================================================
+
+    /** Update the focal length 
+     */
+    class UpdateFocalLengthCmd : public PanoCommand
+    {
+    public:
+        UpdateFocalLengthCmd(Panorama & p, UIntSet imgs, double newFocalLength)
+            : PanoCommand(p), imgNrs(imgs), m_focalLength(newFocalLength)
+            { };
+
+        virtual bool processPanorama(Panorama& pano)
+            {
+                pano.UpdateFocalLength(imgNrs,m_focalLength);
+                pano.changeFinished();
+                return true;
+            }
+        
+        virtual std::string getName() const
+            {
+                return "update focal length";
+            }
+
+    private:
+        UIntSet imgNrs;
+        double m_focalLength;
+    };
+
+    //=========================================================================
+    //=========================================================================
+
+
+    /** Update the crop factor 
+     */
+    class UpdateCropFactorCmd : public PanoCommand
+    {
+    public:
+        UpdateCropFactorCmd(Panorama & p, UIntSet imgs, double newCropFactor)
+            : PanoCommand(p), imgNrs(imgs), m_cropFactor(newCropFactor)
+            { };
+
+        virtual bool processPanorama(Panorama& pano)
+            {
+                //search all image with the same lens, otherwise the crop factor is updated via links, 
+                //but not the hfov if the image is not the given UIntSet
+                UIntSet allImgWithSameLens;
+                UIntSet testedLens;
+                HuginBase::StandardImageVariableGroups variable_groups(pano);
+                HuginBase::ImageVariableGroup & lenses = variable_groups.getLenses();
+                for(UIntSet::const_iterator it=imgNrs.begin();it!=imgNrs.end();it++)
+                {
+                    allImgWithSameLens.insert(*it);
+                    unsigned int lensNr=lenses.getPartNumber(*it);
+                    if(set_contains(testedLens,lensNr))
+                        continue;
+                    testedLens.insert(lensNr);
+                    for(unsigned int i=0;i<pano.getNrOfImages();i++)
+                    {
+                        if(lenses.getPartNumber(i)==lensNr)
+                        {
+                            allImgWithSameLens.insert(i);
+                        };
+                    };
+                };
+                pano.UpdateCropFactor(allImgWithSameLens, m_cropFactor);
+                pano.changeFinished();
+                return true;
+            }
+        
+        virtual std::string getName() const
+            {
+                return "update crop factor";
+            }
+
+    private:
+        UIntSet imgNrs;
+        double m_cropFactor;
+    };
+
+    //=========================================================================
+    //=========================================================================
+
+
+    /** Switch the part number of an image
+     */
+    class ChangePartNumberCmd : public PanoCommand
+    {
+    public:
+        ChangePartNumberCmd(Panorama & p,
+                            UIntSet image_numbers,
+                            std::size_t new_part_number,
+                            std::set<HuginBase::ImageVariableGroup::ImageVariableEnum> variables
+                            )
+            :   PanoCommand(p),
+                image_numbers(image_numbers),
+                new_part_number(new_part_number),
+                variables(variables)
+            { };
+
+        virtual bool processPanorama(Panorama& pano)
+            {
+                // it might change as we are setting them
+                std::size_t new_new_part_number = new_part_number;
+                HuginBase::ImageVariableGroup group(variables, pano);
+                for (UIntSet::iterator it = image_numbers.begin();
+                     it != image_numbers.end(); it++)
+                {
+                    group.switchParts(*it, new_new_part_number);
+                    // update the lens number if it changes.
+                    new_new_part_number = group.getPartNumber(*it);
+                }
+                pano.changeFinished();
+                return true;
+            }
+        
+        virtual std::string getName() const
+            {
+                return "Change part number";
+            }
+
+    private:
+        UIntSet image_numbers;
+        std::size_t new_part_number;
+        std::set<HuginBase::ImageVariableGroup::ImageVariableEnum> variables;
+    };
+    
+    //=========================================================================
+    //=========================================================================
+    
+    
+    /** Change the linking of some variables across parts of an
+     * ImageVariableGroup containing some specified images.
+     */
+    class ChangePartImagesLinkingCmd : public PanoCommand
+    {
+    public:
+        /** Constructor.
+         * @param p the panorama this affects
+         * @param image_numbers the set of image numbers that are contained
+         * within the parts you would like to link or unlink.
+         * @param changeVariables the set of variables you would like to change
+         * the linking of across those parts.
+         * @param new_linked_state true to link variables, false to unlink them.
+         * @param groupVariables the variables that make the ImageVariableGroup
+         * that will define which images belong to which parts.
+         */
+        ChangePartImagesLinkingCmd(Panorama & p,
+                    UIntSet image_numbers,
+                    std::set<HuginBase::ImageVariableGroup::ImageVariableEnum> changeVariables,
+                    bool new_linked_state,
+                    std::set<HuginBase::ImageVariableGroup::ImageVariableEnum> groupVariables)
+            :   PanoCommand(p),
+                image_numbers(image_numbers),
+                changeVariables(changeVariables),
+                new_linked_state(new_linked_state),
+                groupVariables(groupVariables)
+            { };
+
+        virtual bool processPanorama(Panorama& pano)
+            {
+                HuginBase::ImageVariableGroup group(groupVariables, pano);
+                if (new_linked_state)
+                {
+                    for (UIntSet::iterator imageIt = image_numbers.begin();
+                        imageIt != image_numbers.end(); imageIt++)
+                    {
+                        // link the variables
+                        for (std::set<HuginBase::ImageVariableGroup::ImageVariableEnum>::iterator variableIt = changeVariables.begin();
+                             variableIt != changeVariables.end(); variableIt++)
+                        {
+                            group.linkVariableImage(*variableIt, *imageIt);
+                        }
+                    }
+                } else {
+                    for (UIntSet::iterator imageIt = image_numbers.begin();
+                        imageIt != image_numbers.end(); imageIt++)
+                    {
+                        // unlink the variable
+                        for (std::set<HuginBase::ImageVariableGroup::ImageVariableEnum>::iterator variableIt = changeVariables.begin();
+                             variableIt != changeVariables.end(); variableIt++)
+                        {
+                            group.unlinkVariableImage(*variableIt, *imageIt);
+                            group.updatePartNumbers();
+                        }
+                    }
+                }
+                pano.changeFinished();
+                return true;
+            }
+        
+        virtual std::string getName() const
+            {
+                return "Change image variable links";
+            }
+
+    private:
+        UIntSet image_numbers;
+        std::set<HuginBase::ImageVariableGroup::ImageVariableEnum> changeVariables;
+        bool new_linked_state;
+        std::set<HuginBase::ImageVariableGroup::ImageVariableEnum> groupVariables;
+    };
+    
+    
+    //=========================================================================
+    //=========================================================================
+    
+    
+    /** Link a set of lens variables for some lens.
+     */
+    class LinkLensVarsCmd : public PanoCommand
+    {
+    public:
+        LinkLensVarsCmd(Panorama & p,
+                    std::size_t lens_number,
+                    std::set<HuginBase::ImageVariableGroup::ImageVariableEnum> variables
+                    )
+            :   PanoCommand(p),
+                lens_number(lens_number),
+                variables(variables)
+            { };
+
+        virtual bool processPanorama(Panorama& pano)
+            {
+                HuginBase::StandardImageVariableGroups variable_groups(pano);
+                HuginBase::ImageVariableGroup & lenses = variable_groups.getLenses();
+                std::set<HuginBase::ImageVariableGroup::ImageVariableEnum>::iterator it;
+                for (it = variables.begin(); it != variables.end(); it++)
+                {
+                    lenses.linkVariablePart(*it, lens_number);
+                }
+                pano.changeFinished();
+                return true;
+            }
+        
+        virtual std::string getName() const
+            {
+                return "Link lens variables";
+            }
+
+    private:
+        std::size_t lens_number;
+        std::set<HuginBase::ImageVariableGroup::ImageVariableEnum> variables;
+    };
+    //=========================================================================
+    //=========================================================================
+    
+/// @todo avoid copying image data in processPanorama
+#define image_variable( name, type, default_value )\
+    class ChangeImage##name##Cmd : public PanoCommand\
+    {\
+    public:\
+        ChangeImage##name##Cmd(Panorama & p,\
+                               UIntSet image_numbers,\
+                               type value)\
+            :   PanoCommand(p),\
+                image_numbers(image_numbers),\
+                value(value)\
+            { };\
+        \
+        virtual bool processPanorama(Panorama& pano)\
+            {\
+                for (UIntSet::iterator it = image_numbers.begin();\
+                     it != image_numbers.end(); it++)\
+                {\
+                    HuginBase::SrcPanoImage img = pano.getSrcImage(*it);\
+                    img.set##name(value);\
+                    pano.setSrcImage(*it, img);\
+                }\
+                pano.changeFinished();\
+                return true;\
+            }\
+        \
+        virtual std::string getName() const\
+            {\
+                return "Change image's " #name;\
+            }\
+    private:\
+        UIntSet image_numbers;\
+        type value;\
+    };
+#include <panodata/image_variables.h>
+#undef image_variable
+    
+    
+    //=========================================================================
+    //=========================================================================
+    
+    
+    /** Make a new part in a ImageVariableGroup for a set of images, given the
+     * variables that make up the group.
+     */
+    class NewPartCmd : public PanoCommand
+    {
+    public:
+        /** Constructor.
+         * @param p Panorama to act up
+         * @param image_numbers A set of images which should all be in a single
+         * new group.
+         * @param vars The set of image variables that make up the group. Should
+         * be got from StandardVariableGroups.
+         */
+        NewPartCmd( Panorama & p,
+                    UIntSet image_numbers,
+                    std::set<HuginBase::ImageVariableGroup::ImageVariableEnum> vars)
+            :   PanoCommand(p),
+                image_numbers(image_numbers),
+                vars(vars)
+        { }
+        
+        virtual bool processPanorama(Panorama& pano)
+        {
+            // unlink all the variables in the first image.
+            DEBUG_ASSERT (image_numbers.size() > 0);
+            unsigned int image_index = *image_numbers.begin();
+            for (std::set<HuginBase::ImageVariableGroup::ImageVariableEnum>::iterator it = vars.begin();
+                    it != vars.end(); it++)
+            {
+                switch (*it)
+                {
+#define image_variable( name, type, default_value )\
+                    case HuginBase::ImageVariableGroup::IVE_##name:\
+                        pano.unlinkImageVariable##name(image_index);\
+                        break;
+#include <panodata/image_variables.h>
+#undef image_variable
+                }
+            }
+            // now the first image should have a new part in the group.
+            // we want to switch the rest of the images to the new part.
+            HuginBase::ImageVariableGroup group(vars, pano);
+            for (UIntSet::iterator it = ++image_numbers.begin();
+                    it != image_numbers.end(); it++)
+            {
+                std::size_t part_number = group.getPartNumber(image_index);
+                group.switchParts(*it, part_number);
+            }
+            pano.changeFinished();
+            return true;
+        }
+        
+        virtual std::string getName() const
+            {
+                return "New Part";
+            }
+
+    private:
+        UIntSet image_numbers;
+        std::set<HuginBase::ImageVariableGroup::ImageVariableEnum> vars;
+    };
+
+    //=========================================================================
+    //=========================================================================
+
+    /** update mask for given image */
+    class UpdateMaskForImgCmd : public PanoCommand
+    {
+    public:
+        UpdateMaskForImgCmd(Panorama & p, unsigned int img, const HuginBase::MaskPolygonVector & mask)
+            : PanoCommand(p),
+              m_img(img), m_mask(mask)
+            { };
+
+        virtual bool processPanorama(Panorama& pano)
+            {
+                pano.updateMasksForImage(m_img,m_mask);
+                pano.changeFinished();
+
+                return true;
+            }
+        
+        virtual std::string getName() const
+            {
+                return "update mask";
+            }
+
+    private:
+        unsigned int m_img;
+        HuginBase::MaskPolygonVector m_mask;
+    };
 
 } // namespace PT
 
