@@ -3,7 +3,7 @@
 *
 *  @author Ippei UKAI <ippei_ukai@mac.com>
 *
-*  $Id: MyExternalCmdExecDialog.h 4600 2009-10-11 12:08:03Z stativ $
+*  $Id$
 *
 *  This is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
@@ -27,11 +27,13 @@
 #ifndef _MYEXTERNALCMDEXECDIALOG__H
 #define _MYEXTERNALCMDEXECDIALOG__H
 
+#include <hugin_shared.h>
+
 const int HUGIN_EXIT_CODE_CANCELLED = -255;
 
 class MyExternalCmdExecDialog;
 class HuginPipedProcess;
-int MyExecuteCommandOnDialog(wxString command, wxString args, wxWindow* parent, wxString title);
+WXIMPEX int MyExecuteCommandOnDialog(wxString command, wxString args, wxWindow* parent, wxString title);
 
 //#define HUGIN_EXEC_LISTBOX 1
 
