@@ -6,7 +6,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: AssistantPanel.cpp 4782 2009-12-11 17:40:38Z jlegg $
+ *  $Id$
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -52,8 +52,8 @@
 #include "hugin/AutoCtrlPointCreator.h"
 #include "hugin/TextKillFocusHandler.h"
 #include "base_wx/PTWXDlg.h"
-#include "hugin/PanoDruid.h"
 #include "base_wx/MyProgressDialog.h"
+#include "hugin/HFOVDialog.h"
 #include "hugin/config_defaults.h"
 #include <algorithms/control_points/CleanCP.h>
 
@@ -147,6 +147,7 @@ bool AssistantPanel::Create(wxWindow *parent, wxWindowID id, const wxPoint& pos,
 
     m_lensTypeChoice = XRCCTRL(*this, "ass_lens_proj_choice", wxChoice);
     DEBUG_ASSERT(m_lensTypeChoice);
+    FillLensProjectionList(m_lensTypeChoice);
     m_lensTypeChoice->SetSelection(0);
 
     m_focalLengthText = XRCCTRL(*this, "ass_focallength_text", wxTextCtrl);
@@ -168,14 +169,6 @@ bool AssistantPanel::Create(wxWindow *parent, wxWindowID id, const wxPoint& pos,
     DEBUG_ASSERT(m_createButton);
     m_createButton->Disable();
 
-    // druid is currently disabled
-    m_druid = 0;
-            /*
-    m_druid = new PanoDruid(this);
-    wxXmlResource::Get()->AttachUnknownControl (wxT("ass_druid"), m_druid );
-    m_druid->Update(m_pano);
-            */
-
     m_panel = XRCCTRL(*this, "ass_control_panel", wxPanel);
     DEBUG_ASSERT(m_panel);
 
@@ -192,6 +185,7 @@ bool AssistantPanel::Create(wxWindow *parent, wxWindowID id, const wxPoint& pos,
 void AssistantPanel::Init(Panorama * pano)
 {
     m_pano = pano;
+    m_variable_groups = new HuginBase::StandardImageVariableGroups(*m_pano);
     // observe the panorama
     m_pano->addObserver(this);
 }
@@ -202,6 +196,7 @@ AssistantPanel::~AssistantPanel(void)
     m_focalLengthText->PopEventHandler(true);
     m_cropFactorText->PopEventHandler(true);
     m_pano->removeObserver(this);
+    delete m_variable_groups;
     DEBUG_TRACE("dtor end");
 }
 
@@ -233,8 +228,8 @@ void AssistantPanel::panoramaImagesChanged(PT::Panorama &pano, const PT::UIntSet
 void AssistantPanel::panoramaChanged(PT::Panorama &pano)
 {
     DEBUG_TRACE("");
-
-    if (m_druid) m_druid->Update(*m_pano);
+    
+    m_variable_groups->update();
 
     m_alignButton->Enable(pano.getNrOfImages() > 1);
 
@@ -272,7 +267,7 @@ void AssistantPanel::panoramaChanged(PT::Panorama &pano)
 
         wxString imgMsg = wxString::Format(_("%d images loaded."), pano.getNrOfImages());
 
-        const Lens & lens = pano.getLens(0);
+        const Lens & lens = m_variable_groups->getLens(0);
 
         /*
         if (!lens.m_hasExif) {
@@ -294,7 +289,7 @@ void AssistantPanel::panoramaChanged(PT::Panorama &pano)
         m_noImage = false;
 
         // update data in lens display
-        m_lensTypeChoice->SetSelection(lens.getProjection());
+        SelectProjection(m_lensTypeChoice, lens.getProjection());
         double focal_length = lens.getFocalLength();
         m_focalLengthText->SetValue(doubleTowxString(focal_length,m_degDigits));
         double focal_length_factor = lens.getCropFactor();
@@ -343,12 +338,12 @@ void AssistantPanel::panoramaChanged(PT::Panorama &pano)
             }
         }
         m_alignText->SetLabel(alignMsg);
-        /** @todo Use width of avaliable space to wrap this message.
-         */
-        m_alignText->Wrap(600);
     } else {
-        m_alignText->SetLabel(wxT(""));
+        m_alignText->SetLabel(_("Note: automatic alignment uses default settings from the preferences. If you customize settings for this project in the advanced tabs and want to use these customized settings, run the CP detection from the Images Tab, the geometrical optimization from the the Optimizer tab and the photometric optimization from the Exposure tab."));
     }
+    /** @todo Use width of avaliable space to wrap this message.
+     */
+        m_alignText->Wrap(600);
     // re-layout panel (adjusts m_alignText size)
     m_panel->Layout();
     m_panel->FitInside();
@@ -586,7 +581,7 @@ void AssistantPanel::OnAlign( wxCommandEvent & e )
 
     // set TIFF_m with enblend
     PanoramaOptions opts = m_pano->getOptions();
-    opts.outputFormat = PanoramaOptions::TIFF;
+    opts.outputFormat = PanoramaOptions::TIFF_m;
     opts.blendMode = PanoramaOptions::ENBLEND_BLEND;
     opts.remapper = PanoramaOptions::NONA;
     opts.tiff_saveROI = true;
@@ -650,17 +645,31 @@ void AssistantPanel::OnAlign( wxCommandEvent & e )
 
     // TODO: photometric optimisation.
     // first, ensure that vignetting and response coefficients are linked
-    const char * varnames[] = {"Va", "Vb", "Vc", "Vd", "Vx", "Vy",
-        "Ra", "Rb", "Rc", "Rd", "Re",  0};
-
-    for (size_t i = 0; i < optPano.getNrOfLenses(); i++) {
-        const Lens & l = optPano.getLens(i);
-        for (const char ** v = varnames; *v; v++) {
-            LensVariable var = const_map_get(l.variables, *v);
-            if (!var.isLinked()) {
-                var.setLinked();
-                optPano.updateLensVariable(i, var);
+    const HuginBase::ImageVariableGroup::ImageVariableEnum vars[] = {
+            HuginBase::ImageVariableGroup::IVE_EMoRParams,
+            HuginBase::ImageVariableGroup::IVE_ResponseType,
+            HuginBase::ImageVariableGroup::IVE_VigCorrMode,
+            HuginBase::ImageVariableGroup::IVE_RadialVigCorrCoeff,
+            HuginBase::ImageVariableGroup::IVE_RadialVigCorrCenterShift
+        };
+    // keep a list of commands needed to fix it:
+    HuginBase::ImageVariableGroup & lenses = m_variable_groups->getLenses();
+    for (size_t i = 0; i < lenses.getNumberOfParts(); i++)
+    {
+        std::set<HuginBase::ImageVariableGroup::ImageVariableEnum> links_needed;
+        links_needed.clear();
+        for (int v = 0; v < 5; v++)
+        {
+            if (!lenses.getVarLinkedInPart(vars[v], i))
+            {
+                links_needed.insert(vars[v]);
             }
+        };
+        if (!links_needed.empty())
+        {
+            GlobalCmdHist::getInstance().addCommand(
+                    new PT::LinkLensVarsCmd(*m_pano, i, links_needed)
+                );
         }
     }
 
@@ -739,16 +748,14 @@ void AssistantPanel::OnAlign( wxCommandEvent & e )
         points, *(MainFrame::Get()), error);
     cout << "Auto align, photometric error: " << error *255 << " grey values" << std::endl;
 
-    // calculate the mean exposure.
-    opts.outputExposureValue = calcMeanExposure(*m_pano);
-
     // TODO: merge the following commands.
 
-
     // copy information into the main panorama
     GlobalCmdHist::getInstance().addCommand(
         new PT::UpdateVariablesCPSetCmd(*m_pano, imgs, optPano.getVariables(), optPano.getCtrlPoints())
         );
+    // calculate the mean exposure.
+    opts.outputExposureValue = calcMeanExposure(*m_pano);
 
     // copy information into our panorama
     GlobalCmdHist::getInstance().addCommand(
@@ -799,22 +806,35 @@ void AssistantPanel::OnCreate( wxCommandEvent & e )
 void AssistantPanel::OnLoadLens(wxCommandEvent & e)
 {
     unsigned int imgNr = 0;
-    unsigned int lensNr = m_pano->getImage(imgNr).getLensNr();
-    Lens lens = m_pano->getLens(lensNr);
+    unsigned int lensNr = m_variable_groups->getLenses().getPartNumber(imgNr);
+    Lens lens = m_variable_groups->getLensForImage(imgNr);
     VariableMap vars = m_pano->getImageVariables(imgNr);
     ImageOptions imgopts = m_pano->getImage(imgNr).getOptions();
 
     if (LoadLensParametersChoose(this, lens, vars, imgopts)) {
-        GlobalCmdHist::getInstance().addCommand(
-                new PT::ChangeLensCmd(*m_pano, lensNr, lens)
-                                               );
+        /** @todo maybe this isn't the best way to load the lens data.
+         * Check with LoadLensParamtersChoose how this is done, and use only
+         * the image variables, rather than merging in the lens ones.
+         */
+        for (LensVarMap::iterator it = lens.variables.begin();
+             it != lens.variables.end(); it++)
+        {
+            vars.insert(pair<std::string, HuginBase::Variable>(
+                        it->first,
+                        HuginBase::Variable(it->second.getName(),
+                                            it->second.getValue() )
+                    ));
+        }
+        /** @todo I think the sensor size should be copied over,
+         * but SrcPanoImage doesn't have such a variable yet.
+         */
         GlobalCmdHist::getInstance().addCommand(
                 new PT::UpdateImageVariablesCmd(*m_pano, imgNr, vars)
                                                );
-                // get all images with the current lens.
+        // get all images with the current lens.
         UIntSet imgs;
         for (unsigned int i = 0; i < m_pano->getNrOfImages(); i++) {
-            if (m_pano->getImage(i).getLensNr() == lensNr) {
+            if (m_variable_groups->getLenses().getPartNumber(i) == lensNr) {
                 imgs.insert(i);
             }
         }
@@ -855,15 +875,22 @@ void AssistantPanel::OnExifToggle (wxCommandEvent & e)
 void AssistantPanel::OnLensTypeChanged (wxCommandEvent & e)
 {
     // uses enum Lens::LensProjectionFormat from PanoramaMemento.h
-    int var = m_lensTypeChoice->GetSelection();
-    Lens lens = m_pano->getLens(0);
+    size_t var = GetSelectedProjection(m_lensTypeChoice);
+    Lens lens = m_variable_groups->getLens(0);
     if (lens.getProjection() != (Lens::LensProjectionFormat) var) {
         //double crop = lens.getCropFactor();
         double fl = lens.getFocalLength();
-        lens.setProjection((Lens::LensProjectionFormat) (var));
-        lens.setFocalLength(fl);
+        UIntSet imgs;
+        imgs.insert(0);
         GlobalCmdHist::getInstance().addCommand(
-                new PT::ChangeLensCmd(*m_pano, 0, lens )
+                new PT::ChangeImageProjectionCmd(
+                                    *m_pano,
+                                    imgs,
+                                    (HuginBase::SrcPanoImage::Projection) var
+                                )
+            );
+        GlobalCmdHist::getInstance().addCommand(
+                new PT::UpdateFocalLengthCmd(*m_pano, imgs, fl)
             );
     }
 }
@@ -879,24 +906,13 @@ void AssistantPanel::OnFocalLengthChanged(wxCommandEvent & e)
     if (!str2double(text, val)) {
         return;
     }
-
-
+    
     // always change first lens...
-    Lens lens = m_pano->getLens(0);
-    lens.setFocalLength(val);
-    LensVariable v = map_get(lens.variables, "v");
-    LensVarMap lmv;
-    lmv.insert(make_pair("v", v));
-
-    std::vector<LensVarMap> lvars;
-    UIntSet lenses;
-    for (unsigned i=0; i < m_pano->getNrOfLenses(); i++) {
-        lenses.insert(i);
-        lvars.push_back(lmv);
-    }
+    UIntSet images0;
+    images0.insert(0);
     GlobalCmdHist::getInstance().addCommand(
-            new PT::SetLensesVariableCmd(*m_pano, lenses, lvars)
-                                           );
+        new PT::UpdateFocalLengthCmd(*m_pano, images0, val)
+    );
 }
 
 void AssistantPanel::OnCropFactorChanged(wxCommandEvent & e)
@@ -908,31 +924,11 @@ void AssistantPanel::OnCropFactorChanged(wxCommandEvent & e)
         return;
     }
 
-    // always change first lens...
-    Lens lens = m_pano->getLens(0);
-    double fl = lens.getFocalLength();
-    lens.setCropFactor(val);
-    lens.setFocalLength(fl);
-    LensVariable v = map_get(lens.variables, "v");
-    LensVarMap lmv;
-    lmv.insert(make_pair("v", v));
-
-    std::vector<LensVarMap> lvars;
-    UIntSet lenses;
-    for (unsigned i=0; i < m_pano->getNrOfLenses(); i++) {
-        lenses.insert(i);
-        lvars.push_back(lmv);
-    }
+    UIntSet images;
+    images.insert(0);
     GlobalCmdHist::getInstance().addCommand(
-            new PT::SetLensesVariableCmd(*m_pano, lenses, lvars)
-                                           );
-    // TODO: update crop factor as well without destroying other information.
-    SrcPanoImage img0 = m_pano->getSrcImage(0);
-    img0.setExifCropFactor(val);
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::UpdateSrcImageCmd(*m_pano, 0, img0)
-                                           );
-    
+        new PT::UpdateCropFactorCmd(*m_pano,images,val)
+    );
 }
 
 IMPLEMENT_DYNAMIC_CLASS(AssistantPanel, wxPanel)
