@@ -6,7 +6,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: CPEditorPanel.cpp 4789 2009-12-14 06:53:35Z harryvanderwolf $
+ *  $Id$
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -447,7 +447,7 @@ void CPEditorPanel::OnCPEvent( CPEvent&  ev)
     } else  if (ev.GetEventObject() == m_rightImg){
         left = false;
     } else {
-        DEBUG_FATAL("UNKOWN SOURCE OF CPEvent");
+        DEBUG_FATAL("UNKNOWN SOURCE OF CPEvent");
     }
 
     switch (ev.getMode()) {
@@ -752,9 +752,9 @@ void CPEditorPanel::estimateAndAddOtherPoint(const FDiff2D & p,
     FDiff2D op;
     op = EstimatePoint(FDiff2D(p.x, p.y), left);
     // check if point is in image.
-    const PanoImage & pImg = m_pano->getImage(otherImgNr);
-    if (p.x < (int) pImg.getWidth() && p.x >= 0
-        && p.y < (int) pImg.getHeight() && p.y >= 0)
+    const SrcPanoImage & pImg = m_pano->getImage(otherImgNr);
+    if (p.x < (int) pImg.getSize().width() && p.x >= 0
+        && p.y < (int) pImg.getSize().height() && p.y >= 0)
     {
         otherImg->setNewPoint(op);
         // if fine-tune is checked, run a fine-tune session as well.
@@ -764,7 +764,7 @@ void CPEditorPanel::estimateAndAddOtherPoint(const FDiff2D & p,
             FDiff2D newPoint = otherImg->getNewPoint();
 
             long templWidth = wxConfigBase::Get()->Read(wxT("/Finetune/TemplateSize"), HUGIN_FT_TEMPLATE_SIZE);
-            const PanoImage & img = m_pano->getImage(thisImgNr);
+            const SrcPanoImage & img = m_pano->getImage(thisImgNr);
             double sAreaPercent = wxConfigBase::Get()->Read(wxT("/Finetune/SearchAreaPercent"),HUGIN_FT_SEARCH_AREA_PERCENT);
             int sWidth = (int) (img.getWidth() * sAreaPercent / 100.0);
             CorrelationResult corrPoint;
@@ -887,7 +887,7 @@ void CPEditorPanel::NewPointChange(FDiff2D p, bool left)
                 FDiff2D newPoint = otherImg->getNewPoint();
 
                 long templWidth = wxConfigBase::Get()->Read(wxT("/Finetune/TemplateSize"),HUGIN_FT_TEMPLATE_SIZE);
-                const PanoImage & img = m_pano->getImage(thisImgNr);
+                const SrcPanoImage & img = m_pano->getImage(thisImgNr);
                 double sAreaPercent = wxConfigBase::Get()->Read(wxT("/Finetune/SearchAreaPercent"),
                                                                 HUGIN_FT_SEARCH_AREA_PERCENT);
                 int sWidth = (int) (img.getWidth() * sAreaPercent / 100.0);
@@ -995,7 +995,7 @@ bool CPEditorPanel::PointFineTune(unsigned int tmplImgNr,
     wxConfigBase::Get()->Read(wxT("/Finetune/CurvThreshold"),&curvThresh,
                               HUGIN_FT_CURV_THRESHOLD);
 
-    const PanoImage & img = m_pano->getImage(subjImgNr);
+    const SrcPanoImage & img = m_pano->getImage(subjImgNr);
 
     // fixme: just cutout suitable gray 
 //    wxImage wxSubjImg;
@@ -1646,24 +1646,40 @@ void CPEditorPanel::OnTextPointChange(wxCommandEvent &e)
     ControlPoint cp = currentPoints[nr].second;
 
     // update point state
-    if (!str2double(m_x1Text->GetValue(), cp.x1)) {
+    double oldValue=cp.x1;
+    bool valid_input=str2double(m_x1Text->GetValue(), cp.x1);
+    if(valid_input)
+        valid_input=(cp.x1>=0) && (cp.x1<=m_pano->getSrcImage(cp.image1Nr).getWidth());
+    if (!valid_input) {
         m_x1Text->Clear();
-        *m_x1Text << cp.x1;
+        *m_x1Text << oldValue;
         return;
     }
-    if (!str2double(m_y1Text->GetValue(), cp.y1)) {
+    oldValue=cp.y1;
+    valid_input=str2double(m_y1Text->GetValue(), cp.y1);
+    if(valid_input)
+        valid_input=(cp.y1>=0) && (cp.y1<=m_pano->getSrcImage(cp.image1Nr).getHeight());
+    if (!valid_input) {
         m_y1Text->Clear();
-        *m_y1Text << cp.y1;
+        *m_y1Text << oldValue;
         return;
     }
-    if (!str2double(m_x2Text->GetValue(),cp.x2)) {
+    oldValue=cp.x2;
+    valid_input=str2double(m_x2Text->GetValue(), cp.x2);
+    if(valid_input)
+        valid_input=(cp.x2>=0) && (cp.x2<=m_pano->getSrcImage(cp.image2Nr).getWidth());
+    if (!valid_input) {
         m_x2Text->Clear();
-        *m_x2Text << cp.x2;
+        *m_x2Text << oldValue;
         return;
     }
-    if (!str2double(m_y2Text->GetValue(),cp.y2)) {
+    oldValue=cp.y2;
+    valid_input=str2double(m_y2Text->GetValue(), cp.y2);
+    if(valid_input)
+        valid_input=(cp.y2>=0) && (cp.y2<=m_pano->getSrcImage(cp.image1Nr).getHeight());
+    if (!valid_input) {
         m_y2Text->Clear();
-        *m_y2Text << cp.x2;
+        *m_y2Text << oldValue;
         return;
     }
 
@@ -1680,7 +1696,7 @@ void CPEditorPanel::OnTextPointChange(wxCommandEvent &e)
         cp.mode = ControlPoint::Y;
         break;
     default:
-        DEBUG_FATAL("unkown control point type selected");
+        DEBUG_FATAL("unknown control point type selected");
         return;
         break;
     }
@@ -2341,7 +2357,7 @@ void CPEditorPanel::FineTuneNewPoint(bool left)
 FDiff2D CPEditorPanel::EstimatePoint(const FDiff2D & p, bool left)
 {
     int imgNr = left? m_rightImageNr : m_leftImageNr;
-    const PanoImage & img = m_pano->getImage(imgNr);
+    const SrcPanoImage & img = m_pano->getImage(imgNr);
     FDiff2D t;
     if (currentPoints.size() == 0) {
         DEBUG_WARN("Cannot estimate position without at least one point");
