@@ -31,7 +31,9 @@
 
 #include <config.h>
 
+#if !defined Hugin_shared || !defined _WINDOWS
 #define GLEW_STATIC
+#endif
 #include <GL/glew.h>
 
 #include "panoinc_WX.h"
@@ -65,7 +67,7 @@ extern "C" {
 #include "PreviewDifferenceTool.h"
 #include "PreviewPanoMaskTool.h"
 #include "PreviewControlPointTool.h"
-#include "NumTransDialog.h"
+#include "PreviewLayoutLinesTool.h"
 
 #include <wx/progdlg.h>
 
@@ -77,15 +79,19 @@ enum {
     PROJ_PARAM_NAMES_ID = wxID_HIGHEST+1300,
     PROJ_PARAM_VAL_ID = wxID_HIGHEST+1400,
     PROJ_PARAM_SLIDER_ID = wxID_HIGHEST+1500,
+    PROJ_PARAM_RESET_ID = wxID_HIGHEST+1550,
     ID_TOGGLE_BUT_LEAVE = wxID_HIGHEST+1600,
     ID_FULL_SCREEN = wxID_HIGHEST+1710,
     ID_SHOW_ALL = wxID_HIGHEST+1711,
-    ID_SHOW_NONE = wxID_HIGHEST+1712
+    ID_SHOW_NONE = wxID_HIGHEST+1712,
+    ID_UNDO = wxID_HIGHEST+1713,
+    ID_REDO = wxID_HIGHEST+1714
 };
 
 /** enum, which contains all different toolbar modes */
 enum{
     mode_preview=0,
+    mode_layout,
     mode_projection,
     mode_drag,
     mode_crop
@@ -111,13 +117,18 @@ BEGIN_EVENT_TABLE(GLPreviewFrame, wxFrame)
     EVT_SPIN_DOWN(XRCID("exposure_spin"), GLPreviewFrame::OnDecreaseExposure)
     EVT_SPIN_UP(XRCID("exposure_spin"), GLPreviewFrame::OnIncreaseExposure)
     EVT_CHOICE(XRCID("blend_mode_choice"), GLPreviewFrame::OnBlendChoice)
+    EVT_CHOICE(XRCID("drag_mode_choice"), GLPreviewFrame::OnDragChoice)
     EVT_CHOICE(XRCID("projection_choice"), GLPreviewFrame::OnProjectionChoice)
 #ifndef __WXMAC__
     // wxMac does not process these
     EVT_SCROLL_CHANGED(GLPreviewFrame::OnChangeFOV)
+    EVT_COMMAND_SCROLL(XRCID("layout_scale_slider"), GLPreviewFrame::OnLayoutScaleChange)
 #else
     EVT_SCROLL_THUMBRELEASE(GLPreviewFrame::OnChangeFOV)
     EVT_SCROLL_ENDSCROLL(GLPreviewFrame::OnChangeFOV)
+    EVT_COMMAND_SCROLL_THUMBRELEASE(XRCID("layout_scale_slider"), GLPreviewFrame::OnLayoutScaleChange)
+    EVT_COMMAND_SCROLL_ENDSCROLL(XRCID("layout_scale_slider"), GLPreviewFrame::OnLayoutScaleChange)
+    EVT_COMMAND_SCROLL_THUMBTRACK(XRCID("layout_scale_slider"), GLPreviewFrame::OnLayoutScaleChange)
 #endif
     EVT_SCROLL_THUMBTRACK(GLPreviewFrame::OnTrackChangeFOV)
     EVT_TEXT_ENTER(XRCID("pano_text_hfov"), GLPreviewFrame::OnHFOVChanged )
@@ -128,7 +139,10 @@ BEGIN_EVENT_TABLE(GLPreviewFrame, wxFrame)
     EVT_TEXT_ENTER(XRCID("pano_val_roi_bottom"), GLPreviewFrame::OnROIChanged)
     EVT_TEXT_ENTER(XRCID("exposure_text"), GLPreviewFrame::OnExposureChanged)
     EVT_COMMAND_RANGE(PROJ_PARAM_VAL_ID,PROJ_PARAM_VAL_ID+PANO_PROJECTION_MAX_PARMS,wxEVT_COMMAND_TEXT_ENTER,GLPreviewFrame::OnProjParameterChanged)
+    EVT_BUTTON(PROJ_PARAM_RESET_ID, GLPreviewFrame::OnProjParameterReset)
     EVT_TOOL(ID_FULL_SCREEN, GLPreviewFrame::OnFullScreen)
+    EVT_TOOL(ID_UNDO, GLPreviewFrame::OnUndo)
+    EVT_TOOL(ID_REDO, GLPreviewFrame::OnRedo)
 END_EVENT_TABLE()
 
 BEGIN_EVENT_TABLE(ImageToogleButtonEventHandler, wxEvtHandler)
@@ -348,6 +362,11 @@ GLPreviewFrame::GLPreviewFrame(wxFrame * frame, PT::Panorama &pano)
     m_BlendModeChoice->Append(_("normal"));
     m_BlendModeChoice->SetSelection(0);
 
+	m_DragModeChoice = XRCCTRL(*this, "drag_mode_choice", wxChoice);
+	m_DragModeChoice->Append(_("normal"));
+	m_DragModeChoice->Append(_("mosaic"));
+	m_DragModeChoice->SetSelection(0);
+
     // TODO implement hdr display in OpenGL, if possible?
     // Disabled until someone can figure out HDR display in OpenGL.
     /*
@@ -381,6 +400,11 @@ GLPreviewFrame::GLPreviewFrame(wxFrame * frame, PT::Panorama &pano)
     m_projection_panel = XRCCTRL(*this, "projection_panel", wxPanel);
     m_projParamSizer = new wxBoxSizer(wxHORIZONTAL);
 
+    wxBitmapButton * resetProjButton=new wxBitmapButton(m_projection_panel, PROJ_PARAM_RESET_ID, 
+        wxArtProvider::GetBitmap(wxART_REDO));
+    resetProjButton->SetToolTip(_("Resets the projections parameters to their default values."));
+    m_projParamSizer->Add(resetProjButton, 0, wxLEFT | wxRIGHT | wxALIGN_CENTER_VERTICAL, 5);
+
     m_projParamNamesLabel.resize(PANO_PROJECTION_MAX_PARMS);
     m_projParamTextCtrl.resize(PANO_PROJECTION_MAX_PARMS);
     m_projParamSlider.resize(PANO_PROJECTION_MAX_PARMS);
@@ -393,7 +417,7 @@ GLPreviewFrame::GLPreviewFrame(wxFrame * frame, PT::Panorama &pano)
                         wxLEFT | wxRIGHT | wxALIGN_CENTER_VERTICAL, // draw border all around
                         5);       // border width
         m_projParamTextCtrl[i] = new wxTextCtrl(m_projection_panel, PROJ_PARAM_VAL_ID+i, _("0"),
-                                    wxDefaultPosition,wxDefaultSize, wxTE_PROCESS_ENTER);
+                                    wxDefaultPosition, wxSize(35,-1), wxTE_PROCESS_ENTER);
         m_projParamSizer->Add(m_projParamTextCtrl[i],
                         0,        // not vertically strechable
                         wxLEFT | wxRIGHT | wxALIGN_CENTER_VERTICAL, // draw border all around
@@ -437,9 +461,11 @@ GLPreviewFrame::GLPreviewFrame(wxFrame * frame, PT::Panorama &pano)
     if (config->Read(wxT("/GLPreviewFrame/isShown"), 0l) != 0) {
         Show();
     }
-    wxAcceleratorEntry entries[1];
+    wxAcceleratorEntry entries[3];
     entries[0].Set(wxACCEL_NORMAL,WXK_F11,ID_FULL_SCREEN);
-    wxAcceleratorTable accel(1, entries);
+    entries[1].Set(wxACCEL_CTRL,(int)'Z',ID_UNDO);
+    entries[2].Set(wxACCEL_CTRL,(int)'Y',ID_REDO);
+    wxAcceleratorTable accel(3, entries);
     SetAcceleratorTable(accel);
 #ifdef __WXGTK__
     // set explicit focus to button panel, otherwise the hotkey F11 is not right processed
@@ -607,8 +633,7 @@ void GLPreviewFrame::panoramaChanged(Panorama &pano)
     
     // Check if autocrop is usable on this projection.
     bool hasActiveImages = pano.getActiveImages().size() > 0;
-    XRCCTRL(*this,"preview_autocrop_tool",wxBitmapButton)->Enable(
-        hasActiveImages && opts.fovCalcSupported(opts.getProjection()));
+    XRCCTRL(*this,"preview_autocrop_tool",wxBitmapButton)->Enable(hasActiveImages);
     m_ROILeftTxt->SetValue(wxString::Format(wxT("%d"), opts.getROI().left() ));
     m_ROIRightTxt->SetValue(wxString::Format(wxT("%d"), opts.getROI().right() ));
     m_ROITopTxt->SetValue(wxString::Format(wxT("%d"), opts.getROI().top() ));
@@ -904,6 +929,15 @@ void GLPreviewFrame::OnProjParameterChanged(wxCommandEvent & e)
                                            );
 }
 
+void GLPreviewFrame::OnProjParameterReset(wxCommandEvent &e)
+{
+    PanoramaOptions opts=m_pano.getOptions();
+    opts.resetProjectionParameters();
+    GlobalCmdHist::getInstance().addCommand(
+        new PT::SetPanoOptionsCmd(m_pano, opts)
+        );
+};
+
 void GLPreviewFrame::OnChangeFOV(wxScrollEvent & e)
 {
     DEBUG_TRACE("");
@@ -916,6 +950,9 @@ void GLPreviewFrame::OnChangeFOV(wxScrollEvent & e)
     } else if (e.GetEventObject() == m_VFOVSlider) {
         DEBUG_DEBUG("VFOV changed (slider): " << e.GetInt());
         opt.setVFOV(e.GetInt());
+    } else if (e.GetEventObject() == XRCCTRL(*this,"layout_scale_slider",wxSlider)) {
+        DEBUG_DEBUG("Layout scale changed (slider): " << e.GetInt());
+        GLPreviewFrame::OnLayoutScaleChange(e);
     } else {
         int nParam = opt.m_projFeatures.numberOfParameters;
         std::vector<double> para = opt.getProjectionParameters();
@@ -976,6 +1013,35 @@ void GLPreviewFrame::OnBlendChoice(wxCommandEvent & e)
     }
 }
 
+void GLPreviewFrame::OnDragChoice(wxCommandEvent & e)
+{
+    if (e.GetEventObject() == m_DragModeChoice)
+    {
+        if (drag_tool) {
+        
+		    int index = m_DragModeChoice->GetSelection();
+		    switch (index) {
+		    	case 0: //normal
+		    		drag_tool->setDragMode(PreviewDragTool::drag_mode_normal);
+		    	break; 
+		    	case 1: //mosaic
+		    		drag_tool->setDragMode(PreviewDragTool::drag_mode_mosaic);
+		    	break;
+		    }
+            XRCCTRL(*this,"label_yaw",wxStaticText)->Enable(index==0);
+            XRCCTRL(*this,"input_yaw",wxTextCtrl)->Enable(index==0);
+            XRCCTRL(*this,"label_pitch",wxStaticText)->Enable(index==0);
+            XRCCTRL(*this,"input_pitch",wxTextCtrl)->Enable(index==0);
+            XRCCTRL(*this,"label_roll",wxStaticText)->Enable(index==0);
+            XRCCTRL(*this,"input_roll",wxTextCtrl)->Enable(index==0);
+            XRCCTRL(*this,"apply_num_transform",wxButton)->Enable(index==0);
+        }
+    }
+    else
+    {
+        // FIXME DEBUG_WARN("wxChoice event from unknown object received");
+    }
+}
 
 void GLPreviewFrame::OnDefaultExposure( wxCommandEvent & e )
 {
@@ -1104,6 +1170,7 @@ void GLPreviewFrame::MakeTools(PreviewToolHelper *helper_in)
     difference_tool = new PreviewDifferenceTool(helper);
     pano_mask_tool = new PreviewPanoMaskTool(helper);
     control_point_tool = new PreviewControlPointTool(helper);
+    m_layoutLinesTool = new PreviewLayoutLinesTool(helper);
     
     // activate tools that are always active.
     helper->ActivateTool(pano_mask_tool);
@@ -1315,11 +1382,11 @@ void GLPreviewFrame::OnAutocrop(wxCommandEvent &e)
 {
     DEBUG_INFO("Dirty ROI Calc\n");
     if (m_pano.getActiveImages().size() == 0) return;
-    
+
     ProgressReporterDialog progress(2, _("Autocrop"), _("Calculating optimal crop"),this, wxPD_AUTO_HIDE | wxPD_APP_MODAL | wxPD_ELAPSED_TIME);
     progress.increaseProgress(1);
     progress.Pulse();
-
+    
     vigra::Rect2D newROI;
     vigra::Size2D newSize;
     
@@ -1353,6 +1420,18 @@ void GLPreviewFrame::OnFullScreen(wxCommandEvent & e)
     ShowFullScreen(!IsFullScreen(), wxFULLSCREEN_NOBORDER | wxFULLSCREEN_NOCAPTION);
 };
 
+void GLPreviewFrame::OnUndo(wxCommandEvent &e)
+{
+    wxCommandEvent dummy(wxEVT_COMMAND_MENU_SELECTED, XRCID("ID_EDITUNDO"));
+    m_parent->AddPendingEvent(dummy);
+};
+
+void GLPreviewFrame::OnRedo(wxCommandEvent &e)
+{
+    wxCommandEvent dummy(wxEVT_COMMAND_MENU_SELECTED, XRCID("ID_EDITREDO"));
+    m_parent->AddPendingEvent(dummy);
+};
+
 void GLPreviewFrame::SetMode(int newMode)
 {
     if(m_mode==newMode)
@@ -1368,6 +1447,16 @@ void GLPreviewFrame::SetMode(int newMode)
             helper->DeactivateTool(control_point_tool);
             XRCCTRL(*this,"preview_control_point_tool",wxCheckBox)->SetValue(false);
             break;
+        case mode_layout:
+            // disable layout mode.
+            helper->DeactivateTool(m_layoutLinesTool);
+            m_GLViewer->SetLayoutMode(false);
+            // Switch the panorama mask back on.
+            helper->ActivateTool(pano_mask_tool);
+            //restore blend mode
+            m_BlendModeChoice->SetSelection(non_layout_blend_mode);
+            updateBlendMode();
+            break;
         case mode_projection:
             break;
         case mode_drag:
@@ -1378,10 +1467,25 @@ void GLPreviewFrame::SetMode(int newMode)
             break;
     };
     m_mode=newMode;
+    wxScrollEvent dummy;
     switch(m_mode)
     {
         case mode_preview:
             break;
+        case mode_layout:
+            //save blend mode setting, set to normal for layout mode
+            non_layout_blend_mode=m_BlendModeChoice->GetSelection();
+            m_BlendModeChoice->SetSelection(0);
+            updateBlendMode();
+            // turn off things not used in layout mode.
+            helper->DeactivateTool(pano_mask_tool);
+            m_GLViewer->SetLayoutMode(true);
+            helper->ActivateTool(m_layoutLinesTool);
+            // we need to update the meshes after switch to layout mode
+            // otherwise the following update of scale has no meshes to scale
+            m_GLViewer->Update();
+            OnLayoutScaleChange(dummy);
+            break;
         case mode_projection:
             break;
         case mode_drag:
@@ -1511,3 +1615,13 @@ void GLPreviewFrame::OnVFOVChanged ( wxCommandEvent & e )
 
     DEBUG_INFO ( "new vfov: " << vfov )
 };
+
+void GLPreviewFrame::OnLayoutScaleChange(wxScrollEvent &e)
+{
+    if(m_mode==mode_layout)
+    {
+        double scale_factor=XRCCTRL(*this,"layout_scale_slider",wxSlider)->GetValue();
+        m_GLViewer->SetLayoutScale(10.0-sqrt(scale_factor));
+        m_GLViewer->Refresh();
+    };
+};
