@@ -46,6 +46,7 @@ class PreviewIdentifyTool;
 class PreviewDifferenceTool;
 class PreviewPanoMaskTool;
 class PreviewControlPointTool;
+class PreviewLayoutLinesTool;
 
 #include "common/utils.h"
 #include <wx/string.h>
@@ -58,8 +59,7 @@ class ImageToogleButtonEventHandler : public wxEvtHandler
 public:
     ImageToogleButtonEventHandler(unsigned int image_number,
                                   PreviewIdentifyTool **identify_tool,
-                                  unsigned int identify_tool_id,
-                                  wxToolBar *tool_bar,
+                                  wxToolBarToolBase* identify_toolbutton_in,
                                   PT::Panorama * m_pano);
     void OnChange(wxCommandEvent &e);
 protected:
@@ -69,8 +69,7 @@ private:
     DECLARE_EVENT_TABLE()
     unsigned int image_number;
     PreviewIdentifyTool **identify_tool;
-    unsigned int identify_tool_id;
-    wxToolBar *tool_bar;
+    wxToolBarToolBase *identify_toolbutton;
     PT::Panorama * m_pano;
 };
 
@@ -107,44 +106,74 @@ public:
     void FillBlendChoice();
 protected:
     void OnClose(wxCloseEvent& e);
-
+    
     void OnCenterHorizontally(wxCommandEvent & e);
     void OnFitPano(wxCommandEvent& e);
     void OnStraighten(wxCommandEvent & e);
     void OnShowAll(wxCommandEvent & e);
     void OnShowNone(wxCommandEvent & e);
     void OnPhotometric(wxCommandEvent & e);
-    void OnCrop(wxCommandEvent & e);
-    void OnDrag(wxCommandEvent & e);
     void OnIdentify(wxCommandEvent &e);
+    void OnAutocrop(wxCommandEvent &e);
     void OnControlPoint(wxCommandEvent &e);
     void OnNumTransform(wxCommandEvent & e);
     void OnChangeFOV(wxScrollEvent & e);
     void OnTrackChangeFOV(wxScrollEvent & e);
-    void OnTextCtrlChanged(wxCommandEvent & e);
+    void OnExposureChanged(wxCommandEvent & e);
+    void OnProjParameterChanged(wxCommandEvent & e);
+    /** event handler for reset projection parameters */
+    void OnProjParameterReset(wxCommandEvent & e);
 
     void OnDefaultExposure( wxCommandEvent & e );
     void OnDecreaseExposure( wxSpinEvent & e );
     void OnIncreaseExposure( wxSpinEvent & e );
 
     void OnBlendChoice(wxCommandEvent & e);
+    void OnDragChoice(wxCommandEvent & e);
     void OnProjectionChoice(wxCommandEvent & e);
+    /** event handler for changed roi */
+    void OnROIChanged(wxCommandEvent & e);
+    void OnHFOVChanged(wxCommandEvent & e);
+    void OnVFOVChanged(wxCommandEvent & e);
     // No HDR display yet
     // void OnOutputChoice(wxCommandEvent & e);
     // update tools according to blend mode choice
     void updateBlendMode();
     // update the panorama display
     void updatePano();
+    /** event handler for full screen */
+    void OnFullScreen(wxCommandEvent &e);
+    /** event handler for undo */
+    void OnUndo(wxCommandEvent &e);
+    /** event handler for redo */
+    void OnRedo(wxCommandEvent &e);
+    /** event handler for selection of new mode */
+    void OnSelectMode(wxNotebookEvent &e);
+    /** event handler for blocking changing mode when panorama contains no images*/
+    void OnToolModeChanging(wxNotebookEvent &e);
+    /** event handler for change scale of layout mode */
+    void OnLayoutScaleChange(wxScrollEvent &e);
 private:
 
+    void SetMode(int newMode);
     PT::Panorama & m_pano;
 
     GLViewer * m_GLViewer;
-    wxToolBar * m_ToolBar;
-    int drag_tool_id, crop_tool_id, identify_tool_id, control_point_tool_id;
+    int m_mode;
+    int non_layout_blend_mode;
+    wxToolBar* m_ToolBar_Identify;
+    wxNotebook* m_tool_notebook;
+    wxPanel* m_projection_panel;
     wxSlider * m_HFOVSlider;
     wxSlider * m_VFOVSlider;
+    wxTextCtrl * m_HFOVText;
+    wxTextCtrl * m_VFOVText;
+    wxTextCtrl * m_ROILeftTxt;
+    wxTextCtrl * m_ROIRightTxt;
+    wxTextCtrl * m_ROITopTxt;
+    wxTextCtrl * m_ROIBottomTxt;
     wxChoice * m_BlendModeChoice;
+    wxChoice * m_DragModeChoice;
     wxChoice * m_ProjectionChoice;
     // No HDR display yet.
     // wxChoice * m_outputModeChoice;
@@ -162,7 +191,7 @@ private:
 	  wxStaticBoxSizer * m_ToggleButtonSizer;
 
     wxBoxSizer * m_topsizer;
-    wxStaticBoxSizer * m_projParamSizer;
+    wxBoxSizer * m_projParamSizer;
     std::vector<wxStaticText *> m_projParamNamesLabel;
     std::vector<wxTextCtrl *>   m_projParamTextCtrl;
     std::vector<wxSlider *>     m_projParamSlider;
@@ -184,6 +213,7 @@ private:
     PreviewDifferenceTool *difference_tool;
     PreviewControlPointTool *control_point_tool;
     PreviewPanoMaskTool *pano_mask_tool;    
+    PreviewLayoutLinesTool *m_layoutLinesTool;
     void TurnOffTools(std::set<PreviewTool*> tools);
     void CleanButtonColours();
 };
