@@ -6,7 +6,7 @@
  *
  *  @author Kai-Uwe Behrmann <web@tiscali.de>
  *
- *  $Id: ImagesList.cpp 4798 2009-12-14 13:31:41Z ade_br $
+ *  $Id$
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -89,6 +89,11 @@ void ImagesList::Init(PT::Panorama * panorama)
 {
     pano = panorama;
     pano->addObserver(this);
+    
+    /// @todo check new didn't return NULL in non-debug builds.
+    variable_groups = new HuginBase::StandardImageVariableGroups(*pano);
+    DEBUG_ASSERT(variable_groups);
+    
 #ifdef __WXMAC__
     SetDropTarget(new PanoDropTarget(*pano, true));
 #endif
@@ -98,6 +103,7 @@ ImagesList::~ImagesList(void)
 {
     DEBUG_TRACE("");
     pano->removeObserver(this);
+    delete variable_groups;
 }
 
 void ImagesList::panoramaImagesChanged(Panorama &pano, const UIntSet &changed)
@@ -122,6 +128,9 @@ void ImagesList::panoramaImagesChanged(Panorama &pano, const UIntSet &changed)
         RemoveItem(i);
         //m_smallIcons->Remove(i);
     }
+    
+    // Make sure the part numbers are up to date before writing them to the table.
+    variable_groups->update();
 
     // update existing items
 //    if ( nrImages >= nrItems ) {
@@ -148,6 +157,13 @@ void ImagesList::panoramaImagesChanged(Panorama &pano, const UIntSet &changed)
         }
 //    }
     
+    // Update the part numbers. We do this for every image, as the images that
+    // have changed may affect the part numbers for images that haven't changed.
+    for (unsigned int i = 0; i < pano.getNrOfImages(); i++)
+    {
+        UpdatePartNumbersForItem(i);
+    }
+    
     Thaw();
     m_notifyParents = true;
     
@@ -304,6 +320,11 @@ void ImagesList::UpdateItem(unsigned int imgNr)
 
 }
 
+void ImagesList::UpdatePartNumbersForItem(unsigned int imgNr)
+{
+
+}
+
 IMPLEMENT_DYNAMIC_CLASS(ImagesList, wxListCtrl)
 
 // ============================================================================
@@ -335,8 +356,12 @@ bool ImagesListImage::Create(wxWindow* parent, wxWindowID id,
     InsertColumn( 4, _("yaw (y)"), wxLIST_FORMAT_RIGHT, 60 );
     InsertColumn( 5, _("pitch (p)"), wxLIST_FORMAT_RIGHT, 60 );
     InsertColumn( 6, _("roll (r)"), wxLIST_FORMAT_RIGHT, 60 );
-    InsertColumn( 7, _("Anchor"), wxLIST_FORMAT_RIGHT, 60 );
-    InsertColumn( 8, _("# Ctrl Pnts"), wxLIST_FORMAT_RIGHT, 60);
+    InsertColumn( 7, _("X (TrX)"), wxLIST_FORMAT_RIGHT, 60 );
+    InsertColumn( 8, _("Y (TrY)"), wxLIST_FORMAT_RIGHT, 60 );
+    InsertColumn( 9, _("Z (TrZ)"), wxLIST_FORMAT_RIGHT, 60 );
+    InsertColumn( 10, _("Anchor"), wxLIST_FORMAT_RIGHT, 60 );
+    InsertColumn( 11, _("# Ctrl Pnts"), wxLIST_FORMAT_RIGHT, 60);
+    InsertColumn( 12, _("Stack Number"), wxLIST_FORMAT_RIGHT, 60);
     
     //get saved width
     for ( int j=0; j < GetColumnCount() ; j++ )
@@ -358,18 +383,20 @@ void ImagesListImage::UpdateItem(unsigned int imgNr)
 {
     DEBUG_DEBUG("update image list item " << imgNr);
     DEBUG_ASSERT((int)imgNr < GetItemCount());
-    const PanoImage & img = pano->getImage(imgNr);
+    const SrcPanoImage & img = pano->getImage(imgNr);
     wxFileName fn(wxString (img.getFilename().c_str(), HUGIN_CONV_FILENAME));
-    VariableMap var = pano->getImageVariables(imgNr);
 
 //    wxLogMessage(wxString::Format(_("updating image list item %d, filename %s"),imgNr, fn.GetFullName()));
 
     SetItem(imgNr, 1, fn.GetFullName() );
-    SetItem(imgNr, 2, wxString::Format(wxT("%d"), img.getWidth()));
-    SetItem(imgNr, 3, wxString::Format(wxT("%d"), img.getHeight()));
-    SetItem(imgNr, 4, doubleTowxString(map_get(var,"y").getValue(),m_degDigits));
-    SetItem(imgNr, 5, doubleTowxString( map_get(var,"p").getValue(),m_degDigits));
-    SetItem(imgNr, 6, doubleTowxString( map_get(var,"r").getValue(),m_degDigits));
+    SetItem(imgNr, 2, wxString::Format(wxT("%d"), img.getSize().width()));
+    SetItem(imgNr, 3, wxString::Format(wxT("%d"), img.getSize().height()));
+    SetItem(imgNr, 4, doubleTowxString(img.getYaw(), m_degDigits));
+    SetItem(imgNr, 5, doubleTowxString(img.getPitch(), m_degDigits));
+    SetItem(imgNr, 6, doubleTowxString(img.getRoll(), m_degDigits));
+    SetItem(imgNr, 7, doubleTowxString(img.getX(), m_distDigits));
+    SetItem(imgNr, 8, doubleTowxString(img.getY(), m_distDigits));
+    SetItem(imgNr, 9, doubleTowxString(img.getZ(), m_distDigits));
     wxChar flags[] = wxT("--");
     if (pano->getOptions().optimizeReferenceImage == imgNr) {
         flags[0]='A';
@@ -377,7 +404,7 @@ void ImagesListImage::UpdateItem(unsigned int imgNr)
     if (pano->getOptions().colorReferenceImage == imgNr) {
         flags[1]='C';
     }
-    SetItem(imgNr,7, wxString(flags, wxConvLocal));
+    SetItem(imgNr,10, wxString(flags, wxConvLocal));
     // urgh.. slow.. stupid.. traverse control point list for each image..
     const CPVector & cps = pano->getCtrlPoints();
     int nCP=0;
@@ -386,7 +413,13 @@ void ImagesListImage::UpdateItem(unsigned int imgNr)
             nCP++;
         }
     }
-    SetItem(imgNr, 8, wxString::Format(wxT("%d"), nCP));
+    SetItem(imgNr, 11, wxString::Format(wxT("%d"), nCP));
+}
+
+void ImagesListImage::UpdatePartNumbersForItem(unsigned int imgNr)
+{
+    unsigned int stackNumber = variable_groups->getStacks().getPartNumber(imgNr);
+    SetItem(imgNr, 12, wxString::Format(wxT("%d"), stackNumber));
 }
 
 IMPLEMENT_DYNAMIC_CLASS(ImagesListImage, ImagesList)
@@ -476,20 +509,22 @@ void ImagesListLens::Init(PT::Panorama * panorama)
 
 void ImagesListLens::UpdateItem(unsigned int imgNr)
 {
-    const PanoImage & img = pano->getImage(imgNr);
+    const SrcPanoImage & img = pano->getImage(imgNr);
     wxFileName fn(wxString (img.getFilename().c_str(), HUGIN_CONV_FILENAME));
     SetItem(imgNr, 1, fn.GetFullName() );
-    SetItem(imgNr, 2, wxString::Format(wxT("%d"),img.getLensNr()));
+    SetItem(imgNr, 2, wxString::Format(wxT("%d"),variable_groups->getLenses().getPartNumber(imgNr)));
 
     VariableMap var = pano->getImageVariables(imgNr);
-    const Lens & lens = pano->getLens( img.getLensNr());
     wxString ps;
-    switch ( (int) lens.getProjection() ) {
-    case Lens::RECTILINEAR:          ps << _("Normal (rectilinear)"); break;
-    case Lens::PANORAMIC:            ps << _("Panoramic (cylindrical)"); break;
-    case Lens::CIRCULAR_FISHEYE:     ps << _("Circular fisheye"); break;
-    case Lens::FULL_FRAME_FISHEYE:   ps << _("Full frame fisheye"); break;
-    case Lens::EQUIRECTANGULAR:      ps << _("Equirectangular"); break;
+    switch ( img.getProjection() ) {
+    case SrcPanoImage::RECTILINEAR:          ps << _("Normal (rectilinear)"); break;
+    case SrcPanoImage::PANORAMIC:            ps << _("Panoramic (cylindrical)"); break;
+    case SrcPanoImage::CIRCULAR_FISHEYE:     ps << _("Circular fisheye"); break;
+    case SrcPanoImage::FULL_FRAME_FISHEYE:   ps << _("Full frame fisheye"); break;
+    case SrcPanoImage::EQUIRECTANGULAR:      ps << _("Equirectangular"); break;
+    case SrcPanoImage::FISHEYE_ORTHOGRAPHIC: ps << _("Orthographic"); break;
+    case SrcPanoImage::FISHEYE_STEREOGRAPHIC:ps << _("Stereographic"); break;
+    case SrcPanoImage::FISHEYE_EQUISOLID:    ps << _("Equisolid"); break;
     }
     SetItem(imgNr, 3, ps);
     SetItem(imgNr, 4, doubleTowxString( map_get(var, "Eev").getValue(),1));
@@ -503,6 +538,12 @@ void ImagesListLens::UpdateItem(unsigned int imgNr)
     SetItem(imgNr, 12, doubleTowxString( map_get(var, "t").getValue(),m_distDigits));
 }
 
+void ImagesListLens::UpdatePartNumbersForItem(unsigned int imgNr)
+{
+    // Update the part numbers.
+    SetItem(imgNr, 2, wxString::Format(wxT("%d"),variable_groups->getLenses().getPartNumber(imgNr)));
+}
+
 
 IMPLEMENT_DYNAMIC_CLASS(ImagesListLens, ImagesList)
 
@@ -579,7 +620,7 @@ void ImagesListCrop::Init(PT::Panorama * panorama)
 
 void ImagesListCrop::UpdateItem(unsigned int imgNr)
 {
-    const PanoImage & img = pano->getImage(imgNr);
+    const SrcPanoImage & img = pano->getImage(imgNr);
     wxFileName fn(wxString (img.getFilename().c_str(), HUGIN_CONV_FILENAME));
 
     wxString cropstr(wxT("-"));
@@ -620,3 +661,84 @@ bool ImagesListCropXmlHandler::CanHandle(wxXmlNode *node)
 
 IMPLEMENT_DYNAMIC_CLASS(ImagesListCropXmlHandler, wxXmlResourceHandler)
 
+
+// ============================================================================
+// ============================================================================
+// Mask
+
+ImagesListMask::ImagesListMask()
+{
+}
+
+
+bool ImagesListMask::Create(wxWindow* parent, wxWindowID id,
+                             const wxPoint& pos,
+                             const wxSize& size,
+                             long style,
+                             const wxString& name)
+{
+    DEBUG_TRACE("");
+    if (!ImagesList::Create(parent, id, pos, size, wxLC_SINGLE_SEL | style, name))
+        return false;
+
+    m_configClassName = wxT("/ImagesListMask");
+
+    InsertColumn(1, _("Number of masks"), wxLIST_FORMAT_RIGHT,120);
+
+    //get saved width
+    for ( int j=0; j < GetColumnCount() ; j++ )
+    {
+        // -1 is auto
+        int width = wxConfigBase::Get()->Read(wxString::Format(m_configClassName+wxT("/ColumnWidth%d"), j ), -1);
+        if(width != -1)
+            SetColumnWidth(j, width);
+    }
+    return true;
+}
+
+
+void ImagesListMask::Init(PT::Panorama * panorama)
+{
+    ImagesList::Init(panorama);
+}
+
+void ImagesListMask::UpdateItem(unsigned int imgNr)
+{
+    wxString maskstr;
+    if(pano->getImage(imgNr).hasMasks())
+        maskstr=wxString::Format(wxT("%d"), pano->getImage(imgNr).getMasks().size());
+    else
+        maskstr=wxT("-");
+    SetItem(imgNr, 1, maskstr);
+}
+
+IMPLEMENT_DYNAMIC_CLASS(ImagesListMask, ImagesList)
+
+ImagesListMaskXmlHandler::ImagesListMaskXmlHandler()
+    : wxXmlResourceHandler()
+{
+    AddWindowStyles();
+}
+
+wxObject *ImagesListMaskXmlHandler::DoCreateResource()
+{
+    XRC_MAKE_INSTANCE(cp, ImagesListMask)
+
+    cp->Create(m_parentAsWindow,
+               GetID(),
+               GetPosition(), GetSize(),
+               GetStyle(wxT("style")),
+               GetName());
+
+    SetupWindow( cp);
+
+    return cp;
+}
+
+bool ImagesListMaskXmlHandler::CanHandle(wxXmlNode *node)
+{
+    return IsOfClass(node, wxT("ImagesListMask"));
+}
+
+IMPLEMENT_DYNAMIC_CLASS(ImagesListMaskXmlHandler, wxXmlResourceHandler)
+
