@@ -6,7 +6,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: MainFrame.cpp 4315 2009-09-03 01:00:39Z yuv $
+ *  $Id$
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -48,6 +48,7 @@
 #include "hugin/ImagesPanel.h"
 #include "hugin/LensPanel.h"
 #include "hugin/CropPanel.h"
+#include "hugin/MaskEditorPanel.h"
 #include "hugin/OptimizePanel.h"
 #include "hugin/OptimizePhotometricPanel.h"
 #include "hugin/PreviewFrame.h"
@@ -60,10 +61,11 @@
 
 #include "base_wx/MyProgressDialog.h"
 #include "base_wx/ImageCache.h"
+#include "base_wx/PTWXDlg.h"
 
 #include "base_wx/huginConfig.h"
 
-
+#include "hugin/AboutDialog.h"
 
 using namespace PT;
 using namespace utils;
@@ -142,8 +144,10 @@ BEGIN_EVENT_TABLE(MainFrame, wxFrame)
     EVT_MENU(XRCID("action_save_as_ptstitcher"),  MainFrame::OnSavePTStitcherAs)
 	EVT_MENU(XRCID("action_send_to_batch"),  MainFrame::OnSendToBatch)
 	EVT_MENU(XRCID("action_open_batch_processor"),  MainFrame::OnOpenPTBatcher)
+    EVT_MENU(XRCID("action_import_project"), MainFrame::OnMergeProject)
     EVT_MENU(XRCID("action_apply_template"),  MainFrame::OnApplyTemplate)
     EVT_MENU(XRCID("action_exit_hugin"),  MainFrame::OnUserQuit)
+    EVT_MENU_RANGE(wxID_FILE1, wxID_FILE9, MainFrame::OnMRUFiles)
     EVT_MENU(XRCID("action_show_about"),  MainFrame::OnAbout)
     EVT_MENU(XRCID("action_show_help"),  MainFrame::OnHelp)
     EVT_MENU(XRCID("action_show_tip"),  MainFrame::OnTipOfDay)
@@ -153,6 +157,7 @@ BEGIN_EVENT_TABLE(MainFrame, wxFrame)
     EVT_MENU(XRCID("action_show_prefs"), MainFrame::OnShowPrefs)
     EVT_MENU(XRCID("ID_EDITUNDO"), MainFrame::OnUndo)
     EVT_MENU(XRCID("ID_EDITREDO"), MainFrame::OnRedo)
+    EVT_MENU(XRCID("ID_SHOW_FULL_SCREEN"), MainFrame::OnFullScreen)
     EVT_MENU(XRCID("ID_SHOW_PREVIEW_FRAME"), MainFrame::OnTogglePreviewFrame)
     EVT_MENU(XRCID("ID_SHOW_GL_PREVIEW_FRAME"), MainFrame::OnToggleGLPreviewFrame)
     EVT_BUTTON(XRCID("ID_SHOW_PREVIEW_FRAME"),MainFrame::OnTogglePreviewFrame)
@@ -162,6 +167,7 @@ BEGIN_EVENT_TABLE(MainFrame, wxFrame)
     EVT_BUTTON(XRCID("action_optimize"),  MainFrame::OnOptimize)
     EVT_MENU(XRCID("action_finetune_all_cp"), MainFrame::OnFineTuneAll)
 //    EVT_BUTTON(XRCID("action_finetune_all_cp"), MainFrame::OnFineTuneAll)
+    EVT_MENU(XRCID("action_remove_cp_in_masks"), MainFrame::OnRemoveCPinMasks)
 
     EVT_MENU(XRCID("ID_CP_TABLE"), MainFrame::OnShowCPFrame)
     EVT_BUTTON(XRCID("ID_CP_TABLE"),MainFrame::OnShowCPFrame)
@@ -170,7 +176,6 @@ BEGIN_EVENT_TABLE(MainFrame, wxFrame)
     EVT_BUTTON(XRCID("action_add_images"),  MainFrame::OnAddImages)
     EVT_MENU(XRCID("action_add_time_images"),  MainFrame::OnAddTimeImages)
     EVT_BUTTON(XRCID("action_add_time_images"),  MainFrame::OnAddTimeImages)
-    EVT_MENU(XRCID( "action_edit_text_dialog"),  MainFrame::OnTextEdit)
     //EVT_NOTEBOOK_PAGE_CHANGED(XRCID( "controls_notebook"), MainFrame::UpdatePanels)
 	EVT_CLOSE(  MainFrame::OnExit)
     EVT_SIZE(MainFrame::OnSize)
@@ -183,7 +188,7 @@ END_EVENT_TABLE()
 //WX_DEFINE_ARRAY()
 
 MainFrame::MainFrame(wxWindow* parent, Panorama & pano)
-    : cp_frame(0), pano(pano), m_doRestoreLayout(false), m_help(0)
+    : cp_frame(0), pano(pano), m_doRestoreLayout(false)
 {
     preview_frame = 0;
     m_progressMax = 1;
@@ -209,11 +214,7 @@ MainFrame::MainFrame(wxWindow* parent, Panorama & pano)
             dc.SetTextBackground(*wxWHITE);
             int tw, th;
             wxString version;
-#ifdef HUGIN_DEVELOPMENT_VERSION
-            version.Printf(_("Prerelease %s"),wxString(DISPLAY_VERSION, wxConvLocal).c_str());
-#else
             version.Printf(_("Version %s"),wxString(DISPLAY_VERSION, wxConvLocal).c_str());
-#endif
             dc.GetTextExtent(version, &tw, &th);
             // place text on bitmap.
             dc.DrawText(version, bitmap.GetWidth() - tw - 3, bitmap.GetHeight() - th - 3);
@@ -295,6 +296,11 @@ MainFrame::MainFrame(wxWindow* parent, Panorama & pano)
     assert(crop_panel);
     crop_panel->Init(&pano);
 
+    // the mask panel
+    mask_panel = XRCCTRL(*this, "mask_panel_unknown", MaskEditorPanel);
+    assert(mask_panel);
+    mask_panel->Init(&pano);
+
     // the pano_panel
     DEBUG_TRACE("");
     pano_panel = XRCCTRL(*this, "panorama_panel_unknown", PanoPanel);
@@ -383,6 +389,7 @@ MainFrame::MainFrame(wxWindow* parent, Panorama & pano)
 
     if(splash) {
         splash->Close();
+        delete splash;
     }
     wxYield();
 	
@@ -410,6 +417,11 @@ MainFrame::MainFrame(wxWindow* parent, Panorama & pano)
 #endif
 #endif
 #endif
+    // generate list of most recently uses files and add it to menu
+    wxConfigBase * config=wxConfigBase::Get();
+    m_mruFiles.Load(*config);
+    m_mruFiles.UseMenu(GetMenuBar()->GetMenu(0)->FindItem(XRCID("menu_mru"))->GetSubMenu());
+    m_mruFiles.AddFilesToMenu();
     DEBUG_TRACE("");
 }
 
@@ -429,6 +441,8 @@ MainFrame::~MainFrame()
 
     StoreFramePosition(this, wxT("MainFrame"));
 
+    //store most recently used files
+    m_mruFiles.Save(*config);
     config->Flush();
 
     DEBUG_TRACE("dtor end");
@@ -515,6 +529,7 @@ void MainFrame::OnExit(wxCloseEvent & e)
 void MainFrame::OnSaveProject(wxCommandEvent & e)
 {
     DEBUG_TRACE("");
+    bool savedProjectFile=false;
     try {
     wxFileName scriptName = m_filename;
     if (m_filename == wxT("")) {
@@ -525,6 +540,7 @@ void MainFrame::OnSaveProject(wxCommandEvent & e)
         std::string path = getPathPrefix(std::string(scriptName.GetFullPath().mb_str(HUGIN_CONV_FILENAME)));
         DEBUG_DEBUG("stripping " << path << " from image filenames");
         std::ofstream script(scriptName.GetFullPath().mb_str(HUGIN_CONV_FILENAME));
+        script.exceptions ( std::ofstream::eofbit | std::ofstream::failbit | std::ofstream::badbit );
         PT::OptimizeVector optvec = opt_panel->getOptimizeVector();
         PT::UIntSet all;
         if (pano.getNrOfImages() > 0) {
@@ -532,6 +548,7 @@ void MainFrame::OnSaveProject(wxCommandEvent & e)
         }
         pano.printPanoramaScript(script, optvec, pano.getOptions(), all, false, path);
         script.close();
+        savedProjectFile=true;
 
         int createMakefile = 1;
 #if defined __WXMAC__ && defined MAC_SELF_CONTAINED_BUNDLE
@@ -540,6 +557,7 @@ void MainFrame::OnSaveProject(wxCommandEvent & e)
         if (createMakefile && pano.getNrOfImages() > 0) {
             wxString makefn = scriptName.GetFullPath() + wxT(".mk");
             std::ofstream makefile(makefn.mb_str(HUGIN_CONV_FILENAME));
+            makefile.exceptions ( std::ofstream::eofbit | std::ofstream::failbit | std::ofstream::badbit );
             wxString ptoFnWX = scriptName.GetFullPath();
             std::string ptoFn(ptoFnWX.mb_str(HUGIN_CONV_FILENAME));
             wxString bindir = huginApp::Get()->GetUtilsBinDir();
@@ -562,12 +580,19 @@ void MainFrame::OnSaveProject(wxCommandEvent & e)
                                                               tmpDir);
         }
         SetStatusText(wxString::Format(_("saved project %s"), m_filename.c_str()),0);
-        this->SetTitle(scriptName.GetName() + wxT(".") + scriptName.GetExt() + wxT(" - ") + _("Hugin - Panorama Tools Frontend"));
+        this->SetTitle(scriptName.GetName() + wxT(".") + scriptName.GetExt() + wxT(" - ") + _("Hugin - Panorama Stitcher"));
         pano.clearDirty();
     }
     } catch (std::exception & e) {
         wxString err(e.what(), wxConvLocal);
-        wxMessageBox(err, wxT("Exception during saving project file"));
+        if(savedProjectFile)
+        {
+            wxMessageBox(wxString::Format(_("Could not save project makefile \"%s\".\nBut the project file was saved.\nMaybe the file or the folder is read-only.\n\n(Error code: %s)"),(m_filename+wxT(".mk")).c_str(),err.c_str()),_("Warning"),wxOK|wxICON_INFORMATION);
+        }
+        else
+        {
+            wxMessageBox(wxString::Format(_("Could not save project file \"%s\".\nMaybe the file or the folder is read-only.\n\n(Error code: %s)"),m_filename.c_str(),err.c_str()),_("Error"),wxOK|wxICON_ERROR);
+        };
     }
 }
 
@@ -598,6 +623,7 @@ void MainFrame::OnSaveProjectAs(wxCommandEvent & e)
             }
         }
         m_filename = fn;
+        m_mruFiles.AddFileToHistory(m_filename);
         OnSaveProject(e);
     }
 }
@@ -647,13 +673,16 @@ void MainFrame::LoadProjectFile(const wxString & filename)
     wxString path = fname.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR);
     if (fname.IsOk() && fname.FileExists()) {
         wxBusyCursor wait;
+        deregisterPTWXDlgFcn();
         GlobalCmdHist::getInstance().addCommand(
            new wxLoadPTProjectCmd(pano,(const char *)filename.mb_str(HUGIN_CONV_FILENAME), (const char *)path.mb_str(HUGIN_CONV_FILENAME))
            );
+        registerPTWXDlgFcn(MainFrame::Get());
         DEBUG_DEBUG("project contains " << pano.getNrOfImages() << " after load");
         opt_panel->setModeCustom();
         SetStatusText(_("Project opened"));
-        this->SetTitle(fname.GetName() + wxT(".") + fname.GetExt() + wxT(" - ") + _("Hugin - Panorama Tools Frontend"));
+        m_mruFiles.AddFileToHistory(fname.GetFullPath());
+        this->SetTitle(fname.GetName() + wxT(".") + fname.GetExt() + wxT(" - ") + _("Hugin - Panorama Stitcher"));
         if (! (fname.GetExt() == wxT("pto"))) {
             // do not remember filename if its not a hugin project
             // to avoid overwriting the original project with an
@@ -737,19 +766,21 @@ void MainFrame::OnNewProject(wxCommandEvent & e)
     GlobalCmdHist::getInstance().addCommand( new NewPanoCmd(pano));
     // remove old images from cache
     ImageCache::getInstance().flush();
-    this->SetTitle(_("Hugin - Panorama Tools Frontend"));
+    this->SetTitle(_("Hugin - Panorama Stitcher"));
     pano.clearDirty();
 
     // Setup pano with options from preferences
     PanoramaOptions opts = pano.getOptions();
     wxConfigBase* config = wxConfigBase::Get();
-    opts.outputFormat = PanoramaOptions::TIFF;
+    opts.outputFormat = PanoramaOptions::TIFF_m;
     opts.blendMode = PanoramaOptions::ENBLEND_BLEND;
     opts.enblendOptions = config->Read(wxT("Enblend/Args"),wxT(HUGIN_ENBLEND_ARGS)).mb_str(wxConvLocal);
     opts.enfuseOptions = config->Read(wxT("Enfuse/Args"),wxT(HUGIN_ENFUSE_ARGS)).mb_str(wxConvLocal);
 	opts.interpolator = (vigra_ext::Interpolator)config->Read(wxT("Nona/Interpolator"),HUGIN_NONA_INTERPOLATOR);
 	opts.remapUsingGPU = (bool)config->Read(wxT("Nona/useGPU"),HUGIN_NONA_USEGPU);
 	opts.tiff_saveROI = (bool)config->Read(wxT("Nona/CroppedImages"),HUGIN_NONA_CROPPEDIMAGES);
+    opts.hdrMergeMode = PanoramaOptions::HDRMERGE_AVERAGE;
+    opts.hdrmergeOptions = HUGIN_HDRMERGE_ARGS;
     pano.setOptions(opts);
 
     wxCommandEvent dummy;
@@ -804,20 +835,31 @@ void MainFrame::OnAddImages( wxCommandEvent& event )
         // e safe the current path to config
         config->Write(wxT("/actualPath"), dlg.GetDirectory());  // remember for later
 
-        std::vector<std::string> filesv;
-        for (unsigned int i=0; i< Pathnames.GetCount(); i++) {
-            filesv.push_back((const char *)Pathnames[i].mb_str(HUGIN_CONV_FILENAME));
+        bool foundForbiddenChars=false;
+        for(unsigned int i=0;i<Pathnames.GetCount(); i++)
+           foundForbiddenChars=foundForbiddenChars || containsInvalidCharacters(Pathnames[i]);
+        if(foundForbiddenChars)
+        {
+            wxMessageBox(wxString::Format(_("The filename(s) contains one of the following invalid characters: %s\nHugin can not work with these filenames. Please rename your file(s) and try again."),getInvalidCharacters().c_str()),
+                _("Error"),wxOK | wxICON_EXCLAMATION,this);
         }
+        else
+        {
+            std::vector<std::string> filesv;
+            for (unsigned int i=0; i< Pathnames.GetCount(); i++) {
+                filesv.push_back((const char *)Pathnames[i].mb_str(HUGIN_CONV_FILENAME));
+            }
 
-        // we got some images to add.
-        if (filesv.size() > 0) {
-            // use a Command to ensure proper undo and updating of GUI
-            // parts
-            wxBusyCursor();
-            GlobalCmdHist::getInstance().addCommand(
-                new wxAddImagesCmd(pano,filesv)
-                );
-        }
+            // we got some images to add.
+            if (filesv.size() > 0) {
+                // use a Command to ensure proper undo and updating of GUI
+                // parts
+                wxBusyCursor();
+                GlobalCmdHist::getInstance().addCommand(
+                    new wxAddImagesCmd(pano,filesv)
+                    );
+            };
+        };
         DEBUG_INFO ( wxString::Format(wxT("img_ext: %d"), dlg.GetFilterIndex()).mb_str(wxConvLocal) )
         // save the image extension
         switch ( dlg.GetFilterIndex() ) {
@@ -931,7 +973,7 @@ void MainFrame::OnAddTimeImages( wxCommandEvent& event )
         --images;
 
         // Get the filename.
-        const PanoImage& image = pano.getImage(images);
+        const SrcPanoImage& image = pano.getImage(images);
         std::string filename = image.getFilename();
         wxString file(filename.c_str(), HUGIN_CONV_FILENAME);
         preloaded[file] = 1;
@@ -988,7 +1030,7 @@ void MainFrame::OnAddTimeImages( wxCommandEvent& event )
             while (images)
             {
                 --images;
-                const PanoImage& image = pano.getImage(images);
+                const SrcPanoImage& image = pano.getImage(images);
                 std::string filename = image.getFilename();
                 wxString file(filename.c_str(), HUGIN_CONV_FILENAME);
                 if (file == recruit)
@@ -1024,21 +1066,18 @@ void MainFrame::OnAddTimeImages( wxCommandEvent& event )
 }
 
 
-void MainFrame::OnTextEdit( wxCommandEvent& WXUNUSED(event) )
-{
-        DEBUG_TRACE("");
-  wxDialog dlg;
-  wxXmlResource::Get()->LoadDialog(&dlg, this, wxT("text_edit_dialog"));
-  dlg.ShowModal();
-//  dlg.Show (TRUE);
-        DEBUG_TRACE("");
-}
-
 void MainFrame::OnShowDonate(wxCommandEvent & e)
 {
     wxLaunchDefaultBrowser(wxT("http://sourceforge.net/project/project_donations.php?group_id=77506"));
 }
 
+void MainFrame::OnAbout(wxCommandEvent & e)
+{
+    AboutDialog dlg(this);
+    dlg.ShowModal();
+}
+
+/*
 void MainFrame::OnAbout(wxCommandEvent & e)
 {
     DEBUG_TRACE("");
@@ -1068,65 +1107,73 @@ void MainFrame::OnAbout(wxCommandEvent & e)
 #endif
     dlg.ShowModal();
 }
+*/
 
 void MainFrame::OnHelp(wxCommandEvent & e)
 {
-    DisplayHelp(wxT("contents.html"));
+    DisplayHelp(wxT("/Hugin.html"));
 }
 
 void MainFrame::OnKeyboardHelp(wxCommandEvent & e)
 {
-    DisplayHelp(wxT("Hugin_Keyboard_shortcuts.html"));
+    DisplayHelp(wxT("/Hugin_Keyboard_shortcuts.html"));
 }
 
 void MainFrame::OnFAQ(wxCommandEvent & e)
 {
-    DisplayHelp(wxT("Hugin_FAQ.html"));
+    DisplayHelp(wxT("/Hugin_FAQ.html"));
 }
 
 
 void MainFrame::DisplayHelp(wxString section)
 {
+    // TODO:
+    // structure a frame with navigation on the left and content on the right
+    // always load the same navigation on the left and the section into the frame
+    // find a way to target always the same window rather than opening new window / tabs in the browser every time
+    // make it look nicer with some CSS styling
+
+    // section is the HTML document to be displayed, from inside the data folder
+    if(section==wxT("")){
+        section = wxT("/Hugin.html");
+    }
+
     DEBUG_TRACE("");
 
-    if (m_help == 0)
-    {
 #if defined __WXMAC__ && defined MAC_SELF_CONTAINED_BUNDLE
-        // On Mac, xrc/data/help_LOCALE should be in the bundle as LOCALE.lproj/help
-        // which we can rely on the operating sytem to pick the right locale's.
-        wxString strFile = MacGetPathToBundledResourceFile(CFSTR("help"));
-        if(strFile!=wxT(""))
-        {
-            strFile += wxT("/hugin.hhp");
-        } else {
-            wxLogError(wxString::Format(wxT("Could not find help directory in the bundle"), strFile.c_str()));
-            return;
-        }
+    // On Mac, xrc/data/help_LOCALE should be in the bundle as LOCALE.lproj/help
+    // which we can rely on the operating sytem to pick the right locale's.
+    wxString strFile = MacGetPathToBundledResourceFile(CFSTR("help"));
+    if(strFile!=wxT(""))
+    {
+        strFile += section;
+    } else {
+        wxLogError(wxString::Format(wxT("Could not find help directory in the bundle"), strFile.c_str()));
+        return;
+    }
 #else
-        // find base filename
-        wxString helpFile = wxT("help_") + huginApp::Get()->GetLocale().GetCanonicalName() + wxT("/hugin.hhp");
-        DEBUG_INFO("help file candidate: " << helpFile.mb_str(wxConvLocal));
-        //if the language is not default, load custom About file (if exists)
-        wxString strFile = GetXRCPath() + wxT("data/") + helpFile;
-        if(wxFile::Exists(strFile))
-        {
-            DEBUG_TRACE("Using About: " << strFile.mb_str(wxConvLocal));
-        } else {
-            // try default
-            strFile = GetXRCPath() + wxT("data/help_en_EN/hugin.hhp");
-        }
+    // find base filename
+    wxString helpFile = wxT("help_") + huginApp::Get()->GetLocale().GetCanonicalName() + section;
+    DEBUG_INFO("help file candidate: " << helpFile.mb_str(wxConvLocal));
+    //if the language is not default, load custom About file (if exists)
+    wxString strFile = GetXRCPath() + wxT("data/") + helpFile;
+    if(wxFile::Exists(strFile))
+    {
+        DEBUG_TRACE("Using About: " << strFile.mb_str(wxConvLocal));
+    } else {
+        strFile = GetXRCPath() + wxT("data/help_en_EN") + section;
+    }
 #endif
-
-        if(!wxFile::Exists(strFile))
-        {
-            wxLogError(wxString::Format(wxT("Could not open help file: %s"), strFile.c_str()));
-            return;
-        }
-        DEBUG_INFO("help file: " << strFile.mb_str(wxConvLocal));
-        m_help = new wxHtmlHelpController();
-        m_help->AddBook(strFile);
+    if(!wxFile::Exists(strFile))
+    {
+        wxLogError(wxString::Format(wxT("Could not open help file: %s"), strFile.c_str()));
+        return;
+    }
+    DEBUG_INFO("help file: " << strFile.mb_str(wxConvLocal));
+    if(!wxLaunchDefaultBrowser(strFile))
+    {
+        wxLogError(_("Can't start system's web browser"));
     }
-    m_help->Display(section);
 }
 
 void MainFrame::OnTipOfDay(wxCommandEvent& WXUNUSED(e))
@@ -1231,6 +1278,54 @@ void MainFrame::OnDoStitch(wxCommandEvent & e)
     pano_panel->DoStitch();
 }
 
+void MainFrame::OnMergeProject(wxCommandEvent & e)
+{
+    // get the global config object
+    wxConfigBase* config = wxConfigBase::Get();
+
+    wxString defaultdir = config->Read(wxT("/actualPath"),wxT(""));
+    wxFileDialog dlg(this,
+                     _("Open project file"),
+                     defaultdir, wxT(""),
+                     _("Project files (*.pto,*.ptp,*.pts,*.oto)|*.pto;*.ptp;*.pts;*.oto;|All files (*)|*"),
+                     wxOPEN, wxDefaultPosition);
+    dlg.SetDirectory(defaultdir);
+    if (dlg.ShowModal() == wxID_OK) 
+    {
+        wxString filename = dlg.GetPath();
+        wxFileName fname(filename);
+        wxString path = fname.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR);
+        if (fname.IsOk() && fname.FileExists())
+        {
+            wxBusyCursor wait;
+            PanoramaMemento newPano;
+            std::ifstream in((const char *)fname.GetFullPath().mb_str(HUGIN_CONV_FILENAME));
+            int ptoversion=0;
+            if (newPano.loadPTScript(in, ptoversion, (const char *)path.mb_str(HUGIN_CONV_FILENAME))) 
+            {
+                Panorama new_pano;
+                new_pano.setMemento(newPano);
+                GlobalCmdHist::getInstance().addCommand(
+                    new MergePanoCmd(pano, new_pano)
+                );
+                opt_panel->setModeCustom();
+                pano.clearDirty();
+                m_mruFiles.AddFileToHistory(fname.GetFullPath());
+                // force update of preview window
+                if ( !(preview_frame->IsIconized() ||(! preview_frame->IsShown()) ) ) 
+                {
+                    wxCommandEvent dummy;
+                    preview_frame->OnUpdate(dummy);
+                };
+            }
+            else
+            {
+                wxMessageBox(wxString::Format(_("Could not read project file %s."),fname.GetFullPath().c_str()),_("Error"),wxOK|wxICON_ERROR);
+            };
+        };
+    }
+}
+
 void MainFrame::OnApplyTemplate(wxCommandEvent & e)
 {
     // get the global config object
@@ -1405,6 +1500,62 @@ void MainFrame::OnFineTuneAll(wxCommandEvent & e)
         );
 }
 
+void MainFrame::OnRemoveCPinMasks(wxCommandEvent & e)
+{
+    HuginBase::UIntSet cps;
+    HuginBase::CPVector cpList=pano.getCtrlPoints();
+    if(cpList.size()>0)
+    {
+        for(unsigned int i=0;i<cpList.size();i++)
+        {
+            HuginBase::ControlPoint cp=cpList[i];
+            // ignore line control points
+            if(cp.mode!=HuginBase::ControlPoint::X_Y)
+                continue;
+            bool insideMask=false;
+            // check first image
+            // remark: we could also use pano.getImage(cp.image1Nr).isInside(vigra::Point2D(cp.x1,cp.y1))
+            //   this would also check the crop rectangles/circles
+            //   but it would require that the pano is correctly align, otherwise the positive masks
+            //   would not correctly checked
+            HuginBase::MaskPolygonVector masks=pano.getImage(cp.image1Nr).getMasks();
+            if(masks.size()>0)
+            {
+                unsigned int j=0;
+                while((!insideMask) && (j<masks.size()))
+                {
+                    insideMask=masks[j].isInside(hugin_utils::FDiff2D(cp.x1,cp.y1));
+                    j++;
+                };
+            };
+            // and now the second
+            if(!insideMask)
+            {
+                masks=pano.getImage(cp.image2Nr).getMasks();
+                if(masks.size()>0)
+                {
+                    unsigned int j=0;
+                    while((!insideMask) && (j<masks.size()))
+                    {
+                        insideMask=masks[j].isInside(hugin_utils::FDiff2D(cp.x2,cp.y2));
+                        j++;
+                    };
+                };
+            };
+            if(insideMask)
+                cps.insert(i);
+        };
+        if(cps.size()>0)
+        {
+            GlobalCmdHist::getInstance().addCommand(
+                        new PT::RemoveCtrlPointsCmd(pano,cps)
+                        );
+        };
+    };
+    wxMessageBox(wxString::Format(_("Removed %d control points"), cps.size()), 
+                 _("Removing control points in masks"),wxOK|wxICON_INFORMATION,this);
+}
+
 void MainFrame::OnUndo(wxCommandEvent & e)
 {
     DEBUG_TRACE("OnUndo");
@@ -1421,13 +1572,13 @@ void MainFrame::OnRedo(wxCommandEvent & e)
 void MainFrame::ShowCtrlPoint(unsigned int cpNr)
 {
     DEBUG_DEBUG("Showing control point " << cpNr);
-    m_notebook->SetSelection(4);
+    m_notebook->SetSelection(5);
     cpe->ShowControlPoint(cpNr);
 }
 
 void MainFrame::ShowCtrlPointEditor(unsigned int img1, unsigned int img2)
 {
-    m_notebook->SetSelection(4);
+    m_notebook->SetSelection(5);
     cpe->setLeftImage(img1);
     cpe->setRightImage(img2);
 }
@@ -1620,5 +1771,32 @@ bool getLensDataFromUser(wxWindow * parent, SrcPanoImage & srcImg,
     }
 }
 
+void MainFrame::OnMRUFiles(wxCommandEvent &e)
+{
+    size_t index = e.GetId() - wxID_FILE1;
+    wxString f(m_mruFiles.GetHistoryFile(index));
+    if (!f.empty())
+    {
+        wxFileName fn(f);
+        if(fn.FileExists())
+            LoadProjectFile(f);
+        else
+        {
+            m_mruFiles.RemoveFileFromHistory(index);
+            wxMessageBox(wxString::Format(_("File \"%s\" not found.\nMaybe file was renamed, moved or deleted."),f.c_str()),
+                _("Error!"),wxOK | wxICON_INFORMATION );
+        };
+    };
+}
+
+void MainFrame::OnFullScreen(wxCommandEvent & e)
+{
+    ShowFullScreen(!IsFullScreen(), wxFULLSCREEN_NOBORDER | wxFULLSCREEN_NOCAPTION);
+#ifdef __WXGTK__
+    //workaround a wxGTK bug that also the toolbar is hidden, but not requested to hide
+    GetToolBar()->Show(true);
+#endif
+};
+
 MainFrame * MainFrame::m_this = 0;
 
