@@ -6,7 +6,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: OptimizePanel.cpp 4361 2009-09-10 16:14:09Z yuv $
+ *  $Id$
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -57,11 +57,21 @@ BEGIN_EVENT_TABLE(OptimizePanel, wxPanel)
 //    EVT_BUTTON(XRCID("opt_pitch_equalize"), OptimizePanel::OnEqPitch)
     EVT_BUTTON(XRCID("opt_roll_select"), OptimizePanel::OnListButton)
     EVT_BUTTON(XRCID("opt_roll_clear"), OptimizePanel::OnListButton)
+    EVT_BUTTON(XRCID("opt_x_select"), OptimizePanel::OnListButton)
+    EVT_BUTTON(XRCID("opt_x_clear"), OptimizePanel::OnListButton)
+    EVT_BUTTON(XRCID("opt_y_select"), OptimizePanel::OnListButton)
+    EVT_BUTTON(XRCID("opt_y_clear"), OptimizePanel::OnListButton)
+    EVT_BUTTON(XRCID("opt_z_select"), OptimizePanel::OnListButton)
+    EVT_BUTTON(XRCID("opt_z_clear"), OptimizePanel::OnListButton)
+
     EVT_CHOICE(XRCID("optimize_panel_mode"), OptimizePanel::OnChangeMode)
 //    EVT_BUTTON(XRCID("opt_roll_equalize"), OptimizePanel::OnEqRoll)
     EVT_CHECKLISTBOX(XRCID("optimizer_yaw_list"), OptimizePanel::OnCheckBoxChanged)
     EVT_CHECKLISTBOX(XRCID("optimizer_pitch_list"), OptimizePanel::OnCheckBoxChanged)
     EVT_CHECKLISTBOX(XRCID("optimizer_roll_list"), OptimizePanel::OnCheckBoxChanged)
+    EVT_CHECKLISTBOX(XRCID("optimizer_x_list"), OptimizePanel::OnCheckBoxChanged)
+    EVT_CHECKLISTBOX(XRCID("optimizer_y_list"), OptimizePanel::OnCheckBoxChanged)
+    EVT_CHECKLISTBOX(XRCID("optimizer_z_list"), OptimizePanel::OnCheckBoxChanged)
     EVT_CHECKLISTBOX(XRCID("optimizer_v_list"), OptimizePanel::OnCheckBoxChanged)
     EVT_CHECKLISTBOX(XRCID("optimizer_a_list"), OptimizePanel::OnCheckBoxChanged)
     EVT_CHECKLISTBOX(XRCID("optimizer_b_list"), OptimizePanel::OnCheckBoxChanged)
@@ -71,8 +81,8 @@ BEGIN_EVENT_TABLE(OptimizePanel, wxPanel)
 END_EVENT_TABLE()
 
 // local optimize definition. need to be in sync with the xrc file
-enum OptimizeMode { OPT_PAIRWISE=0, OPT_YRP, OPT_YRP_V,
-                    OPT_YRP_B, OPT_YRP_BV, OPT_ALL, OPT_CUSTOM,
+enum OptimizeMode { OPT_PAIRWISE=0, OPT_YRP, OPT_YRP_XYZ, OPT_YRP_V, OPT_YRP_XYZ_V,
+                    OPT_YRP_B, OPT_YRP_XYZ_B, OPT_YRP_BV, OPT_YRP_XYZ_BV, OPT_ALL_NOTXYZ, OPT_ALL, OPT_CUSTOM,
                     OPT_END_MARKER};
 
 OptimizePanel::OptimizePanel()
@@ -106,6 +116,9 @@ bool OptimizePanel::Create(wxWindow* parent, wxWindowID id , const wxPoint& pos,
     m_yaw_list = XRCCTRL(*this, "optimizer_yaw_list", wxCheckListBox);
     m_pitch_list = XRCCTRL(*this, "optimizer_pitch_list", wxCheckListBox);
     m_roll_list = XRCCTRL(*this, "optimizer_roll_list", wxCheckListBox);
+    m_x_list = XRCCTRL(*this, "optimizer_x_list", wxCheckListBox);
+    m_y_list = XRCCTRL(*this, "optimizer_y_list", wxCheckListBox);
+    m_z_list = XRCCTRL(*this, "optimizer_z_list", wxCheckListBox);
 
     m_v_list = XRCCTRL(*this, "optimizer_v_list", wxCheckListBox);
     m_a_list = XRCCTRL(*this, "optimizer_a_list", wxCheckListBox);
@@ -137,6 +150,7 @@ void OptimizePanel::Init(PT::Panorama * pano)
 {
     DEBUG_TRACE("");
     m_pano = pano;
+    variable_groups = new HuginBase::StandardImageVariableGroups(*pano);
 
 //    wxConfigBase * config = wxConfigBase::Get();
 //    long w = config->Read(wxT("/OptimizerPanel/width"),-1);
@@ -164,6 +178,7 @@ OptimizePanel::~OptimizePanel()
 //    config->Write(wxT("/OptimizerPanel/width"),sz.GetWidth());
 //    config->Write(wxT("/OptimizerPanel/height"),sz.GetHeight());
     m_pano->removeObserver(this);
+    delete variable_groups;
     DEBUG_TRACE("dtor end");
 }
 
@@ -182,6 +197,18 @@ void OptimizePanel::OnListButton(wxCommandEvent & e)
         SetCheckMark(m_roll_list,true);
     } else if (e.GetId() == XRCID("opt_roll_clear")) {
         SetCheckMark(m_roll_list,false);
+    } else if (e.GetId() == XRCID("opt_x_select")) {
+        SetCheckMark(m_x_list,true);
+    } else if (e.GetId() == XRCID("opt_x_clear")) {
+        SetCheckMark(m_x_list,false);
+    } else if (e.GetId() == XRCID("opt_y_select")) {
+        SetCheckMark(m_y_list,true);
+    } else if (e.GetId() == XRCID("opt_y_clear")) {
+        SetCheckMark(m_y_list,false);
+    } else if (e.GetId() == XRCID("opt_z_select")) {
+        SetCheckMark(m_z_list,true);
+    } else if (e.GetId() == XRCID("opt_z_clear")) {
+        SetCheckMark(m_z_list,false);
     } else {
         DEBUG_FATAL("An error has occured");
     }
@@ -243,7 +270,7 @@ OptimizeVector OptimizePanel::getOptimizeVector()
 
         set<string> imgopt;
         // lens variables
-        unsigned int lensNr = m_pano->getImage(i).getLensNr();
+        unsigned int lensNr = variable_groups->getLenses().getPartNumber(i);
 
         if (m_v_list->IsChecked(lensNr)) {
             imgopt.insert("v");
@@ -275,6 +302,16 @@ OptimizeVector OptimizePanel::getOptimizeVector()
             imgopt.insert("y");
         }
 
+        if (m_x_list->IsChecked(i)) {
+            imgopt.insert("TrX");
+        }
+        if (m_y_list->IsChecked(i)) {
+            imgopt.insert("TrY");
+        }
+        if (m_z_list->IsChecked(i)) {
+            imgopt.insert("TrZ");
+        }
+
         optvars.push_back(imgopt);
     }
 
@@ -284,6 +321,8 @@ OptimizeVector OptimizePanel::getOptimizeVector()
 void OptimizePanel::panoramaChanged(PT::Panorama & pano)
 {
 	DEBUG_TRACE("");
+    // make sure all the lens numbers are correct.
+    variable_groups->update();
     setOptimizeVector(pano.getOptimizeVector());
 
     // update accordingly to the choosen mode
@@ -295,6 +334,8 @@ void OptimizePanel::panoramaChanged(PT::Panorama & pano)
 void OptimizePanel::panoramaImagesChanged(PT::Panorama &pano,
                                           const PT::UIntSet & imgNr)
 {
+    // make sure all the lens numbers are correct.
+    variable_groups->update();
     DEBUG_TRACE("nr of changed images: " << imgNr.size());
 	if (pano.getNrOfImages() == 0)
 	{
@@ -310,7 +351,7 @@ void OptimizePanel::panoramaImagesChanged(PT::Panorama &pano,
     int nrLensList = m_v_list->GetCount();
     assert(nrLensList >=0);
     unsigned int nr = (unsigned int) nrLensList;
-    unsigned int nLens = m_pano->getNrOfLenses();
+    unsigned int nLens = variable_groups->getLenses().getNumberOfParts();
     while (nr < nLens) {
         // add checkboxes.
         m_v_list->Append(wxString::Format(wxT("%d"),nr));
@@ -344,6 +385,9 @@ void OptimizePanel::panoramaImagesChanged(PT::Panorama &pano,
         m_yaw_list->Append(wxString::Format(wxT("%d"),nr));
         m_pitch_list->Append(wxString::Format(wxT("%d"),nr));
         m_roll_list->Append(wxString::Format(wxT("%d"),nr));
+        m_x_list->Append(wxString::Format(wxT("%d"),nr));
+        m_y_list->Append(wxString::Format(wxT("%d"),nr));
+        m_z_list->Append(wxString::Format(wxT("%d"),nr));
         nr++;
     }
 
@@ -353,6 +397,9 @@ void OptimizePanel::panoramaImagesChanged(PT::Panorama &pano,
         m_yaw_list->Delete(nr-1);
         m_pitch_list->Delete(nr-1);
         m_roll_list->Delete(nr-1);
+        m_x_list->Delete(nr-1);
+        m_y_list->Delete(nr-1);
+        m_z_list->Delete(nr-1);
         nr--;
     }
 
@@ -376,11 +423,26 @@ void OptimizePanel::panoramaImagesChanged(PT::Panorama &pano,
         m_roll_list->SetString(*it, wxString::Format(wxT("%d (%.3f)"),
                                 *it, const_map_get(vars,"r").getValue()));
         m_roll_list->Check(*it,sel);
+
+        sel = m_x_list->IsChecked(*it);
+        m_x_list->SetString(*it, wxString::Format(wxT("%d (%.3f)"),
+                                *it, const_map_get(vars,"TrX").getValue()));
+        m_x_list->Check(*it,sel);
+
+        sel = m_y_list->IsChecked(*it);
+        m_y_list->SetString(*it, wxString::Format(wxT("%d (%.3f)"),
+                                *it, const_map_get(vars,"TrY").getValue()));
+        m_y_list->Check(*it,sel);
+
+        sel = m_z_list->IsChecked(*it);
+        m_z_list->SetString(*it, wxString::Format(wxT("%d (%.3f)"),
+                                *it, const_map_get(vars,"TrZ").getValue()));
+        m_z_list->Check(*it,sel);
     }
 
     // display lens values if they are linked
     for (unsigned int i=0; i < nLens; i++) {
-        const Lens & lens = pano.getLens(i);
+        const Lens & lens = variable_groups->getLens(i);
         const LensVariable & v = const_map_get(lens.variables,"v");
         bool sel = m_v_list->IsChecked(i);
         if (v.isLinked()) {
@@ -455,7 +517,7 @@ void OptimizePanel::setOptimizeVector(const OptimizeVector & optvec)
 {
     DEBUG_ASSERT((int)optvec.size() == (int)m_yaw_list->GetCount());
 
-    for (int i=0; i < (int) m_pano->getNrOfLenses(); i++) {
+    for (int i=0; i < (int) variable_groups->getLenses().getNumberOfParts(); i++) {
         m_v_list->Check(i,false);
         m_a_list->Check(i,false);
         m_b_list->Check(i,false);
@@ -469,7 +531,10 @@ void OptimizePanel::setOptimizeVector(const OptimizeVector & optvec)
         m_yaw_list->Check(i,false);
         m_pitch_list->Check(i,false);
         m_roll_list->Check(i,false);
-        unsigned int lensNr = m_pano->getImage(i).getLensNr();
+        m_x_list->Check(i,false);
+        m_y_list->Check(i,false);
+        m_z_list->Check(i,false);
+        unsigned int lensNr = variable_groups->getLenses().getPartNumber(i);
 
         for(set<string>::const_iterator it = optvec[i].begin();
                 it != optvec[i].end(); ++it)
@@ -483,6 +548,15 @@ void OptimizePanel::setOptimizeVector(const OptimizeVector & optvec)
             if (*it == "r") {
                 m_roll_list->Check(i);
             }
+            if (*it == "TrX") {
+                m_x_list->Check(i);
+            }
+            if (*it == "TrY") {
+                m_y_list->Check(i);
+            }
+            if (*it == "TrZ") {
+                m_z_list->Check(i);
+            }
 
             if (*it == "v") {
                 m_v_list->Check(lensNr);
@@ -539,6 +613,10 @@ void OptimizePanel::runOptimizer(const UIntSet & imgs)
         optvars2.insert("y");
         optvars2.insert("p");
         optvars2.insert("r");
+	// TODO: check tilt mode here, before inserting the parameters.
+        /* optvars2.insert("TrX");
+        optvars2.insert("TrY");
+        optvars2.insert("TrZ"); */
 
         // remove vertical and horizontal control points
         CPVector cps = optPano.getCtrlPoints();
@@ -677,9 +755,15 @@ void OptimizePanel::OnChangeMode(wxCommandEvent & e)
   	  XRCCTRL(*this, "opt_yaw_select", wxButton)->Disable();
   	  XRCCTRL(*this, "opt_roll_select", wxButton)->Disable();
   	  XRCCTRL(*this, "opt_pitch_select", wxButton)->Disable();
+      XRCCTRL(*this, "opt_x_select", wxButton)->Disable();
+      XRCCTRL(*this, "opt_y_select", wxButton)->Disable();
+      XRCCTRL(*this, "opt_z_select", wxButton)->Disable();
   	  XRCCTRL(*this, "opt_yaw_clear", wxButton)->Disable();
   	  XRCCTRL(*this, "opt_roll_clear", wxButton)->Disable();
   	  XRCCTRL(*this, "opt_pitch_clear", wxButton)->Disable();
+      XRCCTRL(*this, "opt_x_clear", wxButton)->Disable();
+      XRCCTRL(*this, "opt_y_clear", wxButton)->Disable();
+      XRCCTRL(*this, "opt_z_clear", wxButton)->Disable();
 	} else {
       switch (mode) {
 	    case OPT_PAIRWISE:
@@ -687,6 +771,9 @@ void OptimizePanel::OnChangeMode(wxCommandEvent & e)
       	  SetCheckMark(m_yaw_list,true);
       	  SetCheckMark(m_roll_list,true);
       	  SetCheckMark(m_pitch_list,true);
+          SetCheckMark(m_x_list,false);
+          SetCheckMark(m_y_list,false);
+          SetCheckMark(m_z_list,false);
       	  SetCheckMark(m_v_list,false);
           SetCheckMark(m_a_list,false);
           SetCheckMark(m_b_list,false);
@@ -699,6 +786,24 @@ void OptimizePanel::OnChangeMode(wxCommandEvent & e)
           SetCheckMark(m_yaw_list,true);
           SetCheckMark(m_roll_list,true);
           SetCheckMark(m_pitch_list,true);
+          SetCheckMark(m_x_list,false);
+          SetCheckMark(m_y_list,false);
+          SetCheckMark(m_z_list,false);
+          SetCheckMark(m_v_list,false);
+          SetCheckMark(m_a_list,false);
+          SetCheckMark(m_b_list,false);
+          SetCheckMark(m_c_list,false);
+          SetCheckMark(m_d_list,false);
+          SetCheckMark(m_e_list,false);
+          break;
+        case OPT_YRP_XYZ:
+          // position + translation
+          SetCheckMark(m_yaw_list,true);
+          SetCheckMark(m_roll_list,true);
+          SetCheckMark(m_pitch_list,true);
+          SetCheckMark(m_x_list,true);
+          SetCheckMark(m_y_list,true);
+          SetCheckMark(m_z_list,true);
           SetCheckMark(m_v_list,false);
           SetCheckMark(m_a_list,false);
           SetCheckMark(m_b_list,false);
@@ -711,6 +816,24 @@ void OptimizePanel::OnChangeMode(wxCommandEvent & e)
           SetCheckMark(m_yaw_list,true);
           SetCheckMark(m_roll_list,true);
           SetCheckMark(m_pitch_list,true);
+          SetCheckMark(m_x_list,false);
+          SetCheckMark(m_y_list,false);
+          SetCheckMark(m_z_list,false);
+          SetCheckMark(m_v_list,true);
+          SetCheckMark(m_a_list,false);
+          SetCheckMark(m_b_list,false);
+          SetCheckMark(m_c_list,false);
+          SetCheckMark(m_d_list,false);
+          SetCheckMark(m_e_list,false);
+          break;
+        case OPT_YRP_XYZ_V:
+          // v + position + translation
+          SetCheckMark(m_yaw_list,true);
+          SetCheckMark(m_roll_list,true);
+          SetCheckMark(m_pitch_list,true);
+          SetCheckMark(m_x_list,true);
+          SetCheckMark(m_y_list,true);
+          SetCheckMark(m_z_list,true);
           SetCheckMark(m_v_list,true);
           SetCheckMark(m_a_list,false);
           SetCheckMark(m_b_list,false);
@@ -723,6 +846,24 @@ void OptimizePanel::OnChangeMode(wxCommandEvent & e)
           SetCheckMark(m_yaw_list,true);
           SetCheckMark(m_roll_list,true);
           SetCheckMark(m_pitch_list,true);
+          SetCheckMark(m_x_list,false);
+          SetCheckMark(m_y_list,false);
+          SetCheckMark(m_z_list,false);
+          SetCheckMark(m_v_list,false);
+          SetCheckMark(m_a_list,false);
+          SetCheckMark(m_b_list,true);
+          SetCheckMark(m_c_list,false);
+          SetCheckMark(m_d_list,false);
+          SetCheckMark(m_e_list,false);
+          break;
+        case OPT_YRP_XYZ_B:
+          // important lens distortion + position + translation
+          SetCheckMark(m_yaw_list,true);
+          SetCheckMark(m_roll_list,true);
+          SetCheckMark(m_pitch_list,true);
+          SetCheckMark(m_x_list,true);
+          SetCheckMark(m_y_list,true);
+          SetCheckMark(m_z_list,true);
           SetCheckMark(m_v_list,false);
           SetCheckMark(m_a_list,false);
           SetCheckMark(m_b_list,true);
@@ -735,6 +876,9 @@ void OptimizePanel::OnChangeMode(wxCommandEvent & e)
           SetCheckMark(m_yaw_list,true);
           SetCheckMark(m_roll_list,true);
           SetCheckMark(m_pitch_list,true);
+          SetCheckMark(m_x_list,false);
+          SetCheckMark(m_y_list,false);
+          SetCheckMark(m_z_list,false);
           SetCheckMark(m_v_list,true);
           SetCheckMark(m_a_list,false);
           SetCheckMark(m_b_list,true);
@@ -742,11 +886,44 @@ void OptimizePanel::OnChangeMode(wxCommandEvent & e)
           SetCheckMark(m_d_list,false);
           SetCheckMark(m_e_list,false);
           break;
-  	    case OPT_ALL:
+        case OPT_YRP_XYZ_BV:
+          // important lens distortion + v + position + translation
+          SetCheckMark(m_yaw_list,true);
+          SetCheckMark(m_roll_list,true);
+          SetCheckMark(m_pitch_list,true);
+          SetCheckMark(m_x_list,true);
+          SetCheckMark(m_y_list,true);
+          SetCheckMark(m_z_list,true);
+          SetCheckMark(m_v_list,true);
+          SetCheckMark(m_a_list,false);
+          SetCheckMark(m_b_list,true);
+          SetCheckMark(m_c_list,false);
+          SetCheckMark(m_d_list,false);
+          SetCheckMark(m_e_list,false);
+          break;
+  	    case OPT_ALL_NOTXYZ:
           // everything
           SetCheckMark(m_yaw_list,true);
           SetCheckMark(m_roll_list,true);
           SetCheckMark(m_pitch_list,true);
+          SetCheckMark(m_x_list,false);
+          SetCheckMark(m_y_list,false);
+          SetCheckMark(m_z_list,false);
+          SetCheckMark(m_v_list,true);
+          SetCheckMark(m_a_list,true);
+          SetCheckMark(m_b_list,true);
+          SetCheckMark(m_c_list,true);
+          SetCheckMark(m_d_list,true);
+          SetCheckMark(m_e_list,true);
+	  break;
+    	    case OPT_ALL:
+          // everything
+          SetCheckMark(m_yaw_list,true);
+          SetCheckMark(m_roll_list,true);
+          SetCheckMark(m_pitch_list,true);
+          SetCheckMark(m_x_list,true);
+          SetCheckMark(m_y_list,true);
+          SetCheckMark(m_z_list,true);
           SetCheckMark(m_v_list,true);
           SetCheckMark(m_a_list,true);
           SetCheckMark(m_b_list,true);
@@ -761,8 +938,20 @@ void OptimizePanel::OnChangeMode(wxCommandEvent & e)
       // if the user selected custom
       if (mode != OPT_CUSTOM && m_pano->getNrOfImages() > 0)
 	  {
-        // get anchor image
+        // get anchor image and images linked to its position.
         unsigned int refImg = m_pano->getOptions().optimizeReferenceImage;
+        std::vector<unsigned int> refImgs;
+        const HuginBase::SrcPanoImage & refImage = m_pano->getImage(refImg);
+        for (unsigned int imgNr = 0; imgNr < m_pano->getNrOfImages(); imgNr++)
+        {
+            const HuginBase::SrcPanoImage & compImage = m_pano->getImage(imgNr);
+            if (refImage.RollisLinkedWith(compImage) &&
+                refImage.PitchisLinkedWith(compImage) &&
+                refImage.YawisLinkedWith(compImage))
+            {
+                refImgs.push_back(imgNr);
+            }
+        }
 
         // count number of vertical/horizontal control points
         int nHCP = 0;
@@ -782,26 +971,57 @@ void OptimizePanel::OnChangeMode(wxCommandEvent & e)
 
         // try to select sensible position optimisation parameters,
         // dependent on output projection
+        bool linkRefImgsYaw, linkRefImgsPitch, linkRefImgsRoll;
         switch (m_pano->getOptions().getProjection()) {
           case PT::PanoramaOptions::RECTILINEAR:
-            m_roll_list->Check(refImg, (nVCP+nHCP) >=1 );
-            m_yaw_list->Check(refImg, (nVCP+nHCP) >=3 && (nVCP >= 1) && (nHCP >= 1) );
-            m_pitch_list->Check(refImg, (nVCP+nHCP) >= 2 );
+            linkRefImgsRoll = nVCP + nHCP >= 1;
+            linkRefImgsYaw = nVCP + nHCP >= 3 && nVCP >= 1 && nHCP >= 1;
+            linkRefImgsPitch = nVCP + nHCP >= 2;
+            for (std::vector<unsigned int>::iterator it = refImgs.begin();
+                 it != refImgs.end(); it++)
+            {
+                m_roll_list->Check(*it, linkRefImgsRoll);
+                m_yaw_list->Check(*it, linkRefImgsYaw);
+                m_pitch_list->Check(*it, linkRefImgsPitch);
+            }
             break;
           case PT::PanoramaOptions::CYLINDRICAL:
           case PT::PanoramaOptions::EQUIRECTANGULAR:
-            m_yaw_list->Check(refImg,false);
-            m_pitch_list->Check(refImg, (nHCP+nVCP > 1));
-            m_roll_list->Check(refImg, (nHCP+nVCP >= 1));
+            linkRefImgsPitch =  nHCP + nVCP > 1;
+            linkRefImgsRoll = nHCP + nVCP >= 1;
+            for (std::vector<unsigned int>::iterator it = refImgs.begin();
+                 it != refImgs.end(); it++)
+            {
+                m_roll_list->Check(*it, linkRefImgsRoll);
+                m_yaw_list->Check(*it, false);
+                m_pitch_list->Check(*it, linkRefImgsPitch);
+            }
             break;
           default:
             break;
         }
-
+        bool optXYZ=m_x_list->IsChecked(0) || m_y_list->IsChecked(0) || m_z_list->IsChecked(0);
+        // don't optimize translation of anchor image
+        // if translation should be optimized, then also optimise yaw and pitch of anchor
+        for (std::vector<unsigned int>::iterator it = refImgs.begin();
+             it != refImgs.end(); it++)
+        {
+            m_x_list->Check(*it, false);
+            m_y_list->Check(*it, false);
+            m_z_list->Check(*it, false);
+            if(optXYZ)
+            {
+                m_yaw_list->Check(*it,true);
+                m_pitch_list->Check(*it,true);
+            };
+        }
 	    // disable all manual settings
 	    m_yaw_list->Disable();
 	    m_pitch_list->Disable();
 	    m_roll_list->Disable();
+	    m_x_list->Disable();
+	    m_y_list->Disable();
+	    m_z_list->Disable();
 	    m_v_list->Disable();
 	    m_a_list->Disable();
 	    m_b_list->Disable();
@@ -811,13 +1031,22 @@ void OptimizePanel::OnChangeMode(wxCommandEvent & e)
   	    XRCCTRL(*this, "opt_yaw_select", wxButton)->Disable();
   	    XRCCTRL(*this, "opt_roll_select", wxButton)->Disable();
   	    XRCCTRL(*this, "opt_pitch_select", wxButton)->Disable();
+        XRCCTRL(*this, "opt_x_select", wxButton)->Disable();
+        XRCCTRL(*this, "opt_y_select", wxButton)->Disable();
+        XRCCTRL(*this, "opt_z_select", wxButton)->Disable();
   	    XRCCTRL(*this, "opt_yaw_clear", wxButton)->Disable();
   	    XRCCTRL(*this, "opt_roll_clear", wxButton)->Disable();
   	    XRCCTRL(*this, "opt_pitch_clear", wxButton)->Disable();
+        XRCCTRL(*this, "opt_x_clear", wxButton)->Disable();
+        XRCCTRL(*this, "opt_y_clear", wxButton)->Disable();
+        XRCCTRL(*this, "opt_z_clear", wxButton)->Disable();
       } else {
 	    m_yaw_list->Enable();
 	    m_pitch_list->Enable();
 	    m_roll_list->Enable();
+	    m_x_list->Enable();
+	    m_y_list->Enable();
+	    m_z_list->Enable();
 	    m_v_list->Enable();
 	    m_a_list->Enable();
 	    m_b_list->Enable();
@@ -827,9 +1056,15 @@ void OptimizePanel::OnChangeMode(wxCommandEvent & e)
         XRCCTRL(*this, "opt_yaw_select", wxButton)->Enable();
         XRCCTRL(*this, "opt_roll_select", wxButton)->Enable();
         XRCCTRL(*this, "opt_pitch_select", wxButton)->Enable();
+        XRCCTRL(*this, "opt_x_select", wxButton)->Enable();
+        XRCCTRL(*this, "opt_y_select", wxButton)->Enable();
+        XRCCTRL(*this, "opt_z_select", wxButton)->Enable();
         XRCCTRL(*this, "opt_yaw_clear", wxButton)->Enable();
         XRCCTRL(*this, "opt_roll_clear", wxButton)->Enable();
         XRCCTRL(*this, "opt_pitch_clear", wxButton)->Enable();
+        XRCCTRL(*this, "opt_x_clear", wxButton)->Enable();
+        XRCCTRL(*this, "opt_y_clear", wxButton)->Enable();
+        XRCCTRL(*this, "opt_z_clear", wxButton)->Enable();
       }
 	}
     m_edit_cb->Enable(mode != OPT_PAIRWISE);
