@@ -7,7 +7,7 @@
  *  @author Kai-Uwe Behrmann <web@tiscali.de> and
  *          Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: PanoPanel.cpp 5034 2010-02-28 21:44:14Z brunopostle $
+ *  $Id$
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -159,6 +159,7 @@ bool PanoPanel::Create(wxWindow *parent, wxWindowID id, const wxPoint& pos, cons
     wxLogMessage(_("Equirectangular Panini"));
     wxLogMessage(_("Biplane"));
     wxLogMessage(_("Triplane"));
+    wxLogMessage(_("Panini General"));
 #endif
 
     /* populate with all available projection types */
@@ -380,7 +381,7 @@ void PanoPanel::UpdateDisplay(const PanoramaOptions & opt, const bool hasStacks)
     m_VFOVText->Enable(m_keepViewOnResize);
     m_CalcOptWidthButton->Enable(m_keepViewOnResize && hasImages);
     m_CalcHFOVButton->Enable(m_keepViewOnResize && hasImages);
-    m_CalcOptROIButton->Enable(m_keepViewOnResize && hasImages);
+    m_CalcOptROIButton->Enable(hasImages);
 
     m_WidthTxt->SetValue(wxString::Format(wxT("%d"), opt.getWidth()));
     m_HeightTxt->SetValue(wxString::Format(wxT("%d"), opt.getHeight()));
@@ -925,7 +926,6 @@ void PanoPanel::DoCalcOptimalROI(wxCommandEvent & e)
 {
     DEBUG_INFO("Dirty ROI Calc\n");
     if (pano->getActiveImages().size() == 0) return;
-
     ProgressReporterDialog progress(2, _("Autocrop"), _("Calculating optimal crop"),this, wxPD_AUTO_HIDE | wxPD_APP_MODAL | wxPD_ELAPSED_TIME);
     progress.increaseProgress(1);
     progress.Pulse();
