@@ -6,7 +6,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: PreviewFrame.cpp 4806 2009-12-14 20:37:26Z ade_br $
+ *  $Id$
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -66,7 +66,10 @@ enum {
     PROJ_PARAM_NAMES_ID = wxID_HIGHEST+1000,
     PROJ_PARAM_VAL_ID = wxID_HIGHEST+1100,
     PROJ_PARAM_SLIDER_ID = wxID_HIGHEST+1200,
-    ID_FULL_SCREEN = wxID_HIGHEST+1700
+    PROJ_PARAM_RESET_ID = wxID_HIGHEST+1250,
+    ID_FULL_SCREEN = wxID_HIGHEST+1700,
+    ID_UNDO = wxID_HIGHEST+1701,
+    ID_REDO = wxID_HIGHEST+1702
 };
 
 BEGIN_EVENT_TABLE(PreviewFrame, wxFrame)
@@ -103,6 +106,9 @@ BEGIN_EVENT_TABLE(PreviewFrame, wxFrame)
     EVT_SCROLL_THUMBTRACK(PreviewFrame::OnChangeFOV)
 #endif
     EVT_TOOL(ID_FULL_SCREEN, PreviewFrame::OnFullScreen)
+    EVT_TOOL(ID_UNDO, PreviewFrame::OnUndo)
+    EVT_TOOL(ID_REDO, PreviewFrame::OnRedo)
+    EVT_BUTTON(PROJ_PARAM_RESET_ID, PreviewFrame::OnProjParameterReset)
 END_EVENT_TABLE()
 
 #define PF_STYLE (wxMAXIMIZE_BOX | wxRESIZE_BORDER | wxSYSTEM_MENU | wxCAPTION | wxCLOSE_BOX | wxCLIP_CHILDREN)
@@ -290,6 +296,11 @@ PreviewFrame::PreviewFrame(wxFrame * frame, PT::Panorama &pano)
     new wxStaticBox(this, -1, _("Projection Parameters")),
      wxHORIZONTAL);
 
+    wxBitmapButton * resetProjButton=new wxBitmapButton(this, PROJ_PARAM_RESET_ID, 
+        wxArtProvider::GetBitmap(wxART_REDO));
+    resetProjButton->SetToolTip(_("Resets the projections parameters to their default values."));
+    m_projParamSizer->Add(resetProjButton, 0, wxLEFT | wxRIGHT | wxALIGN_CENTER_VERTICAL, 5);
+
     m_projParamNamesLabel.resize(PANO_PROJECTION_MAX_PARMS);
     m_projParamTextCtrl.resize(PANO_PROJECTION_MAX_PARMS);
     m_projParamSlider.resize(PANO_PROJECTION_MAX_PARMS);
@@ -302,7 +313,7 @@ PreviewFrame::PreviewFrame(wxFrame * frame, PT::Panorama &pano)
                         wxALL | wxALIGN_CENTER_VERTICAL, // draw border all around
                         5);       // border width
         m_projParamTextCtrl[i] = new wxTextCtrl(this, PROJ_PARAM_VAL_ID+i, wxT("0"),
-                                    wxDefaultPosition,wxDefaultSize, wxTE_PROCESS_ENTER);
+                                    wxDefaultPosition, wxSize(35,-1), wxTE_PROCESS_ENTER);
         m_projParamSizer->Add(m_projParamTextCtrl[i],
                         0,        // not vertically strechable
                         wxALL | wxALIGN_CENTER_VERTICAL, // draw border all around
@@ -362,9 +373,11 @@ PreviewFrame::PreviewFrame(wxFrame * frame, PT::Panorama &pano)
         Show();
     }
     SetStatusText(_("Center panorama with left mouse button, set horizon with right button"),0);
-    wxAcceleratorEntry entries[1];
+    wxAcceleratorEntry entries[3];
     entries[0].Set(wxACCEL_NORMAL,WXK_F11,ID_FULL_SCREEN);
-    wxAcceleratorTable accel(1, entries);
+    entries[1].Set(wxACCEL_CTRL,(int)'Z',ID_UNDO);
+    entries[2].Set(wxACCEL_CTRL,(int)'Y',ID_REDO);
+    wxAcceleratorTable accel(3, entries);
     SetAcceleratorTable(accel);
 #ifdef __WXGTK__
     // set explicit focus to button panel, otherwise the hotkey F11 is not right processed
@@ -780,6 +793,15 @@ void PreviewFrame::OnTextCtrlChanged(wxCommandEvent & e)
 
 }
 
+void PreviewFrame::OnProjParameterReset(wxCommandEvent &e)
+{
+    PanoramaOptions opts=m_pano.getOptions();
+    opts.resetProjectionParameters();
+    GlobalCmdHist::getInstance().addCommand(
+        new PT::SetPanoOptionsCmd(m_pano, opts)
+        );
+};
+
 void PreviewFrame::OnChangeFOV(wxScrollEvent & e)
 {
     DEBUG_TRACE("");
@@ -947,3 +969,15 @@ void PreviewFrame::OnFullScreen(wxCommandEvent & e)
     m_PreviewPanel->ForceUpdate();
 };
 
+void PreviewFrame::OnUndo(wxCommandEvent &e)
+{
+    wxCommandEvent dummy(wxEVT_COMMAND_MENU_SELECTED, XRCID("ID_EDITUNDO"));
+    m_parent->AddPendingEvent(dummy);
+};
+
+void PreviewFrame::OnRedo(wxCommandEvent &e)
+{
+    wxCommandEvent dummy(wxEVT_COMMAND_MENU_SELECTED, XRCID("ID_EDITREDO"));
+    m_parent->AddPendingEvent(dummy);
+};
+
