@@ -6,7 +6,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: huginApp.cpp 5025 2010-02-24 17:15:43Z brunopostle $
+ *  $Id$
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -40,6 +40,7 @@
 #include "hugin/AssistantPanel.h"
 #include "hugin/ImagesPanel.h"
 #include "hugin/CropPanel.h"
+#include "hugin/MaskEditorPanel.h"
 #include "hugin/CPEditorPanel.h"
 #include "hugin/OptimizePhotometricPanel.h"
 #include "hugin/PanoPanel.h"
@@ -53,6 +54,9 @@
 
 #include "base_wx/platform.h"
 #include "base_wx/huginConfig.h"
+#ifdef __WXMSW__
+#include "wx/dir.h"
+#endif
 
 
 #include <tiffio.h>
@@ -174,8 +178,6 @@ bool huginApp::OnInit()
 
     // here goes and comes configuration
     wxConfigBase * config = wxConfigBase::Get();
-    // update incompatible configuration entries.
-    updateHuginConfig(config);
     // do not record default values in the preferences file
     config->SetRecordDefaults(false);
 
@@ -222,6 +224,9 @@ bool huginApp::OnInit()
     wxXmlResource::Get()->AddHandler(new CPEditorPanelXmlHandler());
     wxXmlResource::Get()->AddHandler(new CPImageCtrlXmlHandler());
     wxXmlResource::Get()->AddHandler(new CPImagesComboBoxXmlHandler());
+    wxXmlResource::Get()->AddHandler(new MaskEditorPanelXmlHandler());
+    wxXmlResource::Get()->AddHandler(new ImagesListMaskXmlHandler());
+    wxXmlResource::Get()->AddHandler(new MaskImageCtrlXmlHandler());
     wxXmlResource::Get()->AddHandler(new OptimizePanelXmlHandler());
     wxXmlResource::Get()->AddHandler(new OptimizePhotometricPanelXmlHandler());
     wxXmlResource::Get()->AddHandler(new PanoPanelXmlHandler());
@@ -238,7 +243,6 @@ bool huginApp::OnInit()
     wxXmlResource::Get()->Load(m_xrcPrefix + wxT("pref_dialog.xrc"));
     wxXmlResource::Get()->Load(m_xrcPrefix + wxT("cpdetector_dialog.xrc"));
     wxXmlResource::Get()->Load(m_xrcPrefix + wxT("reset_dialog.xrc"));
-    wxXmlResource::Get()->Load(m_xrcPrefix + wxT("vig_corr_dlg.xrc"));
     wxXmlResource::Get()->Load(m_xrcPrefix + wxT("optimize_photo_panel.xrc"));
     wxXmlResource::Get()->Load(m_xrcPrefix + wxT("cp_editor_panel.xrc"));
     wxXmlResource::Get()->Load(m_xrcPrefix + wxT("images_panel.xrc"));
@@ -247,6 +251,7 @@ bool huginApp::OnInit()
     wxXmlResource::Get()->Load(m_xrcPrefix + wxT("main_frame.xrc"));
     wxXmlResource::Get()->Load(m_xrcPrefix + wxT("optimize_panel.xrc"));
     wxXmlResource::Get()->Load(m_xrcPrefix + wxT("pano_panel.xrc"));
+    wxXmlResource::Get()->Load(m_xrcPrefix + wxT("mask_editor_panel.xrc"));
 #endif
 
 #ifdef __WXMAC__
@@ -312,7 +317,7 @@ bool huginApp::OnInit()
 
     // set some suitable defaults
     PanoramaOptions opts = pano.getOptions();
-    opts.outputFormat = PanoramaOptions::TIFF;
+    opts.outputFormat = PanoramaOptions::TIFF_m;
     opts.blendMode = PanoramaOptions::ENBLEND_BLEND;
     opts.enblendOptions = config->Read(wxT("Enblend/Args"),wxT(HUGIN_ENBLEND_ARGS)).mb_str(wxConvLocal);
     opts.enfuseOptions = config->Read(wxT("Enfuse/Args"),wxT(HUGIN_ENFUSE_ARGS)).mb_str(wxConvLocal);
@@ -330,20 +335,35 @@ bool huginApp::OnInit()
             file.GetExt().CmpNoCase(wxT("pts")) == 0 ||
             file.GetExt().CmpNoCase(wxT("ptp")) == 0 )
         {
-            wxString filename(argv[1]);
-            if (! wxIsAbsolutePath(filename)) {
-                filename.Prepend(wxFileName::GetPathSeparator());
-                filename.Prepend(cwd);
-            }
+            if(file.IsRelative())
+                file.MakeAbsolute(cwd);
 	    // Loading the project file with set actualPath to its
 	    // parent directory.  (actualPath is used as starting
 	    // directory by many subsequent file selection dialogs.)
-            frame->LoadProjectFile(filename);
+            frame->LoadProjectFile(file.GetFullPath());
         } else {
             std::vector<std::string> filesv;
-            for (int i=1; i< argc; i++) {
-		bool actualPathSet = false;
+            for (int i=1; i< argc; i++) 
+            {
+                bool actualPathSet = false;
+#if defined __WXMSW__
+                //expand wildcards
+                wxFileName fileList(argv[i]);
+                if(fileList.IsRelative())
+                    fileList.MakeAbsolute(cwd);
+                wxDir dir;
+                wxString foundFile;
+                wxFileName file;
+                if(fileList.DirExists())
+                    if(dir.Open(fileList.GetPath()))
+                        if(dir.GetFirst(&foundFile,fileList.GetFullName(),wxDIR_FILES | wxDIR_HIDDEN))
+                            do
+                            {
+                                file=foundFile;
+                                file.MakeAbsolute(dir.GetName());
+#else
                 wxFileName file(argv[i]);
+#endif
                 if (file.GetExt().CmpNoCase(wxT("jpg")) == 0 ||
                     file.GetExt().CmpNoCase(wxT("jpeg")) == 0 ||
                     file.GetExt().CmpNoCase(wxT("tif")) == 0 ||
@@ -356,22 +376,20 @@ bool huginApp::OnInit()
                     file.GetExt().CmpNoCase(wxT("hdr")) == 0 ||
                     file.GetExt().CmpNoCase(wxT("viff")) == 0 )
                 {
-		    wxString filename(argv[i]);
-		    // Make sure the filename is absolute.
-		    if (! wxIsAbsolutePath(filename)) {
-			filename.Prepend(wxFileName::GetPathSeparator());
-			filename.Prepend(cwd);
-		    }
+                    if(file.IsRelative())
+                        file.MakeAbsolute(cwd);
+                    filesv.push_back((const char *)(file.GetFullPath().mb_str(HUGIN_CONV_FILENAME)));
 
-                    filesv.push_back((const char *)(filename.mb_str(HUGIN_CONV_FILENAME)));
-
-		    // Use the first filename to set actualPath.
-		    if (! actualPathSet) {
-			file = filename;
-			config->Write(wxT("/actualPath"), file.GetPath());
-			actualPathSet = true;
-		    }
+                    // Use the first filename to set actualPath.
+                    if (! actualPathSet)
+                    {
+                        config->Write(wxT("/actualPath"), file.GetPath());
+                        actualPathSet = true;
+                    }
                 }
+#if defined __WXMSW__
+                } while (dir.GetNext(&foundFile));
+#endif
             }
             GlobalCmdHist::getInstance().addCommand(
                     new PT::wxAddImagesCmd(pano,filesv)
