@@ -1,12 +1,12 @@
 // -*- c-basic-offset: 4 -*-
 
-/** @file PanorCommand.cpp
+/** @file wxPanoCommand.cpp
  *
  *  @brief implementation of some PanoCommands
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: wxPanoCommand.cpp 5035 2010-02-28 21:46:20Z brunopostle $
+ *  $Id$
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -36,6 +36,7 @@
 
 #include <vigra/cornerdetection.hxx>
 #include <vigra/localminmax.hxx>
+#include <panodata/StandardImageVariableGroups.h>
 
 
 
@@ -51,8 +52,8 @@ void wxAddCtrlPointGridCmd::execute()
 
     // TODO: rewrite, and use same function for modular image creation.
 #if 1
-    const PanoImage & i1 = pano.getImage(img1);
-    const PanoImage & i2 = pano.getImage(img1);
+    const SrcPanoImage & i1 = pano.getImage(img1);
+    const SrcPanoImage & i2 = pano.getImage(img1);
 
     // run both images through the harris corner detector
     ImageCache::EntryPtr eptr = ImageCache::getInstance().getSmallImage(i1.getFilename());
@@ -63,7 +64,7 @@ void wxAddCtrlPointGridCmd::execute()
     vigra::copyImage(srcImageRange(*(eptr->get8BitImage()), ga ),
                      destImage(leftImg));
 
-    double scale = i1.getWidth() / (double) leftImg.width();
+    double scale = i1.getSize().width() / (double) leftImg.width();
 
     //const vigra::BImage & leftImg = ImageCache::getInstance().getPyramidImage(
     //    i1.getFilename(),1);
@@ -167,10 +168,10 @@ void wxAddImagesCmd::execute()
     double focalLength = 0;
 
     Lens lens;
-    VariableMap vars;
-    fillVariableMap(vars);
     ImageOptions imgopts;
     SrcPanoImage srcImg;
+    HuginBase::StandardImageVariableGroups variable_groups(pano);
+    HuginBase::ImageVariableGroup & lenses = variable_groups.getLenses();
 
     // load additional images...
     for (it = files.begin(); it != files.end(); ++it) {
@@ -186,6 +187,8 @@ void wxAddImagesCmd::execute()
 
         // try to read settings automatically.
         srcImg.setFilename(filename);
+        srcImg.setWhiteBalanceRed(1);
+        srcImg.setWhiteBalanceBlue(1);
         bool ok = srcImg.readEXIF(focalLength, cropFactor, true, true);
         if (srcImg.getSize().x == 0 || srcImg.getSize().y == 0) {
             wxMessageBox(wxString::Format(_("Could not decode image:\n%s\nAbort"), fname.c_str()), _("Unsupported image file format"));
@@ -205,14 +208,13 @@ void wxAddImagesCmd::execute()
             std::cerr << "ERROR: caught exception: " << e.what() << std::endl;
             std::cerr << "Could not get pixel type for file " << filename << std::endl;
         };
-//#if 0
         if (! ok && assumeSimilar) {
                  // search for image with matching size and exif data
                  // and re-use it.
                 for (unsigned int i=0; i < pano.getNrOfImages(); i++) {
                     SrcPanoImage other = pano.getSrcImage(i);
                     double dummyfl=0;
-		    double dummycrop = 0; 
+                    double dummycrop = 0; 
                     other.readEXIF(dummyfl, dummycrop, false, false);
                     if ( other.getSize() == srcImg.getSize() 
                          &&
@@ -224,19 +226,23 @@ void wxAddImagesCmd::execute()
                         double ev = srcImg.getExposureValue();
                         srcImg = pano.getSrcImage(i);
                         srcImg.setFilename(filename);
-                        srcImg.setExposureValue(ev);
                         // add image
-                        PanoImage img(filename, srcImg.getSize().x, srcImg.getSize().y, other.getLensNr());
-                        srcImg.setLensNr(other.getLensNr());
-                        int imgNr = pano.addImage(img, vars);
+                        int imgNr = pano.addImage(srcImg);
+                        variable_groups.update();
+                        lenses.switchParts(imgNr, lenses.getPartNumber(i));
+                        lenses.unlinkVariableImage(HuginBase::ImageVariableGroup::IVE_ExposureValue, i);
+                        srcImg.setExposureValue(ev);
+                        lenses.unlinkVariableImage(HuginBase::ImageVariableGroup::IVE_WhiteBalanceRed, i);
+                        lenses.unlinkVariableImage(HuginBase::ImageVariableGroup::IVE_WhiteBalanceBlue, i);
+                        srcImg.setWhiteBalanceRed(1);
+                        srcImg.setWhiteBalanceBlue(1);
                         pano.setSrcImage(imgNr, srcImg);
                         added=true;
                         break;
                     }
                 }
-		if (added) continue;
+                if (added) continue;
         }
-//#endif
         int matchingLensNr=-1;
         // if no similar image found, ask user
         if (! ok) {
@@ -257,6 +263,9 @@ void wxAddImagesCmd::execute()
 
         // FIXME: check if the exif information
         // indicates this image matches a already used lens
+        variable_groups.update();
+        double ev = 0;
+        bool set_exposure = false;
         for (unsigned int i=0; i < pano.getNrOfImages(); i++) {
             SrcPanoImage other = pano.getSrcImage(i);
             // force reading of exif data, as it is currently not stored in the
@@ -268,10 +277,11 @@ void wxAddImagesCmd::execute()
                     && other.getExifFocalLength() == srcImg.getExifFocalLength()
                    )
                 {
-                    matchingLensNr = pano.getImage(i).getLensNr();
+                    matchingLensNr = lenses.getPartNumber(i);
                     // copy data from other image, just keep
                     // the file name and reload the exif data (for exposure)
-                    double ev = srcImg.getExposureValue();
+                    ev = srcImg.getExposureValue();
+                    set_exposure = true;
                     srcImg = pano.getSrcImage(i);
                     srcImg.setFilename(filename);
                     srcImg.setExposureValue(ev);
@@ -280,7 +290,7 @@ void wxAddImagesCmd::execute()
             } else if (assumeSimilar) {
                 // no exiv information, just check image size.
                 if (other.getSize() == srcImg.getSize() ) {
-                    matchingLensNr = pano.getImage(i).getLensNr();
+                    matchingLensNr = lenses.getPartNumber(i);
                     // copy data from other image, just keep
                     // the file name
                     srcImg = pano.getSrcImage(i);
@@ -289,23 +299,43 @@ void wxAddImagesCmd::execute()
                 }
             }
         }
-
-        if (matchingLensNr == -1) {
-            // create and add new lens
-            Lens lens;
-            lens.setImageSize(srcImg.getSize());
-            matchingLensNr = pano.addLens(lens);
+        
+        // If matchingLensNr == -1 still, we haven't found a good lens to use.
+        // We shouldn't attach the image to a lens in this case, it will have
+        // its own new lens.
+        int imgNr = pano.addImage(srcImg);
+        variable_groups.update();
+        if (matchingLensNr != -1)
+        {
+            lenses.switchParts(imgNr, matchingLensNr);
+            // unlink and set exposure value, if wanted.
+            if (set_exposure)
+            {
+                lenses.unlinkVariableImage(HuginBase::ImageVariableGroup::IVE_ExposureValue, imgNr);
+                lenses.unlinkVariableImage(HuginBase::ImageVariableGroup::IVE_WhiteBalanceRed, imgNr);
+                lenses.unlinkVariableImage(HuginBase::ImageVariableGroup::IVE_WhiteBalanceBlue, imgNr);
+                //don't link image size, this will foul the photometric optimizer
+                lenses.unlinkVariableImage(HuginBase::ImageVariableGroup::IVE_Size, imgNr);
+                /// @todo avoid copying the SrcPanoImage.
+                SrcPanoImage t = pano.getSrcImage(imgNr);
+                t.setExposureValue(ev);
+                pano.setSrcImage(imgNr, t);
+            }
         }
-
-        PanoImage img(filename, srcImg.getSize().x, srcImg.getSize().y, (unsigned int) matchingLensNr);
-        srcImg.setLensNr(matchingLensNr);
-        int imgNr = pano.addImage(img, vars);
-        pano.setSrcImage(imgNr, srcImg);
         if (imgNr == 0) {
             // get initial value for output exposure
             PanoramaOptions opts = pano.getOptions();
             opts.outputExposureValue = srcImg.getExposureValue();
             pano.setOptions(opts);
+            // set the exposure, but there isn't anything to link to so don't try unlinking.
+            // links are made by default when adding new images.
+            if (set_exposure)
+            {
+                /// @todo avoid copying the SrcPanoImage.
+                SrcPanoImage t = pano.getSrcImage(imgNr);
+                t.setExposureValue(ev);
+                pano.setSrcImage(imgNr, t);
+            }
         }
     }
     pano.changeFinished();
@@ -337,7 +367,10 @@ void wxLoadPTProjectCmd::execute()
         // stored in the file.
         opts.remapUsingGPU = wxConfigBase::Get()->Read(wxT("/Nona/UseGPU"),HUGIN_NONA_USEGPU) == 1;
         pano.setOptions(opts);
-
+        
+        HuginBase::StandardImageVariableGroups variableGroups(pano);
+        HuginBase::ImageVariableGroup & lenses = variableGroups.getLenses();
+        
         unsigned int nImg = pano.getNrOfImages();
         wxString basedir;
         double focalLength=0;
@@ -400,10 +433,8 @@ void wxLoadPTProjectCmd::execute()
                 srcImg.resize(imginfo.size());
             }
             // check if script contains invalid HFOV
-            unsigned lNr = pano.getImage(i).getLensNr();
-            Lens cLens = pano.getLens(lNr);
-            double hfov = const_map_get(pano.getVariables()[i], "v").getValue();
-            if (cLens.getProjection() == Lens::RECTILINEAR
+            double hfov = pano.getImage(i).getHFOV();
+            if (pano.getImage(i).getProjection() == HuginBase::SrcPanoImage::RECTILINEAR
                 && hfov >= 180 && autopanoSiftFile == false)
             {
                 autopanoSiftFile = true;
@@ -422,6 +453,15 @@ void wxLoadPTProjectCmd::execute()
             }
             pano.setSrcImage(i, srcImg);
         }
+        // Link image projection across each lens, since it is not saved.
+        for (unsigned int i = 0; i < lenses.getNumberOfParts(); i++)
+        {
+            // link the variables
+            lenses.linkVariablePart(
+                                HuginBase::ImageVariableGroup::IVE_Projection,
+                                i
+                                   );
+        }
     } else {
         DEBUG_ERROR("could not load panotools script");
     }
@@ -437,8 +477,8 @@ void wxLoadPTProjectCmd::execute()
             it != oldCPs.end(); ++it)
     {
         PT::ControlPoint point = *it;
-        SrcPanoImage img1 = pano.getSrcImage(point.image1Nr);
-        SrcPanoImage img2 = pano.getSrcImage(point.image2Nr);
+        const SrcPanoImage & img1 = pano.getImage(point.image1Nr);
+        const SrcPanoImage & img2 = pano.getImage(point.image2Nr);
         if (0 > point.x1 || point.x1 > img1.getSize().x || 
             0 > point.y1 || point.y1 > img1.getSize().y ||
             0 > point.x2 || point.x2 > img2.getSize().x || 
@@ -476,11 +516,6 @@ void wxApplyTemplateCmd::execute()
 
     if (pano.getNrOfImages() == 0) {
         // TODO: prompt for images!
-
-        // add a dummy lens
-        Lens dummyLens;
-        pano.addLens(dummyLens);
-
         wxString path = config->Read(wxT("actualPath"), wxT(""));
         wxFileDialog dlg(MainFrame::Get(), _("Add images"),
                 path, wxT(""),
@@ -529,15 +564,18 @@ void wxApplyTemplateCmd::execute()
                 case 5: config->Write(wxT("lastImageType"), wxT("exr")); break;
                 case 6: config->Write(wxT("lastImageType"), wxT("all files")); break;
             }
-
+            
+            HuginBase::StandardImageVariableGroups variable_groups(pano);
+            HuginBase::ImageVariableGroup & lenses = variable_groups.getLenses();
             // add images.
             for (unsigned int i=0; i< Pathnames.GetCount(); i++) {
                 std::string filename = (const char *)Pathnames[i].mb_str(HUGIN_CONV_FILENAME);
                 vigra::ImageImportInfo inf(filename.c_str());
-                PanoImage img(filename, inf.width(), inf.height(), 0);
-                VariableMap vars;
-                fillVariableMap(vars);
-                (void)pano.addImage(img, vars);
+                SrcPanoImage img(filename);
+                img.setSize(inf.size());
+                int imgNr = pano.addImage(img);
+                lenses.updatePartNumbers();
+                if (i > 0) lenses.switchParts(imgNr, 0);
             }
 
         }
@@ -563,7 +601,7 @@ void wxApplyTemplateCmd::execute()
         for (unsigned int i = 0; i < nNewImg; i++) {
 
             // check if image size is correct
-            SrcPanoImage oldSrcImg = pano.getSrcImage(i);
+            const SrcPanoImage & oldSrcImg = pano.getImage(i);
             SrcPanoImage newSrcImg = newPano.getSrcImage(i);
 
             // just keep the file name
@@ -585,7 +623,5 @@ void wxApplyTemplateCmd::execute()
     pano.changeFinished();
 }
 
-
-
 } // namespace
 
