@@ -6,7 +6,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: hugin_stitch_project.cpp 4314 2009-09-03 00:58:22Z yuv $
+ *  $Id$
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -280,9 +280,6 @@ bool stitchApp::OnInit()
     }
 #endif
     
-    // update incompatible configuration entries.
-    updateHuginConfig(wxConfigBase::Get());
-
     // set the name of locale recource to look for
     m_locale.AddCatalog(wxT("hugin"));
 
@@ -374,6 +371,13 @@ bool stitchApp::OnInit()
                          wxSAVE, wxDefaultPosition);
         dlg.SetDirectory(wxConfigBase::Get()->Read(wxT("/actualPath"),wxT("")));
         if (dlg.ShowModal() == wxID_OK) {
+            while(containsInvalidCharacters(dlg.GetPath()))
+            {
+                wxMessageBox(wxString::Format(_("The given filename contains one of the following invalid characters: %s\nHugin can not work with this filename. Please enter a valid filename."),getInvalidCharacters().c_str()),
+                    _("Error"),wxOK | wxICON_EXCLAMATION);
+                if(dlg.ShowModal()!=wxID_OK)
+                    return false;
+            };
             wxConfig::Get()->Write(wxT("/actualPath"), dlg.GetDirectory());  // remember for later
             outname = dlg.GetPath();
         } else { // bail
