@@ -29,6 +29,7 @@
 #include <panodata/PanoramaData.h>
 #include <hugin_utils/utils.h>
 #include <algorithms/nona/ComputeImageROI.h>
+#include <algorithms/basic/CalculateOverlap.h>
 
 #if defined MAC_SELF_CONTAINED_BUNDLE
  #define COULD_EXECUTE_EXIFTOOL_WITH_PERL
@@ -54,23 +55,18 @@ vector<UIntSet> getHDRStacks(const PanoramaData & pano, UIntSet allImgs)
 
     UIntSet stack;
 
+    HuginBase::CalculateImageOverlap overlap(&pano);
+    overlap.calculate(10);  // we are testing 10*10=100 points
     do {
         unsigned srcImg = *(allImgs.begin());
         stack.insert(srcImg);
         allImgs.erase(srcImg);
 
         // find all images that have a suitable overlap.
-        SrcPanoImage simg = pano.getSrcImage(srcImg);
-        // FIXME this should be a user preference
-        double maxShift = simg.getHFOV() / 10.0;
-        double minShift = 360.0 - maxShift;
         for (UIntSet::iterator it = allImgs.begin(); it !=  allImgs.end(); ) {
             unsigned srcImg2 = *it;
             it++;
-            SrcPanoImage simg2 = pano.getSrcImage(srcImg2);
-            if ( (fabs(simg.getYaw() - simg2.getYaw()) < maxShift
-                || fabs(simg.getYaw() - simg2.getYaw()) > minShift)
-                && fabs(simg.getPitch() - simg2.getPitch()) < maxShift  )
+            if(overlap.getOverlap(srcImg,srcImg2)>0.7)
             {
                 stack.insert(srcImg2);
                 allImgs.erase(srcImg2);
@@ -102,13 +98,13 @@ vector<UIntSet> getExposureLayers(const PanoramaData & pano, UIntSet allImgs)
         allImgs.erase(srcImg);
 
         // find all images that have a suitable overlap.
-        SrcPanoImage simg = pano.getSrcImage(srcImg);
+        const SrcPanoImage & simg = pano.getImage(srcImg);
         // FIXME this should be a user preference
         double maxEVDiff = 0.5;
         for (UIntSet::iterator it = allImgs.begin(); it !=  allImgs.end(); ) {
             unsigned srcImg2 = *it;
             it++;
-            SrcPanoImage simg2 = pano.getSrcImage(srcImg2);
+            const SrcPanoImage & simg2 = pano.getImage(srcImg2);
             if ( fabs(simg.getExposureValue() - simg2.getExposureValue()) < maxEVDiff )
             {
                 stack.insert(srcImg2);
@@ -138,6 +134,19 @@ UIntSet getImagesinROI (const PanoramaData& pano, const UIntSet activeImages)
     return images;
 }
 
+/* write make file line for checking, if progCommand can be runned
+   prints progName on command line */
+void checkProg(std::ostream& o,const std::string& progName, const std::string& progCommand)
+{
+#ifdef _WINDOWS
+    o << "\t@echo Checking " << progName << "..." << endl
+      << "\t" << progCommand << " > NUL 2>&1 && echo " << progName << " is ok || echo " 
+      << progName << " failed" << endl;
+#else
+    o << "\t@echo -n 'Checking " << progName << "...'" << endl
+      << "\t" << progCommand << " > /dev/null 2>&1 && echo '[OK]' || echo '[FAILED]'" << endl;
+#endif
+};
 
 void PanoramaMakefileExport::createMakefile(const PanoramaData& pano,
                                             const UIntSet& rimages,
@@ -158,12 +167,6 @@ void PanoramaMakefileExport::createMakefile(const PanoramaData& pano,
     setlocale(LC_NUMERIC,"C");
 #endif
 
-#ifdef UNIX_LIKE
- std::string NULL_DEVICE("/dev/null");
-#else // WINDOWS
- std::string NULL_DEVICE("NUL");
-#endif
-
     // output only images in current ROI
     UIntSet images = getImagesinROI(pano,rimages);
 
@@ -195,6 +198,12 @@ void PanoramaMakefileExport::createMakefile(const PanoramaData& pano,
 #endif
     }
 
+#ifdef _WINDOWS
+    o << "# Force using cmd.exe" << endl
+      << "SHELL=" << getenv("ComSpec") << endl <<endl;
+    
+#endif
+
     o << endl
       << endl
       << "# Tool configuration" << endl
@@ -208,7 +217,11 @@ void PanoramaMakefileExport::createMakefile(const PanoramaData& pano,
       << "ENFUSE=" << quoteStringShell(progs.enfuse) << endl
       << "SMARTBLEND=" << quoteStringShell(progs.smartblend) << endl
       << "HDRMERGE=" << quoteStringShell(progs.hdrmerge) << endl
+#ifdef _WINDOWS
+      << "RM=del" << endl
+#else
       << "RM=rm" << endl
+#endif
 #ifdef COULD_EXECUTE_EXIFTOOL_WITH_PERL
       << "EXIFTOOL=" << (executeWithPerl? perlCommand+" " : "") << quoteStringShell(progs.exiftool) << endl
 #else
@@ -543,7 +556,7 @@ void PanoramaMakefileExport::createMakefile(const PanoramaData& pano,
         o << expImgVar.str() << "_INPUT = ";
         double exposure=0;
         for (UIntSet::iterator it = similarExposures[i].begin(); it != similarExposures[i].end();) {
-            exposure += pano.getSrcImage(*it).getExposureValue();
+            exposure += pano.getImage(*it).getExposureValue();
             std::ostringstream fns;
             fns << output << "_exposure_layers_" << std::setfill('0') << std::setw(4) << *it << ldrRemappedExt;
             similarExposureRemappedImages.push_back(fns.str());
@@ -763,8 +776,7 @@ void PanoramaMakefileExport::createMakefile(const PanoramaData& pano,
         // test remapper
         switch(opts.remapper) {
             case PanoramaOptions::NONA:
-                o << "\t@echo -n 'Checking nona...'" << endl
-                  << "\t@-$(NONA) --help > " << NULL_DEVICE << " 2>&1 && echo '[OK]'" << endl;
+                checkProg(o,"nona","@-$(NONA) --help");
                 break;
             case PanoramaOptions::PTMENDER:
                 break;
@@ -772,27 +784,21 @@ void PanoramaMakefileExport::createMakefile(const PanoramaData& pano,
         // test blender
         switch(opts.blendMode) {
             case PanoramaOptions::ENBLEND_BLEND:
-                o << "\t@echo -n 'Checking enblend...'" << endl
-                  << "\t@-$(ENBLEND) -h > " << NULL_DEVICE << " 2>&1 && echo '[OK]'" << endl;
+                checkProg(o,"enblend","@-$(ENBLEND) -h");
                 break;
             case PanoramaOptions::PTBLENDER_BLEND:
-                o << "\t@echo -n 'Checking PTblender...'" << endl
-                  << "\t@-$(PTBLENDER) -h > " << NULL_DEVICE << " 2>&1 && echo '[OK]'" << endl;
+                checkProg(o,"PTblender","@-$(PTBLENDER) -h");
                 break;
             case PanoramaOptions::SMARTBLEND_BLEND:
-                o << "\t@echo -n 'Checking smartblend...'" << endl
-                  << "\t@-$(SMARTBLEND) > " << NULL_DEVICE << " 2>&1 && echo '[OK]'" << endl;
+                checkProg(o,"smartblend","@-$(SMARTBLEND)");
                 break;
         }
         // test enfuse
-        o << "\t@echo -n 'Checking enfuse...'" << endl
-          << "\t@-$(ENFUSE) -h > " << NULL_DEVICE << " 2>&1 && echo '[OK]'" << endl;
+        checkProg(o,"enfuse","@-$(ENFUSE) -h");
         // test hugin_hdrmerge
-        o << "\t@echo -n 'Checking hugin_hdrmerge...'" << endl
-          << "\t@-$(HDRMERGE) -h > " << NULL_DEVICE << " 2>&1 && echo '[OK]'" << endl;
+        checkProg(o,"hugin_hdrmerge","@-$(HDRMERGE) -h");
         // test exiftool
-        o << "\t@echo -n 'Checking exiftool...'" << endl
-          << "\t@-$(EXIFTOOL) -ver > " << NULL_DEVICE << " 2>&1 && echo '[OK]' || echo '[FAIL]'" << endl;
+        checkProg(o,"exiftool","@-$(EXIFTOOL) -ver");
         // test rm
 /* Needs to be replaced by a test that creates and deletes a file in the TEMP dir
         o << "\t@echo -n 'Checking rm...'" << endl
@@ -851,7 +857,7 @@ void PanoramaMakefileExport::createMakefile(const PanoramaData& pano,
                               << " $(PROJECT_FILE)" << endl << endl;
                             */
                             o << destImg << ": " << srcImg << " $(PROJECT_FILE)" << endl
-                              << "\t$(NONA) $(NONA_OPTS) $(NONA_LDR_REMAPPED_COMP) -r ldr -e " << pano.getSrcImage(*it).getExposureValue()
+                              << "\t$(NONA) $(NONA_OPTS) $(NONA_LDR_REMAPPED_COMP) -r ldr -e " << pano.getImage(*it).getExposureValue()
                               << " -m " << ldrRemappedMode << " -o $(LDR_EXPOSURE_REMAPPED_PREFIX_SHELL) -i " << *it
                               << " $(PROJECT_FILE_SHELL)" << endl << endl;
                             j++;
