@@ -3,7 +3,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: PanoramaOptions.h 5026 2010-02-24 23:39:00Z brunopostle $
+ *  $Id$
  *
  *  This is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -73,6 +73,7 @@ extern "C" {
 //
 //#include "PT/PanoImage.h"
 
+#include <hugin_shared.h>
 #include <vigra_ext/Interpolators.h>
 #include <panodata/DestPanoImage.h>
 
@@ -82,7 +83,7 @@ namespace HuginBase {
  *
  *  this holds the settings for the final panorama
  */
-class PanoramaOptions
+class IMPEX PanoramaOptions
 {
         
     public:
@@ -106,8 +107,9 @@ class PanoramaOptions
             ORTHOGRAPHIC = 14,
             EQUISOLID = 15,
             EQUI_PANINI = 16,
-			BIPLANE = 17,
-			TRIPLANE = 18
+            BIPLANE = 17,
+            TRIPLANE = 18,
+            GENERAL_PANINI = 19
         };
 
         /** PTStitcher acceleration */
@@ -293,6 +295,9 @@ class PanoramaOptions
          */
         void setProjection(ProjectionFormat f);
 
+        /** sets the optional parameters to their default values */
+        void resetProjectionParameters();
+
         ///
         PanoramaOptions::ProjectionFormat getProjection() const
             { return m_projectionFormat; };
@@ -324,10 +329,12 @@ class PanoramaOptions
         double getVFOV() const;
 
         /** get maximum possible hfov with current projection */
-        double getMaxHFOV() const;
+        double getMaxHFOV() const
+            { return m_projFeatures.maxHFOV; }
 
         /** get maximum possible vfov with current projection */
-        double getMaxVFOV() const;
+        double getMaxVFOV() const
+            { return m_projFeatures.maxVFOV; }
 
         ///
         DestPanoImage getDestImage() const;
@@ -404,6 +411,7 @@ class PanoramaOptions
         static const std::string fileformatNames[];
         static const std::string fileformatExt[];
         double m_hfov;
+        double m_vfov;
     //    unsigned int m_width;
     //    unsigned int m_height;
         ProjectionFormat m_projectionFormat;
