@@ -5,10 +5,11 @@
  *  @brief 
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
+ *          James Legg
  *
  * !! from PanoImage.h 1970
  *
- *  $Id: SrcPanoImage.cpp 4484 2009-09-25 01:53:37Z gpatters1 $
+ *  $Id$
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -45,6 +46,8 @@
 #define log2(x)        (log(x) / M_LN2)
 #endif /* __FreeBSD__ */
 
+#include "ImageVariableTranslate.h"
+
 namespace HuginBase {
 
 using namespace hugin_utils;
@@ -53,281 +56,255 @@ void SrcPanoImage::resize(const vigra::Size2D & sz)
 {
         // TODO: check if images have the same orientation.
         // calculate scaling ratio
-        double scale = (double) sz.x / m_size.x;
+        double scale = (double) sz.x / m_Size.getData().x;
         
         // center shift
-        m_centerShift *= scale;
-        m_shear *= scale;
+        m_RadialDistortionCenterShift.setData(m_RadialDistortionCenterShift.getData() * scale);
+        m_Shear.setData(m_Shear.getData() * scale);
         
         // crop
         // ensure the scaled rectangle is inside the new image size
-        switch (m_crop)
+        switch (m_CropMode.getData())
         {
             case NO_CROP:
-                m_cropRect = vigra::Rect2D(sz);
+                m_CropRect.setData(vigra::Rect2D(sz));
                 break;
             case CROP_RECTANGLE:
-                m_cropRect = m_cropRect * scale;
-                m_cropRect = m_cropRect & vigra::Rect2D(sz);
+                m_CropRect.setData(m_CropRect.getData() * scale);
+                m_CropRect.setData(m_CropRect.getData() & vigra::Rect2D(sz));
                 break;
             case CROP_CIRCLE:
-                m_cropRect = m_cropRect * scale;
+                m_CropRect.setData(m_CropRect.getData() * scale);
                 break;
         }
         
-        m_size = sz;
+        m_Size = sz;
         // vignetting correction
-        m_radialVigCorrCenterShift *=scale;
+        m_RadialVigCorrCenterShift.setData(m_RadialVigCorrCenterShift.getData() *scale);
+        // resize masks
+        MaskPolygonVector scaledMasks=m_Masks.getData();
+        for(unsigned int i=0;i<scaledMasks.size();i++)
+            scaledMasks[i].scale(scale);
+        m_Masks.setData(scaledMasks);
+        scaledMasks.clear();
+        scaledMasks=m_ActiveMasks.getData();
+        for(unsigned int i=0;i<scaledMasks.size();i++)
+            scaledMasks[i].scale(scale);
+        m_ActiveMasks.setData(scaledMasks);
 }
 
 bool SrcPanoImage::horizontalWarpNeeded()
 {
-    switch (m_proj)
+    switch (m_Projection.getData())
     {
         case PANORAMIC:
         case EQUIRECTANGULAR:
-            if (m_hfov == 360) return true;
+            if (m_HFOV.getData() == 360) return true;
         case FULL_FRAME_FISHEYE:
         case CIRCULAR_FISHEYE:
         case RECTILINEAR:
+        case FISHEYE_ORTHOGRAPHIC:
+        case FISHEYE_STEREOGRAPHIC:
+        case FISHEYE_EQUISOLID:
         default:
             break;
     }
     return false;
 }
 
-void SrcPanoImage::setDefaults()
+void BaseSrcPanoImage::setDefaults()
 {
-    m_proj = RECTILINEAR;
-    m_hfov = 50;
-    m_roll = 0;
-    m_pitch = 0;
-    m_yaw = 0;
+    /* Some of the vectors are difficult to initalise with the variables list
+     * header, so we make some local variables which are used in it.
+     */
+    // Radial Distortion defaults
+    std::vector<double> distortion_default(4, 0.0);
+    distortion_default[3] = 1;
     
-    m_responseType = RESPONSE_EMOR;
-    m_emorParams.resize(5);
-    for (unsigned i=0; i < 5; i++) {
-        m_emorParams[i] = 0;
-    }
-    m_exposure = 1;
-    m_wbRed = 1;
-    m_wbBlue = 1;
-    
-    m_gamma = 1;
-    
-    m_radialDist.resize(4);
-    m_radialDistRed.resize(4);
-    m_radialDistBlue.resize(4);
-    for (unsigned i=0; i < 3; i++) {
-        m_radialDist[i] = 0;
-        m_radialDistRed[i] = 0;
-        m_radialDistBlue[i] = 0;
-    }
-    m_radialDist[3] = 1;
-    m_radialDistRed[3] = 1;
-    m_radialDistBlue[3] = 1;
-    m_centerShift.x = 0;
-    m_centerShift.y = 0;
-    m_shear.x = 0;
-    m_shear.y = 0;
-    
-    m_crop = NO_CROP;
-    
-    m_vigCorrMode = VIGCORR_RADIAL|VIGCORR_DIV;
-    m_radialVigCorrCoeff.resize(4);
-    m_radialVigCorrCoeff[0] = 1;
-    for (unsigned i=1; i < 4; i++) {
-        m_radialVigCorrCoeff[i] = 0;
-    }
-
-    m_exifCropFactor = 0;
-    m_exifFocalLength = 0;
-    m_exifFocalLength35 = 0;
-    m_exifOrientation = 0;
-    m_exifAperture = 0;
-    m_exifDistance = 0;
-    m_exifISO = 0;
-
-    m_lensNr = 0;
-    m_featherWidth = 10;
-    m_morph = false;
+    std::vector<double> RadialVigCorrCoeff_default(4, 0.0);
+    RadialVigCorrCoeff_default[0] = 1;
+    HuginBase::MaskPolygonVector defaultMaskVector;
+#define image_variable( name, type, default_value ) m_##name.setData(default_value);
+#include "image_variables.h"
+#undef image_variable
 }
 
-
-bool SrcPanoImage::isInside(vigra::Point2D p) const
+bool SrcPanoImage::isInside(vigra::Point2D p, bool ignoreMasks) const
 {
-    switch(m_crop) {
+    bool insideCrop=false;
+    switch(m_CropMode.getData()) {
         case NO_CROP:
         case CROP_RECTANGLE:
-            return m_cropRect.contains(p);
+            insideCrop = m_CropRect.getData().contains(p);
+            break;
         case CROP_CIRCLE:
         {
-            if (0 > p.x || 0 > p.y || p.x >= m_size.x || p.y >= m_size.y) {
+            if (0 > p.x || 0 > p.y || p.x >= m_Size.getData().x || p.y >= m_Size.getData().y) {
                 // outside image
                 return false;
             }
             FDiff2D cropCenter;
-            cropCenter.x = m_cropRect.left() + m_cropRect.width()/2.0;
-            cropCenter.y = m_cropRect.top() + m_cropRect.height()/2.0;
-            double radius2 = std::min(m_cropRect.width()/2.0, m_cropRect.height()/2.0);
+            cropCenter.x = m_CropRect.getData().left() + m_CropRect.getData().width()/2.0;
+            cropCenter.y = m_CropRect.getData().top() + m_CropRect.getData().height()/2.0;
+            double radius2 = std::min(m_CropRect.getData().width()/2.0, m_CropRect.getData().height()/2.0);
             radius2 = radius2 * radius2;
             FDiff2D pf = FDiff2D(p) - cropCenter;
-            return (radius2 > pf.x*pf.x+pf.y*pf.y );
+            insideCrop = (radius2 > pf.x*pf.x+pf.y*pf.y );
         }
     }
-    // this should never be reached..
-    return false;
+    if(insideCrop && !ignoreMasks)
+        return !(isInsideMasks(p));
+    else
+        return insideCrop;
 }
 
     
 bool SrcPanoImage::getCorrectTCA() const
 { 
-    bool nr = (m_radialDistRed[0] == 0.0 && m_radialDistRed[1] == 0.0 &&
-               m_radialDistRed[2] == 0.0 && m_radialDistRed[3] == 1);
-    bool nb = (m_radialDistBlue[0] == 0.0 && m_radialDistBlue[1] == 0.0 &&
-               m_radialDistBlue[2] == 0.0 && m_radialDistBlue[3] == 1);
+    bool nr = (m_RadialDistortionRed.getData()[0] == 0.0 && m_RadialDistortionRed.getData()[1] == 0.0 &&
+               m_RadialDistortionRed.getData()[2] == 0.0 && m_RadialDistortionRed.getData()[3] == 1);
+    bool nb = (m_RadialDistortionBlue.getData()[0] == 0.0 && m_RadialDistortionBlue.getData()[1] == 0.0 &&
+               m_RadialDistortionBlue.getData()[2] == 0.0 && m_RadialDistortionBlue.getData()[3] == 1);
     return !(nr && nb);
 }
 
 
 FDiff2D SrcPanoImage::getRadialDistortionCenter() const
-{ return FDiff2D(m_size)/2.0 + m_centerShift; }
+{ return FDiff2D(m_Size.getData())/2.0 + m_RadialDistortionCenterShift.getData(); }
 
 
 FDiff2D SrcPanoImage::getRadialVigCorrCenter() const
-{ return (FDiff2D(m_size)-FDiff2D(1,1))/2.0 + m_radialVigCorrCenterShift; }
+{ return (FDiff2D(m_Size.getData())-FDiff2D(1,1))/2.0 + m_RadialVigCorrCenterShift.getData(); }
 
 void SrcPanoImage::setCropMode(CropMode val)
 {
-    m_crop = val;
-    if (m_crop == NO_CROP) {
-        m_cropRect = vigra::Rect2D(m_size);
+    m_CropMode.setData(val);
+    if (val == NO_CROP) {
+        m_CropRect.setData(vigra::Rect2D(m_Size.getData()));
+    }
+}
+
+void SrcPanoImage::setSize(vigra::Size2D val)
+{
+    m_Size.setData(val);
+    if (m_CropMode.getData() == NO_CROP) {
+        m_CropRect.setData(vigra::Rect2D(val));
     }
 }
 
 double SrcPanoImage::getExposure() const
-{ return 1.0/pow(2.0, m_exposure); }
+{ return 1.0/pow(2.0, m_ExposureValue.getData()); }
 
 void SrcPanoImage::setExposure(const double & val)
-{ m_exposure = log2(1/val); }
+{ m_ExposureValue = log2(1/val); }
 
 
-bool SrcPanoImage::operator==(const SrcPanoImage & other) const
+bool BaseSrcPanoImage::operator==(const BaseSrcPanoImage & other) const
 {
-    //        return true;
-    return ( m_proj == other.m_proj &&
-             m_hfov == other.m_hfov &&
-             m_roll  == other.m_roll  &&
-             m_pitch == other.m_pitch  &&
-             m_yaw == other.m_yaw &&
-             
-             m_responseType == other.m_responseType &&
-             m_emorParams == other.m_emorParams &&
-             m_exposure == other.m_exposure &&
-             m_gamma == m_gamma &&
-             m_wbRed == m_wbRed &&
-             m_wbBlue == m_wbBlue &&
-             
-             m_radialDist == other.m_radialDist  &&
-             m_radialDistRed == other.m_radialDistRed  &&
-             m_radialDistBlue == other.m_radialDistBlue  &&
-             m_centerShift == other.m_centerShift  &&
-             m_shear == other.m_shear  &&
-             
-             m_crop == other.m_crop  &&
-             m_cropRect == other.m_cropRect &&
-             
-             m_vigCorrMode == other.m_vigCorrMode  &&
-             m_radialVigCorrCoeff == other.m_radialVigCorrCoeff &&
-             
-             m_ka == other.m_ka  &&
-             m_kb == other.m_kb  &&
-             
-             m_exifModel == other.m_exifModel &&
-             m_exifMake == other.m_exifMake &&
-             m_exifCropFactor == other.m_exifCropFactor &&
-             m_exifFocalLength == other.m_exifFocalLength &&
-             m_exifFocalLength35 == other.m_exifFocalLength35 &&
-             m_exifOrientation == other.m_exifOrientation &&
-             m_exifAperture == other.m_exifAperture &&
-             m_exifISO == other.m_exifISO &&
-             m_exifDistance == other.m_exifDistance &&
-
-             m_lensNr == other.m_lensNr  &&
-             m_featherWidth == other.m_featherWidth  &&
-             m_morph == other.m_morph);
+    DEBUG_TRACE("");
+    return (
+#define image_variable( name, type, default_value ) \
+    m_##name.getData() == other.m_##name.getData() &&
+#include "image_variables.h"
+#undef image_variable
+    true // All the variable checks above end with && so we need this.
+    );
 }
 
 // convinience functions to extract a set of variables
-double SrcPanoImage::getVar(const std::string & name) const
+double SrcPanoImage::getVar(const std::string & code) const
 {
-    assert(name.size() > 0);
-    // TODO: support all variables
-    if (name == "Eev") 
-        return m_exposure;
-    else if (name == "Er")
-        return m_wbRed;
-    else if (name == "Eb")
-        return m_wbBlue;
-    else if (name == "Ra")
-        return m_emorParams[0];
-    else if (name[0] == 'R')
-    {
-        assert(name.size() == 2);
-        int i = name[1] - 'a';
-        return m_emorParams[i];
-    } else if (name[0] == 'V')
-    {
-        int i = name[1] - 'a';
-        if (i > 0 && i < 4) {
-            return m_radialVigCorrCoeff[i];
-        } else {
-            if (name[1] == 'x') {
-                return m_radialVigCorrCenterShift.x;
-            } else if (name[1] == 'y') {
-                return m_radialVigCorrCenterShift.y;
-            }
-        }
-    } else {
-        assert(0 || "Unknown variable in getVar()");
+    DEBUG_TRACE("");
+    assert(code.size() > 0);
+#define image_variable( name, type, default_value ) \
+    if (PTOVariableConverterFor##name::checkApplicability(code)) \
+        return PTOVariableConverterFor##name::getValueFromVariable(code, m_##name );\
+    else 
+#include "image_variables.h"
+#undef image_variable
+    {// this is for the final else.
+        DEBUG_ERROR("Unknown variable " << code);
     }
     return 0;
 }
 
-void SrcPanoImage::setVar(const std::string & name, double val)
+void SrcPanoImage::setVar(const std::string & code, double val)
 {
-    assert(name.size() > 0);
-    // TODO: support all variables
-    if (name == "Eev") 
-        m_exposure = val;
-    else if (name == "Er")
-        m_wbRed = val;
-    else if (name == "Eb")
-        m_wbBlue = val;
-    else if (name[0] == 'R')
-    {
-        assert(name.size() == 2);
-        int i = name[1] - 'a';
-        m_emorParams[i] = val;
-    } else if (name[0] == 'V')
-    {
-        int i = name[1] - 'a';
-        if (i >= 0 && i < 4) {
-            m_radialVigCorrCoeff[i] = val;
-        } else {
-            if (name[1] == 'x') {
-                m_radialVigCorrCenterShift.x = val;
-            } else if (name[1] == 'y') {
-                m_radialVigCorrCenterShift.y = val;
-            } else {
-                DEBUG_ERROR("Unknown variable " << name);
-            }
-        }
-    } else {
-        DEBUG_ERROR("Unknown variable " << name);
+    DEBUG_TRACE("Var:" << code << " value: " << val);
+    assert(code.size() > 0);
+#define image_variable( name, type, default_value ) \
+    if (PTOVariableConverterFor##name::checkApplicability(code)) \
+        {PTOVariableConverterFor##name::setValueFromVariable(code, m_##name, val);}\
+    else 
+#include "image_variables.h"
+#undef image_variable
+    {// this is for the final else.
+        DEBUG_ERROR("Unknown variable " << code);
     }
 }
 
+VariableMap SrcPanoImage::getVariableMap() const
+{
+    // make a variable map vector
+    
+    // fill variable map with details about this image.
+    // position
+    DEBUG_TRACE("");
+
+    VariableMap vars;
+#define image_variable( name, type, default_value ) \
+    PTOVariableConverterFor##name::addToVariableMap(m_##name, vars);
+#include "image_variables.h"
+#undef image_variable
+
+    return vars;
+}
+
+
+ImageOptions SrcPanoImage::getOptions() const
+{
+    // convert stuff to ImageOptions object for old stuff not using the new interface.
+    ImageOptions opts;
+    opts.featherWidth = m_FeatherWidth.getData();
+    opts.morph = m_Morph.getData();
+    opts.docrop = m_CropMode.getData() != NO_CROP;
+    opts.autoCenterCrop = m_AutoCenterCrop.getData();
+    opts.cropRect = m_CropRect.getData();
+    opts.m_vigCorrMode = m_VigCorrMode.getData();
+    opts.m_flatfield = m_FlatfieldFilename.getData();
+    opts.responseType = m_ResponseType.getData();
+    opts.active = m_Active.getData();
+    
+    return opts;
+}
+
+void SrcPanoImage::setOptions(const ImageOptions & opts)
+{
+    // set variables from an ImageOptions object for the stuff using the old interface.
+    m_FeatherWidth.setData(opts.featherWidth);
+    m_Morph.setData(opts.morph);
+    if (opts.docrop)
+    {
+        if (m_Projection.getData() == CIRCULAR_FISHEYE)
+        {
+            m_CropMode.setData(CROP_CIRCLE);
+        }
+        else
+        {
+            m_CropMode.setData(CROP_RECTANGLE);
+        }
+        m_CropRect.setData(opts.cropRect);
+    } else {
+        m_CropMode.setData(NO_CROP);
+        m_CropRect.setData((vigra::Rect2D)m_Size.getData());
+    }
+    m_AutoCenterCrop.setData(opts.autoCenterCrop);    
+    m_VigCorrMode.setData(opts.m_vigCorrMode);
+    m_FlatfieldFilename.setData(opts.m_flatfield);
+    m_ResponseType.setData((ResponseType)opts.responseType);
+    m_Active.setData(opts.active);
+}
+
 
 bool SrcPanoImage::readEXIF(double & focalLength, double & cropFactor, bool applyEXIFValues, bool applyExposureValue)
 {
@@ -417,7 +394,7 @@ bool SrcPanoImage::readEXIF(double & focalLength, double & cropFactor, double &
     }
 
     long orientation = 0;
-    if (getExiv2Value(exifData,"Exif.Image.Orientation",orientation)) {
+    if (getExiv2Value(exifData,"Exif.Image.Orientation",orientation) && trustExivOrientation()) {
         switch (orientation) {
             case 3:  // rotate 180
                 roll = 180;
@@ -610,7 +587,6 @@ bool SrcPanoImage::readEXIF(double & focalLength, double & cropFactor, double &
     // store some important EXIF tags for later usage.
     setExifFocalLength(focalLength);
     setExifFocalLength35(eFocalLength35);
-    setExifCropFactor(cropFactor);
     setExifOrientation(roll);
     setExifAperture(photoFNumber);
     setExifISO(isoSpeed);
@@ -628,6 +604,10 @@ bool SrcPanoImage::readEXIF(double & focalLength, double & cropFactor, double &
         setRoll(roll);
         if (applyExposureValue)
             setExposureValue(eV);
+        if(cropFactor>0)
+        {
+            setExifCropFactor(cropFactor);
+        };
         if (focalLength > 0 && cropFactor > 0) {
             setHFOV(calcHFOV(getProjection(), focalLength, cropFactor, getSize()));
             DEBUG_DEBUG("HFOV:         " << getHFOV());
@@ -659,6 +639,9 @@ double SrcPanoImage::calcHFOV(SrcPanoImage::Projection proj, double fl, double c
             break;
         case SrcPanoImage::CIRCULAR_FISHEYE:
         case SrcPanoImage::FULL_FRAME_FISHEYE:
+        case SrcPanoImage::FISHEYE_ORTHOGRAPHIC:
+        case SrcPanoImage::FISHEYE_STEREOGRAPHIC:
+        case SrcPanoImage::FISHEYE_EQUISOLID:
             hfov = sensorSize.x / fl * 180/M_PI;
             break;
         case SrcPanoImage::EQUIRECTANGULAR:
@@ -692,6 +675,9 @@ double SrcPanoImage::calcFocalLength(SrcPanoImage::Projection proj, double hfov,
             break;
         case SrcPanoImage::CIRCULAR_FISHEYE:
         case SrcPanoImage::FULL_FRAME_FISHEYE:
+        case SrcPanoImage::FISHEYE_ORTHOGRAPHIC:
+        case SrcPanoImage::FISHEYE_STEREOGRAPHIC:
+        case SrcPanoImage::FISHEYE_EQUISOLID:
             // same projection equation for both fisheye types,
             // assume equal area projection.
             return sensorSize.x / (hfov/180*M_PI);
@@ -721,6 +707,9 @@ double SrcPanoImage::calcCropFactor(SrcPanoImage::Projection proj, double hfov,
         case SrcPanoImage::CIRCULAR_FISHEYE:
         case SrcPanoImage::FULL_FRAME_FISHEYE:
         case SrcPanoImage::EQUIRECTANGULAR:
+        case SrcPanoImage::FISHEYE_ORTHOGRAPHIC:
+        case SrcPanoImage::FISHEYE_STEREOGRAPHIC:
+        case SrcPanoImage::FISHEYE_EQUISOLID:
         case SrcPanoImage::PANORAMIC:
             // same projection equation for both fisheye types,
             // assume equal area projection.
@@ -736,6 +725,24 @@ double SrcPanoImage::calcCropFactor(SrcPanoImage::Projection proj, double hfov,
     return sqrt(36.0*36.0 + 24.0*24.0) / diag;
 }
 
+void SrcPanoImage::updateFocalLength(double newFocalLength)
+{
+    double newHFOV=calcHFOV(getProjection(),newFocalLength,getExifCropFactor(),getSize());
+    if(newHFOV!=0)
+    {
+        setHFOV(newHFOV);
+    };
+};
+
+void SrcPanoImage::updateCropFactor(double focalLength, double newCropFactor)
+{
+    double newHFOV=calcHFOV(getProjection(),focalLength,newCropFactor,getSize());
+    if(newHFOV!=0)
+    {
+        setHFOV(newHFOV);
+    };
+    setExifCropFactor(newCropFactor);
+};
 
 // Convenience functions to work with Exiv2
 bool SrcPanoImage::getExiv2Value(Exiv2::ExifData& exifData, std::string keyName, long & value)
@@ -778,5 +785,107 @@ bool SrcPanoImage::getExiv2Value(Exiv2::ExifData& exifData, std::string keyName,
         return false;
     }
 }
-    
+
+// mask handling stuff
+void SrcPanoImage::addMask(MaskPolygon newMask)
+{
+    MaskPolygonVector newMasks=m_Masks.getData();
+    newMasks.push_back(newMask);
+    setMasks(newMasks);
+};
+
+void SrcPanoImage::addActiveMask(MaskPolygon newMask)
+{
+    MaskPolygonVector newMasks=m_ActiveMasks.getData();
+    newMasks.push_back(newMask);
+    setActiveMasks(newMasks);
+};
+
+void SrcPanoImage::clearActiveMasks()
+{
+    MaskPolygonVector emptyMaskVector;
+    m_ActiveMasks.setData(emptyMaskVector);
+};
+
+bool SrcPanoImage::hasMasks() const
+{
+    return m_Masks.getData().size()>0;
+};
+
+bool SrcPanoImage::hasPositiveMasks() const
+{
+    MaskPolygonVector masks=m_Masks.getData();
+    if(masks.size()>0)
+    {
+        for(unsigned int i=0;i<masks.size();i++)
+        {
+            if(masks[i].getMaskType()==MaskPolygon::Mask_positive)
+            {
+                return true;
+            };
+        };
+    };
+    return false;
+};
+
+bool SrcPanoImage::hasActiveMasks() const
+{
+    return m_ActiveMasks.getData().size()>0;
+};
+ 
+void SrcPanoImage::printMaskLines(std::ostream &o, unsigned int newImgNr) const
+{
+    if(m_Masks.getData().size()>0)
+        for(unsigned int i=0;i<m_Masks.getData().size();i++)
+            m_Masks.getData()[i].printPolygonLine(o, newImgNr);
+};
+
+void SrcPanoImage::changeMaskType(unsigned int index, HuginBase::MaskPolygon::MaskType newType)
+{
+    if(index<m_Masks.getData().size())
+    {
+        MaskPolygonVector editedMasks=m_Masks.getData();
+        editedMasks[index].setMaskType(newType);
+        m_Masks.setData(editedMasks);
+    };
+};
+
+void SrcPanoImage::deleteMask(unsigned int index)
+{
+    if(index<m_Masks.getData().size())
+    {
+        MaskPolygonVector oldMasks=m_Masks.getData();
+        oldMasks.erase(oldMasks.begin()+index);
+        m_Masks.setData(oldMasks);
+    };
+};
+
+bool SrcPanoImage::isInsideMasks(vigra::Point2D p) const
+{
+    if(!hasActiveMasks())
+        return false;
+    bool insideMask=false;
+    unsigned int i=0;
+    while(!insideMask && i<m_ActiveMasks.getData().size())
+    {
+        insideMask=m_ActiveMasks.getData()[i].isInside(p);
+        i++;
+    };
+    return insideMask;
+};
+
+/**
+ * Decides if the Exiv Orientation Tag of an images is plausible.
+ * Current checks:
+ * - If width is smaller than height, image is probably already rotated, tag may be wrong.
+ * @return true if plausible.
+ */
+bool SrcPanoImage::trustExivOrientation()
+{
+    if(getSize().width() < getSize().height())
+        return false;
+
+    return true;
+}
+
 } // namespace
