@@ -6,7 +6,7 @@
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
- *  $Id: ImageTransforms.h 4431 2009-09-15 20:16:44Z harryvanderwolf $
+ *  $Id$
  *
  *  This is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -42,9 +42,7 @@
 #include "MultiThreadOperations.h"
 #include <boost/thread/thread.hpp>
 #include <boost/bind.hpp>
-#ifdef __APPLE__
-    #include <boost/function.hpp>
-#endif
+#include <boost/function.hpp>
 
 
 namespace vigra_ext
