@@ -5,7 +5,7 @@
  *
  *  @author Andrew Mihal
  *
- *  $Id: ImageTransformsGPU.cpp 4307 2009-09-01 05:42:14Z tmodes $
+ *  $Id$
  *
  *  This is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -26,7 +26,9 @@
 #include <iostream>
 #include <iomanip>
 
+#if !defined Hugin_shared || !defined _WINDOWS
 #define GLEW_STATIC
+#endif
 #include <GL/glew.h>
 #ifdef __APPLE__
   #include <GLUT/glut.h>
@@ -36,7 +38,10 @@
 
 
 #include <string.h>
-
+#ifdef _WIN32
+// weird errors in accessor.hxx if DIFFERENCE is defined
+  #undef DIFFERENCE
+#endif
 #include <vigra/diff2d.hxx>
 #include <vigra/utilities.hxx>
 #include <vigra/error.hxx>
