@@ -6,6 +6,11 @@ BUGADDR="http://sourceforge.net/tracker/?group_id=77506&atid=550441" # MSGID-Bug
 COPYRIGHT="Pablo dAngelo"
 WDIR=`pwd` # working dir
 
+# safety checks
+which wxrc &> /dev/null || { echo "Error: wxrc utility not found"; exit 0; }
+which xgettext &> /dev/null || { echo "Error: xgettext utility not found"; exit 0; }
+which msgmerge &> /dev/null || { echo "Error: msgmerge utility not found"; exit 0; }
+
 echo "Preparing rc files"
 
 cd ${BASEDIR}
@@ -14,14 +19,14 @@ cd ${WDIR}
 echo "Done preparing rc files"
        
 echo "Filtering out ignored strings"
-./filter-ignored-strings.py
-mv -f xrc-filtered.cpp xrc.cpp
+./filter-ignored-strings.py xrc.cpp
 echo "Done filtering"
        
 echo "Extracting messages"
 cd ${BASEDIR}
 # we use simple sorting to make sure the lines do not jump around too much from system to system
-find . -name '*.cpp' -o -name '*.h' -o -name '*.c' | sort > ${WDIR}/infiles.list
+find . -name '*.cpp' -o -name '*.h' -o -name '*.c' \
+    | sort > ${WDIR}/infiles.list
 echo "xrc.cpp" >> ${WDIR}/infiles.list
 cat $WDIR/POTFILES.in >> ${WDIR}/infiles.list
 
